//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================



using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using GuidanceExplorer.Model;
using System.Drawing;

namespace GuidanceExplorer.Controls
{
    /// <summary>
    /// Represents the Create View node in the treeview
    /// </summary>
    [MenuItemAttribute(10, "&File\\New Item", null, Keys.None)]
    [MenuItemAttribute(40, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(50, "&File\\Import Item...", GuidanceExplorer.Commands.ImportGuidanceItemToLibrary, Keys.None)]
    [MenuItemAttribute(60, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(70, "&File\\Export Item as Xml...", GuidanceExplorer.Commands.SaveItemAsXml, Keys.None)]
    [MenuItemAttribute(80, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(90, "&File\\&Save as...", GuidanceExplorer.Commands.SaveViewAs, Keys.None)]
    [MenuItemAttribute(100, "&File\\Save Item &as...", GuidanceExplorer.Commands.SaveItemAs, Keys.None)]
    [MenuItemAttribute(110, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(112, "&File\\Suscribe to &Library\\From &File...", GuidanceExplorer.Commands.SuscribeLibraryFromFile, Keys.None)]
    [MenuItemAttribute(114, "&File\\Suscribe to &Library\\From &Web...", GuidanceExplorer.Commands.SuscribeLibraryFromWeb, Keys.None)]
    [MenuItemAttribute(115, "&File\\Suscribe to &Library\\From &Rss...", GuidanceExplorer.Commands.SuscribeLibraryFromRss, Keys.None)]
    [MenuItemAttribute(116, "&File\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(120, "&File\\&Exit", GuidanceExplorer.Commands.FileExit, Keys.None)]

    [MenuItemAttribute(200, "&Edit\\&Undo drag/drop", GuidanceExplorer.Commands.UndoDragAndDrop, Keys.Control | Keys.Z)]
    [MenuItemAttribute(210, "&Edit\\-", null, Keys.None)]
    [MenuItemAttribute(220, "&Edit\\Cut item", GuidanceExplorer.Commands.CutGuidanceItem, Keys.None)]
    [MenuItemAttribute(230, "&Edit\\Copy item", GuidanceExplorer.Commands.CopyGuidanceItem, Keys.None)]
    [MenuItemAttribute(240, "&Edit\\Paste item", GuidanceExplorer.Commands.PasteGuidanceItem, Keys.None)]
    [MenuItemAttribute(250, "&Edit\\Dele&te item", GuidanceExplorer.Commands.DeleteGuidanceItem, Keys.None)]
    [MenuItemAttribute(260, "&Edit\\-", null, Keys.None)]
    [MenuItemAttribute(270, "&Edit\\&Select All", GuidanceExplorer.Commands.SelectAll, Keys.Control | Keys.A)]

    [MenuItemAttribute(300, "&View\\Reset &Sort Order", GuidanceExplorer.Commands.ResetSortOrder, Keys.None)]
    [MenuItemAttribute(310, "&View\\&Reset Columns", GuidanceExplorer.Commands.ResetColumns, Keys.None)]
    [MenuItemAttribute(320, "&View\\Reset &Filters", GuidanceExplorer.Commands.ResetFilters, Keys.None)]
    [MenuItemAttribute(330, "&View\\&Column Chooser", GuidanceExplorer.Commands.ColumnChooser, Keys.None)]

    [MenuItemAttribute(400, "&Help\\&About", GuidanceExplorer.Commands.HelpAbout, Keys.None)]

    [MenuItemAttribute(500, "context\\New Item", null, Keys.None)]
    [MenuItemAttribute(540, "context\\-", null, Keys.None)]
    [MenuItemAttribute(550, "context\\Import Item...", GuidanceExplorer.Commands.ImportGuidanceItemToLibrary, Keys.None)]
    [MenuItemAttribute(560, "context\\Save as...", GuidanceExplorer.Commands.SaveViewAs, Keys.None)]
    [MenuItemAttribute(570, "context\\-", null, Keys.None)]
    [MenuItemAttribute(580, "context\\Edit View", GuidanceExplorer.Commands.RenameView, Keys.None)]
    public class CreateViewNode : TreeNode
    {
        /// <summary>
        /// Factory.
        /// </summary>
        /// <returns></returns>
        public static CreateViewNode New()
        {
            return new CreateViewNode();
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        private CreateViewNode() : base("Create View", 5, 5) 
        {
            Font italic = new Font(FontFamily.GenericSansSerif, 8.25f, FontStyle.Italic);
            this.NodeFont = italic;
        }
        
    }
}

