//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using GuidanceExplorer.Model;

namespace GuidanceExplorer.Controls
{
    /// <summary>
    /// Represent the root node where all the libraries will be shown.
    /// </summary>
    [MenuItemAttribute(10, "&File\\&New Item", GuidanceExplorer.Commands.NewGuidanceItem, typeof(ModelItemMenuItemIsEditableCondition), typeof(NewItemMenuItemProvider), Keys.None)]
    [MenuItemAttribute(15, "&File\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(20, "&File\\Save &Item as Xml...", "Save Items as Xml...", GuidanceExplorer.Commands.SaveItemAsXml, Keys.None)]
    [MenuItemAttribute(25, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(30, "&File\\&Save Library as...", GuidanceExplorer.Commands.SaveViewAs, Keys.None)]
    [MenuItemAttribute(40, "&File\\Save Item &as...", "Save Items as...", GuidanceExplorer.Commands.SaveItemAs, Keys.None)]
    [MenuItemAttribute(45, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(50, "&File\\Suscribe to &Library\\From &File...", GuidanceExplorer.Commands.SuscribeLibraryFromFile, Keys.None)]
    [MenuItemAttribute(60, "&File\\Suscribe to &Library\\From &Web...", GuidanceExplorer.Commands.SuscribeLibraryFromWeb, Keys.None)]
    [MenuItemAttribute(61, "&File\\Suscribe to &Library\\From &Rss...", GuidanceExplorer.Commands.SuscribeLibraryFromRss, Keys.None)]
    [MenuItemAttribute(62, "&File\\S&ynchronize Libraries", Commands.Synchronize, Keys.None)]
    [MenuItemAttribute(70, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(78, "&File\\Add/Remove &Guidance Type...", GuidanceExplorer.Commands.AddRemoveGuidanceType, Keys.None)]
    [MenuItemAttribute(79, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(80, "&File\\&Exit", GuidanceExplorer.Commands.FileExit, Keys.None)]

    [MenuItemAttribute(200, "&Edit\\&Undo drag/drop", GuidanceExplorer.Commands.UndoDragAndDrop, Keys.Control | Keys.Z)]
    [MenuItemAttribute(210, "&Edit\\-", null, Keys.None)]
    [MenuItemAttribute(230, "&Edit\\&Add To View...", GuidanceExplorer.Commands.MoveGuidanceItemToView, Keys.None)]
    [MenuItemAttribute(260, "&Edit\\-", null, Keys.None)]
    [MenuItemAttribute(270, "&Edit\\&Select All", GuidanceExplorer.Commands.SelectAll, Keys.Control | Keys.A)]

    [MenuItemAttribute(300, "&View\\Reset &Sort Order", GuidanceExplorer.Commands.ResetSortOrder, Keys.None)]
    [MenuItemAttribute(310, "&View\\&Reset Columns", GuidanceExplorer.Commands.ResetColumns, Keys.None)]
    [MenuItemAttribute(320, "&View\\Reset &Filters", GuidanceExplorer.Commands.ResetFilters, Keys.None)]
    [MenuItemAttribute(330, "&View\\&Column Chooser", GuidanceExplorer.Commands.ColumnChooser, Keys.None)]

    [MenuItemAttribute(400, "&Help\\&About", GuidanceExplorer.Commands.HelpAbout, Keys.None)]

    [MenuItemAttribute(500, "context\\New Library", GuidanceExplorer.Commands.NewLibrary, Keys.None)]
    [MenuItemAttribute(510, "context\\Import Library", GuidanceExplorer.Commands.ImportLibrary, Keys.None)]
    [MenuItemAttribute(520, "context\\Suscribe to &Library\\From &File...", GuidanceExplorer.Commands.SuscribeLibraryFromFile, Keys.None)]
    [MenuItemAttribute(530, "context\\Suscribe to &Library\\From &Web...", GuidanceExplorer.Commands.SuscribeLibraryFromWeb, Keys.None)]
    [MenuItemAttribute(535, "context\\Suscribe to &Library\\From &Rss...", GuidanceExplorer.Commands.SuscribeLibraryFromRss, Keys.None)]
    [MenuItemAttribute(540, "context\\Save Library as...", GuidanceExplorer.Commands.SaveLibraryAs, Keys.None)]
    [MenuItemAttribute(550, "context\\-", null, Keys.None)]
    [MenuItemAttribute(560, "context\\Synchronize Library", Commands.Synchronize, Keys.None)]
    public class LibrariesNode : TreeNode
    {
        /// <summary>
        /// Factory.
        /// </summary>
        /// <returns></returns>
        public static LibrariesNode New()
        {
            return new LibrariesNode();
        }

        /// <summary>
        /// Private default constructor.
        /// </summary>
        private LibrariesNode() : base("Libraries", 1, 1) { }

    }
}

