//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using GuidanceExplorer.Model;

namespace GuidanceExplorer.Controls
{
    /// <summary>
    /// Represents the root MyViews node in the tree.
    /// </summary>
    [MenuItemAttribute(15, "&File\\New Folder", GuidanceExplorer.Commands.NewFolder, Keys.None)]
    [MenuItemAttribute(20, "&File\\New View", GuidanceExplorer.Commands.NewView, Keys.None)]
    [MenuItemAttribute(30, "&File\\New Search View", GuidanceExplorer.Commands.NewSearchView, Keys.None)]
    [MenuItemAttribute(40, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(70, "&File\\Save &Item as Xml...", "Save Items as Xml...", GuidanceExplorer.Commands.SaveItemAsXml, Keys.None)]
    [MenuItemAttribute(80, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(90, "&File\\E&xport as Library...", GuidanceExplorer.Commands.ExportMyViewsAsLibrary, Keys.None)]
    [MenuItemAttribute(100, "&File\\Save Item &as...", "Save Items as...", GuidanceExplorer.Commands.SaveItemAs, Keys.None)]
    [MenuItemAttribute(101, "&File\\&Save View as...", GuidanceExplorer.Commands.SaveMyViewsAs, Keys.None)]
    [MenuItemAttribute(110, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(111, "&File\\Suscribe to &Library\\From &File...", GuidanceExplorer.Commands.SuscribeLibraryFromFile, Keys.None)]
    [MenuItemAttribute(112, "&File\\Suscribe to &Library\\From &Web...", GuidanceExplorer.Commands.SuscribeLibraryFromWeb, Keys.None)]
    [MenuItemAttribute(113, "&File\\Suscribe to &Library\\From &Rss...", GuidanceExplorer.Commands.SuscribeLibraryFromRss, Keys.None)]
    [MenuItemAttribute(115, "&File\\S&ynchronize Libraries", Commands.Synchronize, Keys.None)]
    [MenuItemAttribute(116, "&File\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(118, "&File\\Add/Remove &Guidance Type...", GuidanceExplorer.Commands.AddRemoveGuidanceType, Keys.None)]
    [MenuItemAttribute(119, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(120, "&File\\&Exit", GuidanceExplorer.Commands.FileExit, Keys.None)]

    [MenuItemAttribute(200, "&Edit\\&Undo drag/drop", GuidanceExplorer.Commands.UndoDragAndDrop, Keys.Control | Keys.Z)]
    [MenuItemAttribute(210, "&Edit\\-", null, Keys.None)]
    [MenuItemAttribute(230, "&Edit\\&Add To View...", GuidanceExplorer.Commands.MoveGuidanceItemToView, Keys.None)]
    [MenuItemAttribute(260, "&Edit\\-", null, Keys.None)]
    [MenuItemAttribute(270, "&Edit\\&Select All", GuidanceExplorer.Commands.SelectAll, Keys.Control | Keys.A)]

    [MenuItemAttribute(300, "&View\\Reset &Sort Order", GuidanceExplorer.Commands.ResetSortOrder, Keys.None)]
    [MenuItemAttribute(310, "&View\\&Reset Columns", GuidanceExplorer.Commands.ResetColumns, Keys.None)]
    [MenuItemAttribute(320, "&View\\Reset &Filters", GuidanceExplorer.Commands.ResetFilters, Keys.None)]
    [MenuItemAttribute(330, "&View\\&Column Chooser", GuidanceExplorer.Commands.ColumnChooser, Keys.None)]

    [MenuItemAttribute(400, "&Help\\&About", GuidanceExplorer.Commands.HelpAbout, Keys.None)]

    [MenuItemAttribute(510, "context\\New Folder", GuidanceExplorer.Commands.NewFolder, Keys.None)]
    [MenuItemAttribute(520, "context\\New View", GuidanceExplorer.Commands.NewView, Keys.None)]
    [MenuItemAttribute(530, "context\\New Search View", GuidanceExplorer.Commands.NewSearchView, Keys.None)]
    [MenuItemAttribute(540, "context\\-", null, Keys.None)]
    [MenuItemAttribute(560, "context\\Export as Library...", GuidanceExplorer.Commands.ExportMyViewsAsLibrary, Keys.None)]
    [MenuItemAttribute(561, "context\\Save View as...", GuidanceExplorer.Commands.SaveMyViewsAs, Keys.None)]
    public class MyViewsNode : ModelNode<UserViews>
    {
        /// <summary>
        /// Factory.
        /// </summary>
        /// <returns></returns>
        public static MyViewsNode New()
        {
            return new MyViewsNode();
        }

        /// <summary>
        /// Default private constructor.
        /// </summary>
        private MyViewsNode() : base("My Views", 2, 2) { }

        /// <summary>
        /// Whether the node is editable.
        /// </summary>
        public override bool CanDrop(IModelItem modelNode)
        {
            if (modelNode == null)
            {
                return false;
            }

            ViewNode viewNode = modelNode as ViewNode;
            FolderNode folderNode = modelNode as FolderNode;
            SearchViewNode searchNode = modelNode as SearchViewNode;
            if (folderNode != null)
            {
                /*
                 * Test for folders:
                 *  Only folders of the same library can be drop in the library.
                 *  The root folders can't be droped in the library.
                 */
                Library otherLibrary = folderNode.ModelInstance.GetContainerLibrary();
                if (otherLibrary != null)
                {
                    return false;
                }

                //TODO: avoid droping into the same place.
                //if (Object.ReferenceEquals(folderNode.ModelInstance.Container, ModelInstance))
                //{
                //    return false;
                //}

                return true;

            }
            else if (viewNode != null)
            {
                /*
                 * Test for folders:
                 *  Only folders of the same library can be drop in the library.
                 *  The root folders can't be droped in the library.
                 */
                Library otherLibrary = viewNode.ModelInstance.GetContainerLibrary();
                if (otherLibrary != null)
                {
                    return false;
                }

                //Root views cant be dragged
                if (viewNode.ModelInstance.RootView)
                {
                    return false;
                }

                //TODO: avoid droping into the same place.
                //if (Object.ReferenceEquals(viewNode.ModelInstance.Container, ModelInstance))
                //{
                //    return false;
                //}

                return true;
            }
            else if (searchNode != null)
            {
                //Root views cant be dragged
                if (searchNode.ModelInstance.RootView)
                {
                    return false;
                }

                return true;
            }
            else
            {
                //Can't drop this model node.
            }
            return false;
        }

        public override bool IsEditable
        {
            get { return true; }
        }
    }
}

