//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using GuidanceExplorer.Model;

namespace GuidanceExplorer.Controls
{
    /// <summary>
    /// The node for a single library.
    /// </summary>
    [MenuItemAttribute(10, "&File\\&New Item", GuidanceExplorer.Commands.NewGuidanceItem, typeof(ModelItemMenuItemIsEditableCondition), typeof(NewItemMenuItemProvider), Keys.None)]
    [MenuItemAttribute(40, "&File\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(50, "&File\\&Import Item...", GuidanceExplorer.Commands.ImportGuidanceItemToView, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(60, "&File\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(70, "&File\\Save &Item as Xml...", "Save Items as Xml...", GuidanceExplorer.Commands.SaveItemAsXml, Keys.None)]
    [MenuItemAttribute(80, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(90, "&File\\E&xport as Library...", GuidanceExplorer.Commands.ExportViewAsLibrary, Keys.None)]
    [MenuItemAttribute(91, "&File\\&Save View as...", GuidanceExplorer.Commands.SaveViewAs, Keys.None)]
    [MenuItemAttribute(100, "&File\\Save Item &as...", "Save Items as...", GuidanceExplorer.Commands.SaveItemAs, Keys.None)]
    [MenuItemAttribute(110, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(111, "&File\\Suscribe to &Library\\From &File...", GuidanceExplorer.Commands.SuscribeLibraryFromFile, Keys.None)]
    [MenuItemAttribute(112, "&File\\Suscribe to &Library\\From &Web...", GuidanceExplorer.Commands.SuscribeLibraryFromWeb, Keys.None)]
    [MenuItemAttribute(113, "&File\\Suscribe to &Library\\From &Rss...", GuidanceExplorer.Commands.SuscribeLibraryFromRss, Keys.None)]
    [MenuItemAttribute(115, "&File\\S&ynchronize Libraries", Commands.Synchronize, Keys.None)]
    [MenuItemAttribute(116, "&File\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(120, "&File\\&Rename View", GuidanceExplorer.Commands.RenameView, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(130, "&File\\&Delete View", GuidanceExplorer.Commands.DeleteView, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(140, "&File\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(148, "&File\\Add/Remove &Guidance Type...", GuidanceExplorer.Commands.AddRemoveGuidanceType, Keys.None)]
    [MenuItemAttribute(149, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(150, "&File\\&Exit", GuidanceExplorer.Commands.FileExit, Keys.None)]

    [MenuItemAttribute(200, "&Edit\\&Undo drag/drop", GuidanceExplorer.Commands.UndoDragAndDrop, Keys.Control | Keys.Z)]
    [MenuItemAttribute(210, "&Edit\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(230, "&Edit\\&Add To View...", GuidanceExplorer.Commands.MoveGuidanceItemToView, Keys.None)]
    [MenuItemAttribute(250, "&Edit\\Dele&te item", GuidanceExplorer.Commands.DeleteGuidanceItem, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(260, "&Edit\\-", null, Keys.None)]
    [MenuItemAttribute(270, "&Edit\\&Select All", GuidanceExplorer.Commands.SelectAll, Keys.Control | Keys.A)]

    [MenuItemAttribute(300, "&View\\Reset &Sort Order", GuidanceExplorer.Commands.ResetSortOrder, Keys.None)]
    [MenuItemAttribute(310, "&View\\&Reset Columns", GuidanceExplorer.Commands.ResetColumns, Keys.None)]
    [MenuItemAttribute(320, "&View\\Reset &Filters", GuidanceExplorer.Commands.ResetFilters, Keys.None)]
    [MenuItemAttribute(330, "&View\\&Column Chooser", GuidanceExplorer.Commands.ColumnChooser, Keys.None)]

    [MenuItemAttribute(400, "&Help\\&About", GuidanceExplorer.Commands.HelpAbout, Keys.None)]

    [MenuItemAttribute(500, "context\\New Item", GuidanceExplorer.Commands.NewGuidanceItem, typeof(ModelItemMenuItemIsEditableCondition), typeof(NewItemMenuItemProvider), Keys.None)]
    [MenuItemAttribute(510, "context\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(520, "context\\Import Item...", GuidanceExplorer.Commands.ImportGuidanceItemToView, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(521, "context\\Save View as...", GuidanceExplorer.Commands.SaveViewAs, Keys.None)]
    [MenuItemAttribute(530, "context\\Export as Library...", GuidanceExplorer.Commands.ExportViewAsLibrary, Keys.None)]
    [MenuItemAttribute(540, "context\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(550, "context\\Rename View", GuidanceExplorer.Commands.RenameView, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(560, "context\\Delete View", GuidanceExplorer.Commands.DeleteView, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(570, "context\\-", null, typeof(WebSubscribedLibraryCondition), Keys.None)]
    [MenuItemAttribute(575, "context\\Synchronize Library", Commands.Synchronize, typeof(WebSubscribedLibraryCondition), Keys.None)]
#if ONLINEENABLED
    [MenuItemAttribute(580, "context\\Remove (experimental)", Commands.RemoveView, typeof(WebSubscribedLibraryCondition), Keys.None)]
    [MenuItemAttribute(590, "context\\Publish (experimental)...", GuidanceExplorer.Commands.PublishView, Keys.None)]
    //[MenuItemAttribute(595, "context\\SSE Synchronize (experimental)", Commands.SSESynchronize, Keys.None)]
#endif
    public class ViewNode : ModelNode<GuidanceExplorer.Model.View>
    {
        /// <summary>
        /// Factory.
        /// </summary>
        /// <returns></returns>
        public static ViewNode New()
        {
            return new ViewNode();
        }

        /// <summary>
        /// Default constructor.
        /// </summary>
        private ViewNode() : base("<a view>", 0, 0) { }

        /// <summary>
        /// Whether the node can be dropped.
        /// </summary>
        public override bool CanDrop(IModelItem modelNode)
        {
            Library library = ModelInstance.GetContainerLibrary();
            GuidanceItem guidanceItem = modelNode as GuidanceItem;
            GuidanceItemCollection guidanceItemCollection = modelNode as GuidanceItemCollection;
            if (guidanceItem != null)
            {
                if (library == null)
                {
                    return true;
                }
                else if (library.ReadProtection == ReadProtection.ReadWrite)
                {
                    return guidanceItem.Library == library;
                }
                return false;
            }
            else if (guidanceItemCollection != null)
            {
                foreach (GuidanceItem item in guidanceItemCollection)
                {
                    if (library != null)
                    {
                        if (library.ReadProtection == ReadProtection.ReadOnly)
                        {
                            return false;
                        }
                        else if (item.Library != library)
                        {
                            return false;
                        }
                    }
                }
                return true;
            }
            else
            {
                //Item type not supported.
                return false;
            }
        }

        /// <summary>
        /// Whether the node is editable.
        /// </summary>
        public override bool IsEditable
        {
            get
            {
                Library library = ModelInstance.GetContainerLibrary();
                if (library != null)
                {
                    return (library.ReadProtection == ReadProtection.ReadWrite);
                }
                return true;
            }
        }

    }
}

