//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================



using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using System.IO;
using GuidanceExplorer.Services;
using GuidanceExplorer.Model;
using System.Windows.Forms;

namespace GuidanceExplorer.Tests
{
    /*
*DupColumnsInFolderLibrary.xml
*DupColumnsInViewLibrary.xml
*DupFolderNameSimpleLibrary.xml
*DupFolderNameSimpleLibrary1.xml
*DupViewNameSimpleLibrary.xml
*DupViewNameSimpleLibrary1.xml
*SimpleLibrary.xml
*NoViewsLibrary.xml
*NoFoldersLibrary.xml
     */
    [TestFixture]
    public class UserViewsTests
    {
        [Test]
        public void SimpleLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\UserViewsTests\"), "SimpleLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            UserViews userViews = svc.ConvertFileInfoToUserViews(new FileInfo(fileName));

            Assert.IsNotNull(userViews);
            Assert.AreEqual(1, userViews.Folders.Count);
            Assert.AreEqual("root", userViews.Folders["root"].Caption);
            Assert.AreEqual(1, userViews.Folders["root"].Columns.Count);
            Assert.AreEqual("Title", userViews.Folders["root"].Columns["title"].Caption);
            Assert.AreEqual("title", userViews.Folders["root"].Columns["title"].Source);
            Assert.AreEqual(SortOrder.None, userViews.Folders["root"].Columns["title"].Sort);
            Assert.AreEqual(true, userViews.Folders["root"].Columns["title"].Visible);
            Assert.AreEqual(100, userViews.Folders["root"].Columns["title"].Width);
            Assert.AreEqual(2, userViews.Folders["root"].Folders.Count);
            Assert.AreEqual(1, userViews.Folders["root"].Views.Count);

            Assert.AreEqual(0, userViews.Folders["root"].Folders["subRootFolderA"].Views.Count);
            Assert.AreEqual(2, userViews.Folders["root"].Folders["subRootFolderA"].Columns.Count);
            Assert.AreEqual("A", userViews.Folders["root"].Folders["subRootFolderA"].Columns["A"].Caption);
            Assert.AreEqual("A", userViews.Folders["root"].Folders["subRootFolderA"].Columns["A"].Source);
            Assert.AreEqual(SortOrder.None, userViews.Folders["root"].Folders["subRootFolderA"].Columns["A"].Sort);
            Assert.AreEqual(true, userViews.Folders["root"].Folders["subRootFolderA"].Columns["A"].Visible);
            Assert.AreEqual(100, userViews.Folders["root"].Folders["subRootFolderA"].Columns["A"].Width);
            Assert.AreEqual("B", userViews.Folders["root"].Folders["subRootFolderA"].Columns["B"].Caption);
            Assert.AreEqual("B", userViews.Folders["root"].Folders["subRootFolderA"].Columns["B"].Source);
            Assert.AreEqual(SortOrder.Ascending, userViews.Folders["root"].Folders["subRootFolderA"].Columns["B"].Sort);
            Assert.AreEqual(false, userViews.Folders["root"].Folders["subRootFolderA"].Columns["B"].Visible);
            Assert.AreEqual(200, userViews.Folders["root"].Folders["subRootFolderA"].Columns["B"].Width);

            Assert.AreEqual(1, userViews.Views.Count);

        }

        [Test]
        public void NoViewsLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\UserViewsTests\"), "NoViewsLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            UserViews userViews = svc.ConvertFileInfoToUserViews(new FileInfo(fileName));

            Assert.IsNotNull(userViews);
            Assert.AreEqual(0, userViews.Views.Count);
            Assert.AreEqual(1, userViews.Folders.Count);
            Assert.AreEqual("root", userViews.Folders["root"].Caption);
            Assert.AreEqual(1, userViews.Folders["root"].Columns.Count);
            Assert.AreEqual("A", userViews.Folders["root"].Columns["A"].Source);
            Assert.AreEqual("a", userViews.Folders["root"].Columns["A"].Caption);
            Assert.AreEqual(SortOrder.None, userViews.Folders["root"].Columns["A"].Sort);
            Assert.AreEqual(true, userViews.Folders["root"].Columns["A"].Visible);
            Assert.AreEqual(100, userViews.Folders["root"].Columns["A"].Width);
            Assert.AreEqual(0, userViews.Folders["root"].Folders.Count);
            Assert.AreEqual(0, userViews.Folders["root"].Views.Count);
        }

        [Test]
        public void NoFoldersLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\UserViewsTests\"), "NoFoldersLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            UserViews userViews = svc.ConvertFileInfoToUserViews(new FileInfo(fileName));

            Assert.IsNotNull(userViews);
            Assert.AreEqual(0, userViews.Folders.Count);
            Assert.AreEqual(2, userViews.Views.Count);
            Assert.AreEqual("subRootView", userViews.Views["subRootView"].Caption);
            Assert.AreEqual(1, userViews.Views["subRootView"].Columns.Count);
            Assert.AreEqual("A", userViews.Views["subRootView"].Columns["A"].Source);
            Assert.AreEqual("a", userViews.Views["subRootView"].Columns["A"].Caption);
            Assert.AreEqual(SortOrder.None, userViews.Views["subRootView"].Columns["A"].Sort);
            Assert.AreEqual(true, userViews.Views["subRootView"].Columns["A"].Visible);
            Assert.AreEqual(100, userViews.Views["subRootView"].Columns["A"].Width);
            Assert.AreEqual(1, userViews.Views["subRootView"].Items.Count);

            Assert.AreEqual("rootView", userViews.Views["rootView"].Caption);
            Assert.AreEqual(1, userViews.Views["rootView"].Columns.Count);
            Assert.AreEqual("B", userViews.Views["rootView"].Columns["B"].Source);
            Assert.AreEqual("b", userViews.Views["rootView"].Columns["B"].Caption);
            Assert.AreEqual(SortOrder.None, userViews.Views["rootView"].Columns["B"].Sort);
            Assert.AreEqual(true, userViews.Views["rootView"].Columns["B"].Visible);
            Assert.AreEqual(200, userViews.Views["rootView"].Columns["B"].Width);
            Assert.AreEqual(2, userViews.Views["rootView"].Items.Count);

        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupColumnsInFolderLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\UserViewsTests\"), "DupColumnsInFolderLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            UserViews userViews = svc.ConvertFileInfoToUserViews(new FileInfo(fileName));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupColumnsInViewLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\UserViewsTests\"), "DupColumnsInViewLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            UserViews userViews = svc.ConvertFileInfoToUserViews(new FileInfo(fileName));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupFolderNameSimpleLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\UserViewsTests\"), "DupFolderNameSimpleLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            UserViews userViews = svc.ConvertFileInfoToUserViews(new FileInfo(fileName));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupFolderNameSimpleLibrary1()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\UserViewsTests\"), "DupFolderNameSimpleLibrary1.xml");

            ModelStorageService svc = new ModelStorageService();

            UserViews userViews = svc.ConvertFileInfoToUserViews(new FileInfo(fileName));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupViewNameSimpleLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\UserViewsTests\"), "DupViewNameSimpleLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            UserViews userViews = svc.ConvertFileInfoToUserViews(new FileInfo(fileName));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupViewNameSimpleLibrary1()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\UserViewsTests\"), "DupViewNameSimpleLibrary1.xml");

            ModelStorageService svc = new ModelStorageService();

            UserViews userViews = svc.ConvertFileInfoToUserViews(new FileInfo(fileName));
        }

        [Test]
        public void UserViewsSingleView()
        {
            FileInfo fileName = new FileInfo(Path.GetTempFileName());
            //Just in case :)
            File.Delete(fileName.FullName);

            UserViews userViews = UserViews.New();
            userViews.SourceFileInfo = fileName;
            GuidanceExplorer.Model.View view = GuidanceExplorer.Model.View.New();
            view.Caption = "TestView";
            view.Columns = ColumnCollection.DefaultColumns(GuidanceType.New(GuidanceType.DefaultTypeName));
            view.Container = userViews;
            GuidanceItem guidanceItem = GuidanceItem.New();
            guidanceItem.Id = "11";
            view.Items.Add(guidanceItem.Id, guidanceItem);
            guidanceItem = GuidanceItem.New();
            guidanceItem.Id = "22";
            view.Items.Add(guidanceItem.Id, guidanceItem);
            userViews.Views.Add(view.Caption, view);

            ModelStorageService svc = new ModelStorageService();
            svc.RenderUserViewsToFileInfo(userViews, fileName);

            UserViews newUserViews = svc.ConvertFileInfoToUserViews(fileName);

            Assert.AreEqual(userViews.Views.Count, newUserViews.Views.Count);
            Assert.AreEqual(userViews.Folders.Count, newUserViews.Folders.Count);

            Assert.AreEqual(userViews.Views["TestView"].Caption, newUserViews.Views["TestView"].Caption);
            Assert.AreEqual(userViews.Views["TestView"].Columns.Count, newUserViews.Views["TestView"].Columns.Count);

            Assert.AreEqual(userViews.Views["TestView"].Columns, newUserViews.Views["TestView"].Columns);

            Assert.AreEqual(userViews.Views["TestView"].Items.Count, newUserViews.Views["TestView"].Items.Count);
        }

        [Test]
        public void UserViewsFolder()
        {
            FileInfo fileName = new FileInfo(Path.GetTempFileName());
            //Just in case :)
            File.Delete(fileName.FullName);

            UserViews userViews = UserViews.New();
            userViews.SourceFileInfo = fileName;
            Folder folder = Folder.New();
            folder.Caption = "TestFolder";
            folder.Columns = ColumnCollection.DefaultColumns(GuidanceType.New(GuidanceType.DefaultTypeName));
            folder.Container = userViews;
            userViews.Folders.Add(folder.Caption, folder);

            ModelStorageService svc = new ModelStorageService();
            svc.RenderUserViewsToFileInfo(userViews, fileName);

            UserViews newUserViews = svc.ConvertFileInfoToUserViews(fileName);

            Assert.AreEqual(userViews.Views.Count, newUserViews.Views.Count);
            Assert.AreEqual(userViews.Folders.Count, newUserViews.Folders.Count);

            Assert.AreEqual(folder, newUserViews.Folders["TestFolder"]);

            Assert.AreEqual(userViews.Folders["TestFolder"].Views.Count, newUserViews.Folders["TestFolder"].Views.Count);
            Assert.AreEqual(userViews.Folders["TestFolder"].Folders.Count, newUserViews.Folders["TestFolder"].Folders.Count);
        }

        [Test]
        public void UserViewsFolderAndView()
        {
            FileInfo fileName = new FileInfo(Path.GetTempFileName());
            //Just in case :)
            File.Delete(fileName.FullName);

            UserViews userViews = UserViews.New();
            userViews.SourceFileInfo = fileName;
            Folder folder = Folder.New();
            folder.Caption = "TestFolder";
            folder.Columns = ColumnCollection.DefaultColumns(GuidanceType.New(GuidanceType.DefaultTypeName));
            folder.Container = userViews;
            userViews.Folders.Add(folder.Caption, folder);
            GuidanceExplorer.Model.View view = GuidanceExplorer.Model.View.New();
            view.Caption = "TestView";
            view.Columns = ColumnCollection.DefaultColumns(GuidanceType.New(GuidanceType.DefaultTypeName));
            view.Container = userViews;
            GuidanceItem guidanceItem = GuidanceItem.New();
            guidanceItem.Id = "11";
            view.Items.Add(guidanceItem.Id, guidanceItem);
            guidanceItem = GuidanceItem.New();
            guidanceItem.Id = "22";
            view.Items.Add(guidanceItem.Id, guidanceItem);
            userViews.Views.Add(view.Caption, view);

            ModelStorageService svc = new ModelStorageService();
            svc.RenderUserViewsToFileInfo(userViews, fileName);

            UserViews newUserViews = svc.ConvertFileInfoToUserViews(fileName);

            Assert.AreEqual(userViews.Views.Count, newUserViews.Views.Count);
            Assert.AreEqual(userViews.Folders.Count, newUserViews.Folders.Count);

            Assert.AreEqual(userViews.Views["TestView"].Caption, newUserViews.Views["TestView"].Caption);
            Assert.AreEqual(userViews.Views["TestView"].Columns.Count, newUserViews.Views["TestView"].Columns.Count);

            Assert.AreEqual(userViews.Views["TestView"].Columns, newUserViews.Views["TestView"].Columns);

            Assert.AreEqual(userViews.Views["TestView"].Items.Count, newUserViews.Views["TestView"].Items.Count);

            Assert.AreEqual(userViews.Folders["TestFolder"].Caption, newUserViews.Folders["TestFolder"].Caption);
            Assert.AreEqual(userViews.Folders["TestFolder"].Columns.Count, newUserViews.Folders["TestFolder"].Columns.Count);
            Assert.AreEqual(userViews.Folders["TestFolder"].Columns, newUserViews.Folders["TestFolder"].Columns);

            Assert.AreEqual(userViews.Folders["TestFolder"].Views.Count, newUserViews.Folders["TestFolder"].Views.Count);
            Assert.AreEqual(userViews.Folders["TestFolder"].Folders.Count, newUserViews.Folders["TestFolder"].Folders.Count);

        }

        [Test]
        public void UserViewsFolderAndTwoView()
        {
            FileInfo fileName = new FileInfo(Path.GetTempFileName());
            //Just in case :)
            File.Delete(fileName.FullName);

            UserViews userViews = UserViews.New();
            userViews.SourceFileInfo = fileName;
            Folder folder = Folder.New();
            folder.Caption = "TestFolder";
            folder.Columns = ColumnCollection.DefaultColumns(GuidanceType.New(GuidanceType.DefaultTypeName));
            folder.Container = userViews;
            userViews.Folders.Add(folder.Caption, folder);
            GuidanceExplorer.Model.View view = GuidanceExplorer.Model.View.New();
            view.Caption = "TestView";
            view.Columns = ColumnCollection.DefaultColumns(GuidanceType.New(GuidanceType.DefaultTypeName));
            view.Container = userViews;
            GuidanceItem guidanceItem = GuidanceItem.New();
            guidanceItem.Id = "11";
            view.Items.Add(guidanceItem.Id, guidanceItem);
            guidanceItem = GuidanceItem.New();
            guidanceItem.Id = "22";
            view.Items.Add(guidanceItem.Id, guidanceItem);
            view = GuidanceExplorer.Model.View.New();
            view.Caption = "TestView";
            view.Columns = ColumnCollection.DefaultColumns(GuidanceType.New(GuidanceType.DefaultTypeName));
            view.Container = userViews;
            guidanceItem = GuidanceItem.New();
            guidanceItem.Id = "11";
            view.Items.Add(guidanceItem.Id, guidanceItem);
            guidanceItem = GuidanceItem.New();
            guidanceItem.Id = "22";
            view.Items.Add(guidanceItem.Id, guidanceItem);
            userViews.Views.Add(view.Caption, view);

            ModelStorageService svc = new ModelStorageService();
            svc.RenderUserViewsToFileInfo(userViews, fileName);

            UserViews newUserViews = svc.ConvertFileInfoToUserViews(fileName);

            Assert.AreEqual(userViews.Folders, newUserViews.Folders);
            Assert.AreEqual(userViews.Views, newUserViews.Views);
        }
    }
}

