//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================



using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.Practices.CompositeUI.SmartParts;
using Microsoft.Practices.CompositeUI;
using GuidanceExplorer.Services;
using GuidanceExplorer.WorkItems;
using GuidanceExplorer.Model;

namespace GuidanceExplorer.Views
{
    [SmartPart]
    public partial class SearchToolbarView : UserControl
    {
        /// <summary>
        /// The main model service instance.
        /// </summary>
        private IModelStorageService _modelService;

        /// <summary>
        /// The main work item instance.
        /// </summary>
        private GuidanceExplorerWorkItem _workItem;

        /// <summary>
        /// The ultimate search view created.
        /// </summary>
        private SearchView _lastSearchView;

        /// <summary>
        /// The main model service instance.
        /// </summary>
        [ServiceDependency]
        public IModelStorageService ModelService
        {
            set { _modelService = value; }
        }

        /// <summary>
        /// The main work item instance.
        /// </summary>
        [ServiceDependency]
        public GuidanceExplorerWorkItem WorkItem
        {
            set { _workItem = value; }
        }

        /// <summary>
        /// Deafult constructor.
        /// </summary>
        public SearchToolbarView()
        {
            InitializeComponent();
            ResetSearchToolBar();
        }

        /// <summary>
        /// Occurs when a key is pressed in the text box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtFullTextSearch_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                btnSearch.PerformClick();
            }
        }

        /// <summary>
        /// When the search button is clicked.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSearch_Click(object sender, EventArgs e)
        {
            if (_workItem.PerformFullTextSearch(txtFullTextSearch.Text))
            {
                btnSave.Enabled = true;
                btnErase.Enabled = true;
            }
        }

        /// <summary>
        /// Called when the user clicks the erase button.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnErase_Click(object sender, EventArgs e)
        {
            _workItem.ClearFullTextSearch();
        }

        /// <summary>
        /// Called when the user clicks the save button.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, EventArgs e)
        {
            if (_lastSearchView != null)
            {
                _workItem.SaveFullTextSearchView(_lastSearchView);
            }
            else
            {
                MessageBox.Show("No current search view.");
            }
        }

        /// <summary>
        /// Get/set the text entered by the user.
        /// </summary>
        public string FullTextSearch
        {
            get { return txtFullTextSearch.Text; }
            set { txtFullTextSearch.Text = value; }
        }

        /// <summary>
        /// The latest search view created.
        /// </summary>
        public SearchView LastSearchView
        {
            get { return _lastSearchView; }
            set { _lastSearchView = value; }
        }

        /// <summary>
        /// Resets the text box and buttons to its initial state
        /// </summary>
        public void ResetSearchToolBar()
        {
            txtFullTextSearch.Text = string.Empty;
            btnSave.Enabled = false;
            btnErase.Enabled = false;
            btnSearch.Enabled = true;
        }
    }
}

