//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.Configuration;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI.Commands;
using Microsoft.Practices.CompositeUI.EventBroker;
using Microsoft.Practices.CompositeUI.WinForms;
using Microsoft.Practices.CompositeUI.SmartParts;

using GuidanceExplorer.Config;
using GuidanceExplorer.Model;
using GuidanceExplorer.Controls;
using GuidanceExplorer.Views;
using GuidanceExplorer.Services;
using System.ComponentModel;

namespace GuidanceExplorer.WorkItems
{
    public partial class GuidanceExplorerWorkItem
    {
        [CommandHandler(GuidanceExplorer.Commands.FileExit)]
        public void FileExit(object sender, EventArgs e)
        {
            _modelService.SaveGuidance(_modelService.Guidance, Environment.CurrentDirectory);
            Application.Exit();
        }

        public void AddSearchView(SearchViewNode searchViewNode, SearchView searchView, IModelItem containerNode)
        {
            LibraryNode libraryNode = containerNode as LibraryNode;
            FolderNode folderNode = containerNode as FolderNode;
            MyViewsNode myViewsNode = containerNode as MyViewsNode;

            if (libraryNode != null)
            {
                int i = 0;
                while (libraryNode.ModelInstance.Views.ContainsKey(searchView.Caption))
                {
                    searchView.Caption = string.Format(Messages.NEW_SEARCH_VIEW + " {0}", i++);
                }
                searchViewNode.Text = searchView.Caption;
                searchView.ContainerScope = _modelService.Guidance;
                searchView.Container = libraryNode.ModelInstance;
                //Add as a child to the library.
                libraryNode.ModelInstance.Views.Add(searchView.Caption, searchView);
                libraryNode.ModelInstance.HasChanges = true;

                //Add the search view to the tree node.
                libraryNode.Nodes.Add(searchViewNode);

                _treeView.TreeView.SelectedNode = searchViewNode;

                searchViewNode.BeginEdit();
            }
            else if (folderNode != null)
            {
                int i = 0;
                while (folderNode.ModelInstance.Views.ContainsKey(searchView.Caption))
                {
                    searchView.Caption = string.Format(Messages.NEW_SEARCH_VIEW + " {0}", i++);
                }
                searchViewNode.Text = searchView.Caption;
                searchView.ContainerScope = _modelService.Guidance;
                searchView.ParentFolder = folderNode.ModelInstance;
                folderNode.ModelInstance.Views.Add(searchView.Caption, searchView);
                //It's added as a sibling to the folder.
                folderNode.Nodes.Add(searchViewNode);
                if (folderNode.ModelInstance.GetContainerLibrary() != null)
                {
                    folderNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                }

                _treeView.TreeView.SelectedNode = searchViewNode;

                searchViewNode.BeginEdit();
            }
            else if (myViewsNode != null)
            {
                int i = 0;
                while (myViewsNode.ModelInstance.Views.ContainsKey(searchView.Caption))
                {
                    searchView.Caption = string.Format(Messages.NEW_SEARCH_VIEW + " {0}", i++);
                }
                searchViewNode.Text = searchView.Caption;
                searchView.ContainerScope = _modelService.Guidance;
                searchView.Container = myViewsNode.ModelInstance;

                //Add as a child to the library.
                myViewsNode.ModelInstance.Views.Add(searchView.Caption, searchView);

                //Add the search view to the tree node.
                myViewsNode.Nodes.Add(searchViewNode);

                _treeView.TreeView.SelectedNode = searchViewNode;

                searchViewNode.BeginEdit();
            }
            else
            {
                SearchViewNodeSelected(this, new ModelNodeEventArgs<SearchView>(searchViewNode));
            }
        }


        [CommandHandler(GuidanceExplorer.Commands.NewSearchView)]
        public void NewSearchView(object sender, EventArgs e)
        {
            IWorkspace workspace = RootWorkItem.Workspaces.Get<WindowWorkspace>("SearchViewEditor");
            if (workspace != null)
            {
                RootWorkItem.Workspaces.Remove(workspace);
            }

            SearchCriteriaControl searchControl = RootWorkItem.SmartParts.Get<SearchCriteriaControl>("SearchViewControl");
            if (searchControl != null)
            {
                RootWorkItem.SmartParts.Remove(searchControl);
            }

            workspace = new WindowWorkspace(new Form());
            RootWorkItem.Workspaces.Add(workspace, "SearchViewEditor");
            searchControl = new SearchCriteriaControl(_treeView.GetCurrentNode() as IModelItem);
            RootWorkItem.SmartParts.Add(searchControl, "SearchViewControl");
            WindowSmartPartInfo info = new WindowSmartPartInfo();
            info.MaximizeBox = false;
            if (_treeView.GetCurrentNode() is SearchViewNode)
            {
                info.Title = Messages.EDIT_SEARCH_VIEW;
            }
            else
            {
                info.Title = Messages.NEW_SEARCH_VIEW;
            }
            workspace.Show(searchControl, info);
        }

        [CommandHandler(GuidanceExplorer.Commands.RefreshFileSuscribedLibrary)]
        public void RefreshFileSuscribedLibrary(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;

            if (modelNode != null)
            {
                LibraryNode libraryNode = modelNode as LibraryNode;
                if (libraryNode != null)
                {
                    LibrariesNode librariesNode = libraryNode.Parent as LibrariesNode;

                    if (librariesNode != null)
                    {
                        _modelService.Guidance.Libraries.Remove(libraryNode.ModelInstance.Name);
                        libraryNode.Remove();

                        SuscribeLibraryFromFile(librariesNode, libraryNode.ModelInstance.OriginalSourceFileInfo.FullName);
                    }
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ColumnChooser)]
        public void ColumnChooser(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            LibrariesNode librariesNode = treeNode as LibrariesNode;
            if (modelNode != null)
            {
                LibraryNode libraryNode = modelNode as LibraryNode;
                FolderNode folderNode = modelNode as FolderNode;
                MyViewsNode myViewsNode = treeNode as MyViewsNode;
                ViewNode viewNode = treeNode as ViewNode;
                SearchViewNode searchViewNode = treeNode as SearchViewNode;
                GuidanceTypeNode typeNode = treeNode as GuidanceTypeNode;

                if (libraryNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(libraryNode.ModelInstance.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        _listView.ShowContents(libraryNode.ModelInstance, libraryNode.ModelInstance.Columns);
                    }
                }
                else if (folderNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(folderNode.ModelInstance.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        _listView.ShowContents(folderNode.ModelInstance, folderNode.ModelInstance.Columns);
                    }
                }
                else if (myViewsNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(myViewsNode.ModelInstance.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        _listView.ShowContents(myViewsNode.ModelInstance, myViewsNode.ModelInstance.Columns);
                    }
                }
                else if (viewNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(viewNode.ModelInstance.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        _listView.ShowContents(viewNode.ModelInstance, viewNode.ModelInstance.Columns);
                    }
                }
                else if (searchViewNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(searchViewNode.ModelInstance.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        _listView.ShowContents(searchViewNode.ModelInstance, searchViewNode.ModelInstance.Columns);
                    }
                }
                else if (typeNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(typeNode.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        GuidanceTypeNodeSelected(this, new ModelNodeEventArgs<GuidanceType>(typeNode));
                    }
                }
                else
                {
                    //do nothing
                }
            }
            else if (librariesNode != null)
            {
                ColumnChooserForm columnChooser = new ColumnChooserForm(_modelService.Guidance.Columns);
                if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                {
                    LibrariesNodeSelected(this, null);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.NewGuidanceItem)]
        public void NewGuidanceItem(object sender, EventArgs e)
        {
            //Get the new guidance item type.
            GuidanceTypeEventArgs gtea = e as GuidanceTypeEventArgs;
            GuidanceType newGuidanceItemType = gtea.GuidanceType;

            //Find the context libray.
            Library library = GetCurrentLibrary();

            //Create the guidance item.
            GuidanceItemFileReference gifr = GuidanceItemFileReference.New();
            gifr.GuidanceType = newGuidanceItemType;

            if (library != null)
            {
                string baseLibraryPath = null;
                if (library.FileSuscribed)
                {
                    baseLibraryPath = library.OriginalSourceFileInfo.DirectoryName + "\\" + library.Caption;
                }
                else
                {
                    baseLibraryPath = Path.Combine(ModelStorageService.LibraryFolder, library.Caption);
                }
                gifr.FileName = Path.GetFullPath(Path.Combine(baseLibraryPath, gifr.Id + ".xml"));
                gifr.Library = library;
            }
            else
            {
                gifr.FileName = ModelStorageService.LibraryFolder;
            }

            gifr.Id = Guid.NewGuid().ToString();
            gifr.CSSFile = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, ModelStorageService.DefaultCSSFileName));
            gifr.Title = string.Empty;

            //Edit the guidance item.
            GuidanceItemOpened(this, new GuidanceItemFileReferenceEventArgs(new GuidanceItemFileReference[] { gifr }, ReadProtection.ReadWrite, _treeView.GetCurrentNode() as ViewNode));
        }

        [CommandHandler(GuidanceExplorer.Commands.ExportLibrary)]
        public void ExportLibrary(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            if (modelNode == null)
            {
                return;
            }

            LibraryNode libraryNode = modelNode as LibraryNode;
            if (libraryNode != null)
            {
                SaveFileDialog saveFile = new SaveFileDialog();
                saveFile.CheckPathExists = true;
                saveFile.DefaultExt = "*.xml";
                saveFile.Filter = Messages.XML_FILES_FILTER;
                saveFile.OverwritePrompt = true;
                saveFile.Title = "Save Library as";
                saveFile.ValidateNames = true;
                saveFile.RestoreDirectory = true;
                saveFile.FileName = libraryNode.Text + ".xml";

                //Ask the user for the file name.
                if (saveFile.ShowDialog() == DialogResult.OK)
                {
                    //If the target file exists, then delete it.
                    if (File.Exists(saveFile.FileName))
                    {
                        File.Delete(saveFile.FileName);
                    }

                    //Render the library to the file.
                    _modelService.RenderLibraryWithItemsToFileInfo(libraryNode.ModelInstance, new FileInfo(saveFile.FileName), _htmlService);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.NewFolder)]
        public void NewFolder(object sender, EventArgs e)
        {
            Folder folder = Folder.New();

            FolderNode newFolderNode = FolderNode.New();
            newFolderNode.ModelInstance = folder;

            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            if (modelNode == null)
            {
                //a folder has been created in an unknown node
                throw new NotImplementedException();
            }

            AddFolder(folder, newFolderNode, modelNode);

            newFolderNode.BeginEdit();
        }

        public void AddFolder(Folder folder, FolderNode newFolderNode, IModelItem modelNode)
        {
            LibraryNode libraryNode = modelNode as LibraryNode;
            FolderNode folderNode = modelNode as FolderNode;
            MyViewsNode myViewsNode = modelNode as MyViewsNode;
            if (libraryNode != null)
            {
                //Update the folder name
                folder.Caption = _treeView.GetNewNodeName(libraryNode, "New Folder");
                newFolderNode.Text = folder.Caption;

                //Update the model
                folder.Container = libraryNode.ModelInstance;
                folder.ParentFolder = null;
                libraryNode.ModelInstance.Folders.Add(folder.Caption, folder);
                libraryNode.ModelInstance.HasChanges = true;

                //Update the treeview
                libraryNode.Nodes.Add(newFolderNode);
                newFolderNode.TreeView.SelectedNode = newFolderNode;
            }
            else if (folderNode != null)
            {
                //Update the folder name
                folder.Caption = _treeView.GetNewNodeName(folderNode, "New Folder");
                newFolderNode.Text = folder.Caption;

                //Update the model
                folder.ParentFolder = folderNode.ModelInstance;
                folder.Container = null;
                folderNode.ModelInstance.Folders.Add(folder.Caption, folder);
                if (folderNode.ModelInstance.GetContainerLibrary() != null)
                {
                    folderNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                }

                //Update the treeview
                folderNode.Nodes.Add(newFolderNode);
                newFolderNode.TreeView.SelectedNode = newFolderNode;
            }
            else if (myViewsNode != null)
            {
                //Update the folder name
                folder.Caption = _treeView.GetNewNodeName(myViewsNode, "New Folder");
                newFolderNode.Text = folder.Caption;

                //Update the model
                _modelService.Guidance.UserViews.Folders.Add(folder.Caption, folder);
                folder.Container = _modelService.Guidance.UserViews;
                folder.ParentFolder = null;

                //Update the treeview
                myViewsNode.Nodes.Add(newFolderNode);
                newFolderNode.TreeView.SelectedNode = newFolderNode;
            }
            else
            {
                throw new NotImplementedException();
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.SaveViewAs)]
        public void SaveViewAs(object sender, EventArgs e)
        {
            ViewNode viewNode = _treeView.GetCurrentNode() as ViewNode;

            if (viewNode != null)
            {
                GuidanceItem[] items = new GuidanceItem[viewNode.ModelInstance.Items.Values.Count];
                viewNode.ModelInstance.Items.Values.CopyTo(items, 0);
                SaveItems(items);
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.SaveLibraryAs)]
        public void SaveLibraryAs(object sender, EventArgs e)
        {
            List<GuidanceItem> items = new List<GuidanceItem>();

            foreach (GuidanceItem item in _listView.Items)
            {
                items.Add(item);
            }

            SaveItems(items.ToArray());
        }

        [CommandHandler(GuidanceExplorer.Commands.SaveFolderAs)]
        public void SaveFolderAs(object sender, EventArgs e)
        {
            SaveLibraryAs(this, e);
        }

        [CommandHandler(GuidanceExplorer.Commands.SaveMyViewsAs)]
        public void SaveMyViewsAs(object sender, EventArgs e)
        {
            SaveLibraryAs(this, e);
        }

        [CommandHandler(GuidanceExplorer.Commands.SaveItemAs)]
        public void SaveItemAs(object sender, EventArgs e)
        {
            GuidanceItem[] items = _listView.GetSelectedGuidanceItems();

            if (items == null || items.Length == 0)
            {
                MessageBox.Show("Please select an item to save", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                SaveItems(items);
            }
        }

        private void SaveItems(GuidanceItem[] items)
        {
            SaveFileDialog saveFileDlg = new SaveFileDialog();
            saveFileDlg.DefaultExt = ".doc";
            saveFileDlg.AddExtension = true;
            saveFileDlg.Filter = Messages.WORD_FILES_FILTER + "|" + Messages.HTML_FILES_FILTER;
            saveFileDlg.RestoreDirectory = true;
            saveFileDlg.ValidateNames = true;

            if (saveFileDlg.ShowDialog() == DialogResult.OK)
            {
                if (items.Length == 1)
                {
                    GuidanceItemFileReference item = items[0] as GuidanceItemFileReference;

                    if (item == null)
                    {
                        item = _modelService.Guidance.GetGuidanceItemFileReference(items[0]);
                    }

                    FileInfo file = new FileInfo(saveFileDlg.FileName);
                    _htmlService.RenderGuidanceItemToHtml(item, file);
                }
                else
                {
                    List<GuidanceItemFileReference> guidanceItems = new List<GuidanceItemFileReference>(items.Length);

                    foreach (GuidanceItem guidanceItem in items)
                    {
                        GuidanceItemFileReference item = guidanceItem as GuidanceItemFileReference;

                        if (item == null)
                        {
                            item = _modelService.Guidance.GetGuidanceItemFileReference(guidanceItem);
                        }

                        guidanceItems.Add(item);
                    }

                    if (Path.GetExtension(saveFileDlg.FileName) == ".doc")
                    {
                        _htmlService.RenderGuidanceItemsToWordFileInfoWithIndex(guidanceItems, new FileInfo(saveFileDlg.FileName));
                    }
                    else //Path extensionis htm
                    {
                        _htmlService.RenderGuidanceItemsToHtmlFileInfoWithIndex(guidanceItems, new FileInfo(saveFileDlg.FileName));
                    }
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.SelectAll)]
        public void SelectAll(object sender, EventArgs e)
        {
            _listView.SelectAllItems();
            _listView.Focus();
        }

        [CommandHandler(GuidanceExplorer.Commands.NewView)]
        public void NewView(object sender, EventArgs e)
        {
            GuidanceExplorer.Model.View view = GuidanceExplorer.Model.View.New();

            ViewNode newViewNode = ViewNode.New();
            newViewNode.ModelInstance = view;

            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            if (modelNode == null)
            {
                //a view has been created in an unknown node
                throw new NotImplementedException();
            }

            AddView(view, newViewNode, modelNode);

            newViewNode.BeginEdit();
        }

        public void AddView(GuidanceExplorer.Model.View view, ViewNode newViewNode, IModelItem modelNode)
        {
            LibraryNode libraryNode = modelNode as LibraryNode;
            FolderNode folderNode = modelNode as FolderNode;
            MyViewsNode myViewsNode = modelNode as MyViewsNode;
            if (libraryNode != null)
            {
                //Update the view name
                view.Caption = _treeView.GetNewNodeName(libraryNode, "New View");
                newViewNode.Text = view.Caption;

                //Update the model
                view.Container = libraryNode.ModelInstance;
                view.ParentFolder = null;
                libraryNode.ModelInstance.Views.Add(view.Caption, view);
                libraryNode.ModelInstance.HasChanges = true;

                //Update the treeview
                libraryNode.Nodes.Add(newViewNode);
                newViewNode.TreeView.SelectedNode = newViewNode;
            }
            else if (folderNode != null)
            {
                //Update the view name
                view.Caption = _treeView.GetNewNodeName(folderNode, "New View");
                newViewNode.Text = view.Caption;

                //Update the model
                view.ParentFolder = folderNode.ModelInstance;
                view.Container = null;
                folderNode.ModelInstance.Views.Add(view.Caption, view);
                if (folderNode.ModelInstance.GetContainerLibrary() != null)
                {
                    folderNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                }

                //Update the treeview
                folderNode.Nodes.Add(newViewNode);
                newViewNode.TreeView.SelectedNode = newViewNode;
            }
            else if (myViewsNode != null)
            {
                //Update the view name
                view.Caption = _treeView.GetNewNodeName(myViewsNode, "New View");
                newViewNode.Text = view.Caption;

                //Update the model
                _modelService.Guidance.UserViews.Views.Add(view.Caption, view);
                view.Container = _modelService.Guidance.UserViews;
                view.ParentFolder = null;

                //Update the treeview
                myViewsNode.Nodes.Add(newViewNode);
                newViewNode.TreeView.SelectedNode = newViewNode;
            }
            else
            {
                throw new NotImplementedException();
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.NewLibrary)]
        public void NewLibrary(object sender, EventArgs e)
        {
            //Update the TreeView
            LibrariesNode librariesNode = _treeView.GetCurrentNode() as LibrariesNode;
            if (librariesNode == null)
            {
                //The current node is not libraries
                throw new NotImplementedException();
            }

            //Update the model.
            Library library = Library.New();
            library.ReadProtection = ReadProtection.ReadWrite;
            library.Caption = _treeView.GetNewNodeName(librariesNode, "New Library");
            library.Name = library.Caption.Replace(" ", string.Empty);
            library.SourceFileInfo = new FileInfo(Path.GetFullPath(Path.Combine(ModelStorageService.LibraryFolder, library.Name + ".xml")));

            //Add the library to the guidance.
            _modelService.Guidance.Libraries.Add(library.Name, library);

            LibraryNode libraryNode = LibraryNode.New();
            libraryNode.ModelInstance = library;
            libraryNode.Text = library.Caption;

            librariesNode.Nodes.Add(libraryNode);

            if (!librariesNode.IsExpanded)
            {
                librariesNode.Expand();
            }

            libraryNode.BeginEdit();
        }

        [CommandHandler(GuidanceExplorer.Commands.SubmitFeedback)]
        public void SubmitFeedback(object sender, EventArgs e)
        {
            WebBrowser browser = new WebBrowser();
            GuidanceExplorerSection section = ConfigurationManager.GetSection("GuidanceExplorer") as GuidanceExplorerSection;
            browser.Navigate(section.FeedbackLink);
        }

        [CommandHandler(GuidanceExplorer.Commands.NewGuidanceType)]
        public void NewGuidanceType(object sender, EventArgs e)
        {
            InputBox input = new InputBox("New Guidance Type", "Please write the name of the new guidance type:", "guidanceType");
            if (input.ShowDialog() == DialogResult.OK)
            {
                //TODO: Check that previous guidance type does not exist with the same name.

                GuidanceType guidanceType = GuidanceType.New(input.UserText.Replace(" ", string.Empty));
                guidanceType.DisplayName = input.UserText;
                guidanceType.ListDisplayName = input.UserText;
                guidanceType.NewGuidanceTypeDisplayName = input.UserText;
                guidanceType.TemplateFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceType.Name + "_template.xml"));

                //Create the base template file
                if (!File.Exists(guidanceType.TemplateFileName))
                {
                    string templateFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, GuidanceType.New("template").TemplateFileName));
                    File.Copy(templateFileName, guidanceType.TemplateFileName, true);
                }

                //Refresh the list view.
                AuthoringToolkitNodeSelected(this, e);
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.RenameLibrary)]
        public void RenameLibrary(object sender, EventArgs e)
        {
            //Get the selected node.
            TreeNode treeNode = _treeView.GetCurrentNode();
            treeNode.BeginEdit();
        }

        [CommandHandler(GuidanceExplorer.Commands.RenameView)]
        public void RenameView(object sender, EventArgs e)
        {
            RenameLibrary(null, null);
        }

        [CommandHandler(GuidanceExplorer.Commands.RenameFolder)]
        public void RenameFolder(object sender, EventArgs e)
        {
            RenameLibrary(null, null);
        }

        [CommandHandler(GuidanceExplorer.Commands.DeleteFolder)]
        public void DeleteFolder(object sender, EventArgs e)
        {
            DeleteCurrentNode();
        }

        [CommandHandler(GuidanceExplorer.Commands.DeleteView)]
        public void DeleteView(object sender, EventArgs e)
        {
            DeleteCurrentNode();
        }

        [CommandHandler(GuidanceExplorer.Commands.DeleteLibrary)]
        public void DeleteLibrary(object sender, EventArgs e)
        {
            DeleteCurrentNode();
        }

        [CommandHandler(GuidanceExplorer.Commands.ExportViewAsLibrary)]
        public void ExportViewAsLibrary(object sender, EventArgs e)
        {
            //Get the selected node.
            TreeNode treeNode = _treeView.GetCurrentNode();
            ViewNode viewNode = treeNode as ViewNode;
            GuidanceTypeNode guidanceTypeNode = treeNode as GuidanceTypeNode;
            if (viewNode == null && guidanceTypeNode == null)
            {
                //This is only for saving views.
                throw new NotImplementedException();
            }
            string fileName = (viewNode != null) ? viewNode.Text : guidanceTypeNode.Text;

            SaveFileDialog saveFile = new SaveFileDialog();
            saveFile.CheckPathExists = true;
            saveFile.DefaultExt = "*.xml";
            saveFile.Filter = Messages.XML_FILES_FILTER;
            saveFile.OverwritePrompt = true;
            saveFile.Title = "Save view as";
            saveFile.ValidateNames = true;
            saveFile.RestoreDirectory = true;
            saveFile.FileName = fileName;

            //Ask the user for the file name.
            if (saveFile.ShowDialog() == DialogResult.OK)
            {
                //If the target file exists, then delete it.
                if (File.Exists(saveFile.FileName))
                {
                    File.Delete(saveFile.FileName);
                }

                if (viewNode != null)
                {
                    //Render the view to the file.
                    _modelService.RenderViewToFileInfo(viewNode.ModelInstance, new FileInfo(saveFile.FileName), _htmlService);
                }
                else if (guidanceTypeNode != null)
                {
                    //Get the library.
                    LibraryNode libraryNode = (LibraryNode)guidanceTypeNode.Parent;

                    //Create the list to display
                    GuidanceItemFileReferenceCollection guidanceItems = libraryNode.ModelInstance.GetGuidanceItemFileReferenceForGuidanceType(guidanceTypeNode.ModelInstance);
                    guidanceItems.Columns = guidanceTypeNode.Columns;

                    //Render the guidancetype list to the file.
                    _modelService.RenderGuidanceTypeItemsToFileInfo(guidanceTypeNode.ModelInstance, guidanceItems, new FileInfo(saveFile.FileName), _htmlService);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.MoveGuidanceItemToView)]
        public void MoveItemToView(object sender, EventArgs e)
        {
            GuidanceItem[] items = _listView.GetSelectedGuidanceItems();

            if (items != null && items.Length > 0)
            {
                LibrariesBrowser browser = new LibrariesBrowser();
                browser.Guidance = _modelService.Guidance;
                browser.NodeTypes = GuidanceNodeTypes.ReadWriteViews;

                if (browser.ShowDialog() == DialogResult.OK)
                {
                    GuidanceExplorer.Model.View selectedView = browser.SelectedView;
                    if (selectedView != null)
                    {
                        foreach (GuidanceItem item in items)
                        {
                            if (!selectedView.Items.ContainsKey(item.Id))
                            {
                                _modelService.AddGuidanceItemToView(selectedView, item);
                            }
                        }

                        _treeView.TreeView.SelectedNode = GetNodeByModelInstance<GuidanceExplorer.Model.View>(selectedView);

                        if (selectedView.GetContainerLibrary() != null)
                        {
                            selectedView.GetContainerLibrary().HasChanges = true;
                        }
                    }
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ExportMyViewsAsLibrary)]
        public void ExportMyViewsAsLibrary(object sender, EventArgs e)
        {
            //Get the selected node.
            TreeNode treeNode = _treeView.GetCurrentNode();
            MyViewsNode myViewsNode = treeNode as MyViewsNode;
            if (myViewsNode == null)
            {
                //This is only for saving views.
                throw new NotImplementedException();
            }

            SaveFileDialog saveFile = new SaveFileDialog();
            saveFile.CheckPathExists = true;
            saveFile.DefaultExt = "*.xml";
            saveFile.Filter = Messages.XML_FILES_FILTER;
            saveFile.OverwritePrompt = true;
            saveFile.Title = "Save My Views as";
            saveFile.ValidateNames = true;
            saveFile.RestoreDirectory = true;
            saveFile.FileName = myViewsNode.Text;

            //Ask the user for the file name.
            if (saveFile.ShowDialog() == DialogResult.OK)
            {
                //If the target file exists, then delete it.
                if (File.Exists(saveFile.FileName))
                {
                    File.Delete(saveFile.FileName);
                }

                //Render the view to the file.
                _modelService.RenderUserViewsAsLibrary(myViewsNode.ModelInstance, new FileInfo(saveFile.FileName), _htmlService);
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ImportLibrary)]
        public void ImportLibrary(object sender, EventArgs e)
        {
            //Prepare the dialog.
            OpenFileDialog openFile = new OpenFileDialog();
            openFile.CheckFileExists = true;
            openFile.CheckPathExists = true;
            openFile.DefaultExt = "*.xml";
            openFile.Filter = Messages.XML_FILES_FILTER;
            openFile.Multiselect = true;
            openFile.RestoreDirectory = true;
            openFile.Title = "Import Library Xml";

            //Ask the user to select the file.
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    foreach (string file in openFile.FileNames)
                    {
                        //Read the library to a file.
                        Library newLibrary = _modelService.ConvertFileInfoToLibrary(new FileInfo(file));

                        //Check if the library exists
                        if (_modelService.Guidance.Libraries.ContainsKey(newLibrary.Name))
                        {
                            MessageBox.Show(Messages.CANNOT_IMPORT_TWICE, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        else
                        {
                            //Copy the library file to the local library folder.
                            DirectoryInfo newLibraryPath = new DirectoryInfo(Path.Combine(Environment.CurrentDirectory, ModelStorageService.LibraryFolder));
                            DirectoryInfo itemsBaseFolder = new DirectoryInfo(Path.Combine(newLibraryPath.FullName, newLibrary.Name));

                            //Check if there is already a file with such name in the current library folder.
                            if (File.Exists(newLibraryPath.FullName))
                            {
                                MessageBox.Show("a file with such name already exists in the current library folder.");
                                return;
                            }

                            //Copy the items to the local library folder.
                            foreach (GuidanceItem guidanceItem in newLibrary.Items.Values)
                            {
                                GuidanceItemFileReference gifr = guidanceItem as GuidanceItemFileReference;
                                if (gifr != null)
                                {
                                    //Calculate the old and new file names for the item.
                                    string oldItemFileName = Path.Combine(newLibrary.SourceFileInfo.DirectoryName, gifr.FileName);
                                    string newItemFileName = Path.Combine(itemsBaseFolder.FullName, Path.GetFileName(gifr.FileName));

                                    //Validate file names.
                                    if (!File.Exists(oldItemFileName))
                                    {
                                        MessageBox.Show(string.Format("The item '{0}' in the library can't be found.", oldItemFileName));
                                        return;
                                    }
                                    //if (File.Exists(newItemFileName))
                                    //{
                                    //    MessageBox.Show(string.Format("The item '{0}' in the source library already exist in the target library.", newItemFileName));
                                    //    return;
                                    //}

                                    //Create the required directories.
                                    if (!Directory.Exists(Path.GetDirectoryName(newItemFileName)))
                                    {
                                        Directory.CreateDirectory(Path.GetDirectoryName(newItemFileName));
                                    }

                                    //Copy the file.
                                    File.Copy(oldItemFileName, newItemFileName, true);

                                    //Adjust the references of the files in the library.
                                    gifr.FileName = newItemFileName;
                                    gifr.Library = newLibrary;
                                    _asyncLoad.ProcessGuidanceItemFile(gifr);
                                }
                                else
                                {
                                    //Strange situation it should not happens.
                                    throw new NotSupportedException();
                                }
                            }

                            //Add the library to the guidance in the model.
                            _modelService.Guidance.Libraries.Add(newLibrary.Name, newLibrary);

                            //Update the UI
                            LibrariesNode librariesNode = _treeView.GetCurrentNode() as LibrariesNode;
                            if (librariesNode != null)
                            {
                                _winformRender.RenderLibraryToLibrariesNode(_treeView.TreeView, _modelService.Guidance, newLibrary, librariesNode);
                            }
                            else
                            {
                                //The current node is not libraries
                                throw new NotImplementedException();
                            }

                            FileInfo newLibraryFileName = new FileInfo(Path.Combine(newLibraryPath.FullName, newLibrary.Caption + ".xml"));
                            //Set the library file to the new location.
                            newLibrary.SourceFileInfo = newLibraryFileName;

                            //Save the file to the new location.
                            _modelService.RenderLibraryToFileInfo(newLibrary, newLibraryFileName);

                        }
                    }
                }
                catch
                {
                    MessageBox.Show(Messages.LIBRARY_FILE_INVALID, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.AddRemoveGuidanceType)]
        public void AddRemoveGuidanceType(object sender, EventArgs e)
        {
            AuthoringToolkitControl authoringTkit = new AuthoringToolkitControl();
            authoringTkit.Text = Messages.ADD_REMOVE_GUIDANCE_TYPE;

            authoringTkit.ShowDialog();
        }

        [CommandHandler(GuidanceExplorer.Commands.ImportGuidanceItemToView)]
        public void ImportItemToView(object sender, EventArgs e)
        {
            //Get the view selected.
            ViewNode viewNode = _treeView.GetCurrentNode() as ViewNode;
            if (viewNode == null)
            {
                //This is only for importing into views.
                throw new NotImplementedException();
            }

            Library library = viewNode.ModelInstance.GetContainerLibrary();

            if (library == null)
            {
                LibrariesBrowser browser = new LibrariesBrowser();
                browser.Guidance = _modelService.Guidance;
                browser.NodeTypes = GuidanceNodeTypes.ReadWriteLibraries;
                browser.Text = Messages.SELECT_LIBRARY;

                if (browser.ShowDialog() == DialogResult.OK)
                {
                    library = browser.SelectedLibrary;
                }
                else
                {
                    return;
                }
            }

            //Prepare the dialog.
            OpenFileDialog openFile = new OpenFileDialog();
            openFile.CheckFileExists = true;
            openFile.CheckPathExists = true;
            openFile.DefaultExt = "*.xml";
            openFile.Filter = Messages.XML_FILES_FILTER;
            openFile.Multiselect = true;
            openFile.RestoreDirectory = true;
            openFile.Title = "Import Item Xml";

            //Ask the user to select the file.
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    //Iterate through the selected files.
                    foreach (string file in openFile.FileNames)
                    {
                        //Create the file reference.
                        GuidanceItemFileReference fileRef = GuidanceItemFileReference.New();
                        fileRef.FileName = file;
                        fileRef.Library = library;

                        _asyncLoad.ProcessGuidanceItemFile(fileRef);

                        if (ValidateImportedItem(fileRef, library, true, file))
                        {
                            //Add the item to the library.
                            _modelService.AddGuidanceItemToLibrary(library, fileRef);

                            //Add the item to the view.
                            _modelService.AddGuidanceItemToView(viewNode.ModelInstance, fileRef);

                            //Refresh guidance type nodes
                            _winformRender.RefreshGuidanceTypeNodes(_winformRender.GetLibraryNodeByLibrary(_treeView, library), fileRef.GuidanceType);

                            //Refresh the list.
                            _listView.ShowContents(viewNode.ModelInstance, viewNode.ModelInstance.Columns);
                        }
                    }

                    ViewNodeSelected(this, new ModelNodeEventArgs<GuidanceExplorer.Model.View>(viewNode));
                }
                catch
                {
                    MessageBox.Show(Messages.ITEM_FILE_INVALID, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ImportItemToFolder)]
        public void ImportItemToFolder(object sender, EventArgs e)
        {
            //Get the view selected.
            FolderNode folderNode = _treeView.GetCurrentNode() as FolderNode;
            if (folderNode == null)
            {
                return;
            }

            Library library = folderNode.ModelInstance.GetContainerLibrary();
            if (library == null)
            {
                //TODO: ask for a library to import, since this view is defined in the MyViews node.
                throw new NotSupportedException();
            }

            //Prepare the dialog.
            OpenFileDialog openFile = new OpenFileDialog();
            openFile.CheckFileExists = true;
            openFile.CheckPathExists = true;
            openFile.DefaultExt = "*.xml";
            openFile.Filter = Messages.XML_FILES_FILTER;
            openFile.Multiselect = true;
            openFile.RestoreDirectory = true;
            openFile.Title = "Import Item Xml.";

            //Ask the user to select the file.
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    //Iterate through the selected files.
                    foreach (string file in openFile.FileNames)
                    {
                        //Create the file reference.
                        GuidanceItemFileReference fileRef = GuidanceItemFileReference.New();
                        fileRef.FileName = file;
                        fileRef.Library = library;

                        _asyncLoad.ProcessGuidanceItemFile(fileRef);

                        //Check if the item already exists in any other library.
                        if (_modelService.Guidance.GetGuidanceItemFileReference(fileRef) != null)
                        {
                            //The item already exists.
                            MessageBox.Show(Messages.ITEM_EXISTS_IN_ANOTHER_LIBRARY, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }

                        //Add the item to the library.
                        _modelService.AddGuidanceItemToLibrary(library, fileRef);

                        //Check if the item already exists in any other library.
                        if (_modelService.Guidance.GetGuidanceItemFileReference(fileRef) != null)
                        {
                            //The item already exists.
                            MessageBox.Show(Messages.ITEM_EXISTS_IN_ANOTHER_LIBRARY, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }

                        //Add the item to the library.
                        _modelService.AddGuidanceItemToLibrary(library, fileRef);

                        //Refresh the list.
                        _listView.ShowContents(folderNode.ModelInstance, folderNode.ModelInstance.Columns);
                    }
                }
                catch
                {
                    MessageBox.Show(Messages.ITEM_FILE_INVALID, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ImportGuidanceItemToLibrary)]
        public void ImportItemToLibrary(object sender, EventArgs e)
        {
            //Get the view selected.
            LibraryNode libraryNode = _treeView.GetCurrentNode() as LibraryNode;
            GuidanceTypeNode typeNode = _treeView.GetCurrentNode() as GuidanceTypeNode;
            if (libraryNode == null)
            {
                //Check if the selected node is a type node
                if (typeNode != null)
                {
                    libraryNode = typeNode.Parent as LibraryNode;
                }
                else
                {
                    //current node is invalid
                    throw new NotSupportedException();
                }
            }

            Library library = libraryNode.ModelInstance;
            if (library == null)
            {
                //TODO: ask for a library to import, since this folder is defined in the MyViews node.
                throw new NotSupportedException();
            }

            //Prepare the dialog.
            OpenFileDialog openFile = new OpenFileDialog();
            openFile.CheckFileExists = true;
            openFile.CheckPathExists = true;
            openFile.DefaultExt = "*.xml";
            openFile.Filter = Messages.XML_FILES_FILTER;
            openFile.Multiselect = true;
            openFile.RestoreDirectory = true;
            openFile.Title = "Import Item Xml.";

            //Ask the user to select the file.
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    bool warningShown = false;
                    //Iterate through the selected files.
                    foreach (string file in openFile.FileNames)
                    {
                        //Create the file reference.
                        GuidanceItemFileReference fileRef = GuidanceItemFileReference.New();
                        fileRef.FileName = file;
                        fileRef.Library = library;

                        _asyncLoad.ProcessGuidanceItemFile(fileRef);

                        if (ValidateImportedItem(fileRef, library, !warningShown, file))
                        {
                            library.HasChanges = true;

                            //Add the item to the library.
                            _modelService.AddGuidanceItemToLibrary(library, fileRef);

                            //Refresh type nodes
                            _winformRender.RefreshGuidanceTypeNodes(libraryNode, fileRef.GuidanceType);

                            //Refresh the list.
                            if (typeNode != null)
                            {
                                GuidanceTypeNodeSelected(this, new ModelNodeEventArgs<GuidanceType>(typeNode));
                            }
                            else
                            {
                                LibraryNodeSelected(this, new ModelNodeEventArgs<Library>(libraryNode));
                            }
                        }
                        else
                        {
                            warningShown = true;
                        }
                    }
                }
                catch
                {
                    MessageBox.Show(Messages.ITEM_FILE_INVALID, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private bool ValidateImportedItem(GuidanceItemFileReference guidanceItem, Library selectedLibrary, bool showMessage, string sourceFile)
        {
            //Checks if the id exists in any the libraries
            if (_modelService.Guidance.Items.ContainsKey(guidanceItem.Id))
            {
                //Checks is the id exists in the selected library
                if (selectedLibrary.Items.ContainsKey(guidanceItem.Id))
                {
                    //Show error message
                    if (showMessage)
                    {
                        if (MessageBox.Show(Messages.ITEM_CANNOT_BE_IMPORTED_ID, "Error", MessageBoxButtons.OKCancel, MessageBoxIcon.Error) == DialogResult.OK)
                        {
                            //Replace the item.

                            //Copy the file to the libraries folder path
                            string newFilePath =
                                Path.Combine(
                                    Path.Combine(
                                        Path.Combine(ModelStorageService.LibraryFolder, selectedLibrary.Caption),
                                        guidanceItem.GuidanceType.DisplayName),
                                    guidanceItem.Id + ".xml");

                            if (!Directory.Exists(Path.GetDirectoryName(newFilePath)))
                            {
                                Directory.CreateDirectory(Path.GetDirectoryName(newFilePath));
                            }

                            File.Copy(sourceFile, newFilePath, true);

                            guidanceItem.FileName = newFilePath;

                            //Save changes
                            using (StreamReader reader = new StreamReader(_htmlService.GetReadOnlyDocumentStream(guidanceItem)))
                            {
                                _modelService.RenderGuidanceItemToFileInfo(guidanceItem, new FileInfo(guidanceItem.FileName), reader.ReadToEnd());
                            }

                            selectedLibrary.Items[guidanceItem.Id] = guidanceItem;
                        }
                    }
                    return false;
                }
                else
                {
                    //Copy the file to the libraries folder path
                    string newFilePath =
                        Path.Combine(
                            Path.Combine(
                                Path.Combine(ModelStorageService.LibraryFolder, selectedLibrary.Caption),
                                guidanceItem.GuidanceType.DisplayName),
                            guidanceItem.Id + ".xml");

                    if (!Directory.Exists(Path.GetDirectoryName(newFilePath)))
                    {
                        Directory.CreateDirectory(Path.GetDirectoryName(newFilePath));
                    }

                    File.Copy(sourceFile, newFilePath, true);

                    guidanceItem.FileName = newFilePath;

                    //Save changes
                    using (StreamReader reader = new StreamReader(_htmlService.GetReadOnlyDocumentStream(guidanceItem)))
                    {
                        _modelService.RenderGuidanceItemToFileInfo(guidanceItem, new FileInfo(guidanceItem.FileName), reader.ReadToEnd());
                    }
                    return true;
                }
            }
            else
            {

                //Copy the file to the libraries folder path
                string newFilePath = Path.Combine(Path.Combine(Path.Combine(ModelStorageService.LibraryFolder, selectedLibrary.Caption),
                    guidanceItem.GuidanceType.DisplayName), guidanceItem.Id + ".xml");

                if (!Directory.Exists(Path.GetDirectoryName(newFilePath)))
                {
                    Directory.CreateDirectory(Path.GetDirectoryName(newFilePath));
                }

                File.Copy(sourceFile, newFilePath, true);

                guidanceItem.FileName = newFilePath;

                return true;
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.SaveItemAsXml)]
        public void SaveItemAsXml(object sender, EventArgs e)
        {
            GuidanceItem[] guidanceItems = _listView.GetSelectedGuidanceItems();
            if (guidanceItems == null || guidanceItems.Length == 0)
            {
                MessageBox.Show("Please select an item to export", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (guidanceItems.Length == 1)
            {
                GuidanceItemFileReference fileRef = _modelService.Guidance.GetGuidanceItemFileReference(guidanceItems[0]);
                if (fileRef != null)
                {
                    //If the source file does not exists, show an error.
                    if (!File.Exists(fileRef.FileName))
                    {
                        throw new NotSupportedException();
                    }

                    //Request user filename
                    SaveFileDialog saveFile = new SaveFileDialog();
                    saveFile.CheckPathExists = true;
                    saveFile.DefaultExt = "*.xml";
                    saveFile.Filter = Messages.XML_FILES_FILTER;
                    saveFile.OverwritePrompt = true;
                    saveFile.Title = "Save item as";
                    saveFile.ValidateNames = true;
                    saveFile.RestoreDirectory = true;
                    saveFile.FileName = Path.GetFileName(fileRef.FileName);

                    //Ask the user for the file name.
                    if (saveFile.ShowDialog() == DialogResult.OK)
                    {
                        //If the target file exists, then delete it.
                        if (File.Exists(saveFile.FileName))
                        {
                            File.Delete(saveFile.FileName);
                        }

                        //Copy the original file to the new location.
                        //File.Copy(fileRef.FileName, saveFile.FileName);
                        using (StreamReader reader = new StreamReader(_htmlService.GetReadOnlyDocumentStream(fileRef)))
                        {
                            _modelService.RenderGuidanceItemToFileInfo(fileRef, new FileInfo(saveFile.FileName), reader.ReadToEnd());
                        }
                    }
                }
            }
            else
            {
                FolderBrowserDialog folderDialog = new FolderBrowserDialog();
                folderDialog.ShowNewFolderButton = true;
                folderDialog.Description = Messages.SELECT_FOLDER_TO_EXPORT_ITEMS;

                if (folderDialog.ShowDialog() == DialogResult.OK)
                {
                    foreach (GuidanceItem item in guidanceItems)
                    {
                        GuidanceItemFileReference fileRef = _modelService.Guidance.GetGuidanceItemFileReference(item);
                        if (fileRef != null && File.Exists(fileRef.FileName))
                        {
                            string newFileName = Path.Combine(folderDialog.SelectedPath,
                                Path.GetFileName(fileRef.FileName));

                            //Copy the original file to the new location.
                            //File.Copy(fileRef.FileName, newFileName, true);
                            using (StreamReader reader = new StreamReader(_htmlService.GetReadOnlyDocumentStream(fileRef)))
                            {
                                _modelService.RenderGuidanceItemToFileInfo(fileRef, new FileInfo(newFileName), reader.ReadToEnd());
                            }
                        }
                    }
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ExportFolderAsLibrary)]
        public void ExportFolderAsLibrary(object sender, EventArgs e)
        {
            //Get the selected node.
            TreeNode treeNode = _treeView.GetCurrentNode();
            FolderNode folderNode = treeNode as FolderNode;
            if (folderNode == null)
            {
                //This is only for saving folders.
                throw new NotImplementedException();
            }

            //Request user save file.
            SaveFileDialog saveFile = new SaveFileDialog();
            saveFile.CheckPathExists = true;
            saveFile.DefaultExt = "*.xml";
            saveFile.Filter = Messages.XML_FILES_FILTER;
            saveFile.OverwritePrompt = true;
            saveFile.Title = "Save folder as";
            saveFile.ValidateNames = true;
            saveFile.RestoreDirectory = true;
            saveFile.FileName = folderNode.Text;

            //Ask the user for the file name.
            if (saveFile.ShowDialog() == DialogResult.OK)
            {
                //If the target file exists, then delete it.
                if (File.Exists(saveFile.FileName))
                {
                    File.Delete(saveFile.FileName);
                }

                //Render the folder to the file.
                _modelService.RenderFolderToFileInfo(folderNode.ModelInstance, new FileInfo(saveFile.FileName), _htmlService);
            }
        }


        [CommandHandler(GuidanceExplorer.Commands.UndoDragAndDrop)]
        public void UndoDragAndDrop(object sender, EventArgs e)
        {
            _treeView.UndoDragDrop();
        }

        [CommandHandler(GuidanceExplorer.Commands.ResetColumns)]
        public void ResetColumns(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            ViewNode viewNode = treeNode as ViewNode;
            SearchViewNode searchViewNode = treeNode as SearchViewNode;
            FolderNode folderNode = treeNode as FolderNode;
            LibraryNode libraryNode = treeNode as LibraryNode;
            GuidanceTypeNode typeNode = treeNode as GuidanceTypeNode;
            LibrariesNode librariesNode = treeNode as LibrariesNode;
            MyViewsNode myViewsNode = treeNode as MyViewsNode;
            if (viewNode != null)
            {
                ResetColumns(viewNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Position | ResetOptions.Visibility);
                _listView.ShowContents(viewNode.ModelInstance, viewNode.ModelInstance.Columns);
            }
            else if (searchViewNode != null)
            {
                ResetColumns(searchViewNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Position | ResetOptions.Visibility);
                _listView.ShowContents(searchViewNode.ModelInstance, searchViewNode.ModelInstance.Columns);
            }
            else if (folderNode != null)
            {
                ResetColumns(folderNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Position | ResetOptions.Visibility);
                _listView.ShowContents(folderNode.ModelInstance, folderNode.ModelInstance.Columns);
            }
            else if (libraryNode != null)
            {
                ResetColumns(libraryNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Position | ResetOptions.Visibility);
                _listView.ShowContents(libraryNode.ModelInstance, libraryNode.ModelInstance.Columns);
            }
            else if (typeNode != null)
            {
                ColumnCollection columns = ColumnCollection.DefaultColumns(typeNode.ModelInstance);
                foreach (Column col in columns.Values)
                {
                    col.Visible = col.Field.ShowInGuidanceTypeNode;
                }
                ResetColumns(typeNode.Columns, columns, ResetOptions.Position | ResetOptions.Visibility);
                GuidanceTypeNodeSelected(sender, new ModelNodeEventArgs<GuidanceType>(typeNode));
            }
            else if (librariesNode != null)
            {
                ResetColumns(_modelService.Guidance.Columns, _modelService.Guidance.DefaultColumns, ResetOptions.Position | ResetOptions.Visibility);
                LibrariesNodeSelected(this, null);
            }
            else if (myViewsNode != null)
            {
                ResetColumns(myViewsNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Position | ResetOptions.Visibility);
                MyViewsSelected(this, new ModelNodeEventArgs<UserViews>(myViewsNode));
            }
            else
            {
                // Any other tree node.
                throw new NotImplementedException();
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ResetFilters)]
        public void ResetFilters(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            ViewNode viewNode = treeNode as ViewNode;
            SearchViewNode searchViewNode = treeNode as SearchViewNode;
            FolderNode folderNode = treeNode as FolderNode;
            LibraryNode libraryNode = treeNode as LibraryNode;
            GuidanceTypeNode typeNode = treeNode as GuidanceTypeNode;
            LibrariesNode librariesNode = treeNode as LibrariesNode;
            MyViewsNode myViewsNode = treeNode as MyViewsNode;
            if (viewNode != null)
            {
                foreach (Column column in viewNode.ModelInstance.Columns.Values)
                {
                    column.AutoFilter = null;
                }
                _listView.ShowContents(viewNode.ModelInstance, viewNode.ModelInstance.Columns);
            }
            else if (searchViewNode != null)
            {
                foreach (Column column in searchViewNode.ModelInstance.Columns.Values)
                {
                    column.AutoFilter = null;
                }
                _listView.ShowContents(searchViewNode.ModelInstance, searchViewNode.ModelInstance.Columns);
            }
            else if (folderNode != null)
            {
                foreach (Column column in folderNode.ModelInstance.Columns.Values)
                {
                    column.AutoFilter = null;
                }
                _listView.ShowContents(folderNode.ModelInstance, folderNode.ModelInstance.Columns);
            }
            else if (libraryNode != null)
            {
                foreach (Column column in libraryNode.ModelInstance.Columns.Values)
                {
                    column.AutoFilter = null;
                }
                _listView.ShowContents(libraryNode.ModelInstance, libraryNode.ModelInstance.Columns);
            }
            else if (typeNode != null)
            {
                foreach (Column column in typeNode.Columns.Values)
                {
                    column.AutoFilter = null;
                }
                GuidanceTypeNodeSelected(sender, new ModelNodeEventArgs<GuidanceType>(typeNode));
            }
            else if (librariesNode != null)
            {
                foreach (Column column in _modelService.Guidance.Columns.Values)
                {
                    column.AutoFilter = null;
                }

                LibrariesNodeSelected(this, null);
            }
            else if (myViewsNode != null)
            {
                foreach (Column column in myViewsNode.ModelInstance.Columns.Values)
                {
                    column.AutoFilter = null;
                }

                MyViewsSelected(null, new ModelNodeEventArgs<UserViews>(myViewsNode));
            }
            else
            {
                // Any other tree node.
                throw new NotImplementedException();
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ResetSortOrder)]
        public void ResetSortOrder(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            ViewNode viewNode = treeNode as ViewNode;
            SearchViewNode searchViewNode = treeNode as SearchViewNode;
            FolderNode folderNode = treeNode as FolderNode;
            LibraryNode libraryNode = treeNode as LibraryNode;
            GuidanceTypeNode typeNode = treeNode as GuidanceTypeNode;
            LibrariesNode librariesNode = treeNode as LibrariesNode;
            MyViewsNode myViewsNode = treeNode as MyViewsNode;
            if (viewNode != null)
            {
                ResetColumns(viewNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);
                _listView.ShowContents(viewNode.ModelInstance, viewNode.ModelInstance.Columns);
            }
            else if (searchViewNode != null)
            {
                ResetColumns(searchViewNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);
                _listView.ShowContents(searchViewNode.ModelInstance, searchViewNode.ModelInstance.Columns);
            }
            else if (folderNode != null)
            {
                ResetColumns(folderNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);
                _listView.ShowContents(folderNode.ModelInstance, folderNode.ModelInstance.Columns);
            }
            else if (libraryNode != null)
            {
                ResetColumns(libraryNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);
                _listView.ShowContents(libraryNode.ModelInstance, libraryNode.ModelInstance.Columns);
            }
            else if (typeNode != null)
            {
                ResetColumns(typeNode.Columns, ColumnCollection.DefaultColumns(typeNode.ModelInstance), ResetOptions.Sort);
                GuidanceTypeNodeSelected(sender, new ModelNodeEventArgs<GuidanceType>(typeNode));
            }
            else if (librariesNode != null)
            {
                ResetColumns(_modelService.Guidance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);
                LibrariesNodeSelected(this, null);
            }
            else if (myViewsNode != null)
            {
                ResetColumns(myViewsNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);
                MyViewsSelected(null, new ModelNodeEventArgs<UserViews>(myViewsNode));
            }
            else
            {
                // Any other tree node.
                throw new NotImplementedException();
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.DeleteGuidanceItem)]
        public void DeleteGuidanceItem(object sender, EventArgs e)
        {
            GuidanceItem[] guidanceItems = _listView.GetSelectedGuidanceItems();
            if (guidanceItems == null)
            {
                MessageBox.Show(Messages.SELECTED_ITEM_NOT_FOUND, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            GuidanceItemCollection items = GuidanceItemCollection.New();

            foreach (GuidanceItem item in guidanceItems)
            {
                items.Add(item.Id, item);
            }

            GuidanceItemsDeleted(this, new GuidanceItemCollectionEventArgs(items));
        }

        [CommandHandler(GuidanceExplorer.Commands.HelpAbout)]
        public void HelpAbout(object sender, EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }

        [CommandHandler(GuidanceExplorer.Commands.Synchronize)]
        public void Synchronize(object sender, EventArgs e)
        {
            BackgroundWorker bw = new BackgroundWorker();
            bw.DoWork += delegate(object s, DoWorkEventArgs eventArgs)
            {
                //Synchronize the local libraries
                _onlineStorage.Syncronize(_modelService, _htmlService, true, eventArgs);
            };
            bw.RunWorkerCompleted += delegate(object s, RunWorkerCompletedEventArgs ev)
            {
                if (!ev.Cancelled)
                {
                    MessageBox.Show("Synchronization completed", "Online Storage");
                }

                string selectedNode = GetNodePath(_treeView.TreeView.SelectedNode);
                string selectedItem = null;
                GuidanceItem[] items = _listView.GetSelectedGuidanceItems();

                if (items != null && items.Length > 0)
                {
                    selectedItem = items[0].Id;
                }


                //Repaint the guidance in the tree view control
                foreach (Library subscribedLibrary in _modelService.Guidance.Libraries.Values)
                {
                    if (subscribedLibrary.WebSuscribed)
                    {
                        //Remove the previous library node.
                        LibraryNode libNode = _winformRender.GetLibraryNodeByLibrary(_treeView, subscribedLibrary);
                        libNode.Remove();

                        //Add the new one.
                        _winformRender.RenderLibraryToLibrariesNode(_treeView.TreeView, _modelService.Guidance, subscribedLibrary, _winformRender.GetLibrariesNode(_treeView));
                    }
                }

                SelectNode(selectedNode);

                if (selectedItem != null)
                {
                    _listView.SelectItem(selectedItem);
                }
            };
            bw.RunWorkerAsync();
        }

        private void SelectNode(string path)
        {
            string[] nodes = path.Split(new string[] { "/" }, StringSplitOptions.RemoveEmptyEntries);
            TreeNode currentNode = GetNode(_treeView.TreeView.Nodes, nodes[0]);

            if (currentNode != null)
            {
                for (int i = 1; i < nodes.Length && ContainsNode(currentNode.Nodes, nodes[i]); i++)
                {
                    currentNode = GetNode(currentNode.Nodes, nodes[i]);
                }

                SelectNode(currentNode);
            }
        }

        private static bool ContainsNode(TreeNodeCollection nodes, string nodeText)
        {
            foreach (TreeNode node in nodes)
            {
                if (string.Compare(node.Text, nodeText) == 0)
                {
                    return true;
                }
            }

            return false;
        }

        private static TreeNode GetNode(TreeNodeCollection nodes, string nodeText)
        {
            foreach (TreeNode node in nodes)
            {
                if (string.Compare(node.Text, nodeText) == 0)
                {
                    return node;
                }
            }

            return null;
        }

        private static string GetNodePath(TreeNode node)
        {
            string path = string.Empty;
            while (node != null)
            {
                path = node.Text + "/" + path;
                node = node.Parent;
            }

            return path;
        }

        [CommandHandler(GuidanceExplorer.Commands.SuscribeLibraryFromWeb)]
        public void SuscribeLibraryFromWeb(object sender, EventArgs e)
        {
            LibrariesNode librariesNode = _winformRender.GetLibrariesNode(_treeView);
            if (librariesNode == null)
            {
                //Executed from an unexpected context.
                throw new NotSupportedException();
            }

            //Suscribe to the online storage.
            LibraryCollection onlineLibraries = null;

            try
            {
                onlineLibraries = _onlineStorage.GetAvailableLibraries(AvailableLibrariesMode.DoNotAllowCreation);
            }
            catch { }

            if (onlineLibraries == null)
            {
                MessageBox.Show("No online store configured", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                //Display the list of libraries to select the library to subscribe.
                LibrariesBrowser browser = new LibrariesBrowser();
                browser.Text = "Select an online library";
                browser.NodeTypes = GuidanceNodeTypes.All;
                browser.Libraries = onlineLibraries;
                if (browser.ShowDialog() == DialogResult.OK)
                {
                    Library suscribedLibrary = browser.SelectedLibrary;

                    //Check if the library is already suscribed.
                    if (_modelService.Guidance.Libraries.ContainsKey(suscribedLibrary.Name))
                    {
                        MessageBox.Show("The library is already suscribed.", "Online Storage");
                        return;
                    }

                    ReadProtection oldReadProtection = suscribedLibrary.ReadProtection;
                    BackgroundWorker bw = new BackgroundWorker();
                    bw.DoWork += delegate(object s, DoWorkEventArgs eventArgs)
                    {
                        //Keep the read protection
                        suscribedLibrary.ReadProtection = ReadProtection.ReadOnly;

                        //Add the library to the guidance.
                        _modelService.Guidance.Libraries.Add(suscribedLibrary.Name, suscribedLibrary);

                        //Add the guidance items to the library and create the local files.
                        _onlineStorage.AddGuidanceItemsFileReferences(_modelService, suscribedLibrary);
                    };
                    bw.RunWorkerCompleted += delegate(object s, RunWorkerCompletedEventArgs eventArgs)
                    {
                        //Add the library to the tree view.
                        _winformRender.RenderLibraryToLibrariesNode(_treeView.TreeView, _modelService.Guidance, suscribedLibrary, librariesNode);

                        //Re set the read protection.
                        suscribedLibrary.ReadProtection = oldReadProtection;
                    };
                    bw.RunWorkerAsync();
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.SuscribeLibraryFromRss)]
        public void SuscribeLibraryFromRss(object sender, EventArgs e)
        {
            IWorkspace workspace = RootWorkItem.Workspaces.Get<WindowWorkspace>("SuscribeFromRss");
            if (workspace != null)
            {
                RootWorkItem.Workspaces.Remove(workspace);
            }

            SuscribeToRss suscribeSmartPart = RootWorkItem.SmartParts.Get<SuscribeToRss>("SuscribeToRss");
            if (suscribeSmartPart != null)
            {
                RootWorkItem.SmartParts.Remove(suscribeSmartPart);
            }

            workspace = new WindowWorkspace(new Form());
            RootWorkItem.Workspaces.Add(workspace, "SuscribeFromRss");
            suscribeSmartPart = new SuscribeToRss();
            RootWorkItem.SmartParts.Add(suscribeSmartPart, "SuscribeToRss");
            WindowSmartPartInfo info = new WindowSmartPartInfo();
            
            workspace.Show(suscribeSmartPart, info);
            suscribeSmartPart.Dock = DockStyle.Fill;

        }

        [CommandHandler(GuidanceExplorer.Commands.PublishLibrary)]
        public void PublishLibrary(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            if (modelNode == null)
            {
                //It will only work for views
                throw new NotSupportedException();
            }

            LibraryNode libraryNode = modelNode as LibraryNode;
            if (libraryNode == null)
            {
                //It will only work for views
                throw new NotSupportedException();
            }

            if (libraryNode.ModelInstance.FileSuscribed)
            {
                MessageBox.Show("A file suscribed library can't be published.");
                return;
            }

            //if (libraryNode.ModelInstance.WebSuscribed)
            //{
            //    MessageBox.Show("A library is already published, the synchronization proces will control this library.");
            //    return;
            //}

            try
            {
                //Add the library to the online storage.
                _onlineStorage.CreateLibrary(libraryNode.ModelInstance);

                //Publish the selected guidance items to the library
                foreach (GuidanceItem gi in libraryNode.ModelInstance)
                {
                    GuidanceItemFileReference gifr = gi as GuidanceItemFileReference;
                    if (gifr != null)
                    {
                        //Add the item to the library.
                        using (StreamReader reader = new StreamReader(_htmlService.GetRawDocumentStream(gifr)))
                        {
                            _onlineStorage.PublishGuidanceItem(libraryNode.ModelInstance, gifr, reader.ReadToEnd(), GuidanceItemOperation.Create, false);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.PublishGuidanceType)]
        public void PublishGuidanceType(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            if (modelNode == null)
            {
                //It will only work for views
                throw new NotSupportedException();
            }

            GuidanceTypeNode guidanceTypeNode = modelNode as GuidanceTypeNode;
            if (guidanceTypeNode == null)
            {
                //It will only work for views
                throw new NotSupportedException();
            }

            //Get the parent library.
            LibraryNode libraryNode = guidanceTypeNode.Parent as LibraryNode;

            //Get the items for this guidance type.
            GuidanceItemFileReferenceCollection guidanceItems = libraryNode.ModelInstance.GetGuidanceItemFileReferenceForGuidanceType(guidanceTypeNode.ModelInstance);

            //Get the available libraries in the online storage.
            LibraryCollection onlineLibraries = _onlineStorage.GetAvailableLibraries(AvailableLibrariesMode.AllowCreation);

            //Display the list of libraries to select the library to subscribe.
            LibrariesBrowser browser = new LibrariesBrowser();
            browser.NodeTypes = GuidanceNodeTypes.All;
            browser.Libraries = onlineLibraries;
            if (browser.ShowDialog() == DialogResult.OK)
            {
                //Get the selected library.
                Library suscribedLibrary = browser.SelectedLibrary;

                //If the user will create a new library.
                if (suscribedLibrary.Name == GuidanceExplorer.Commands.NewLibrary)
                {
                    InputBox inputBox = new InputBox("New library", "Write the new library name", string.Empty);
                    if (inputBox.ShowDialog() == DialogResult.OK)
                    {
                        //Set the library properties
                        suscribedLibrary.Name = inputBox.UserText;
                        suscribedLibrary.Caption = inputBox.UserText;
                        suscribedLibrary.SourceFileInfo = new FileInfo(Path.GetFullPath(Path.Combine(ModelStorageService.LibraryFolder, suscribedLibrary.Name + ".xml")));

                        //Create the library in the online storage.
                        _onlineStorage.CreateLibrary(suscribedLibrary);
                    }
                }

                //Publish the selected guidance items
                foreach (GuidanceItemFileReference gifr in guidanceItems)
                {
                    //Add the item to the library.
                    using (StreamReader reader = new StreamReader(_htmlService.GetRawDocumentStream(gifr)))
                    {
                        _onlineStorage.PublishGuidanceItem(suscribedLibrary, gifr, reader.ReadToEnd(), GuidanceItemOperation.Create, false);
                    }
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.RemoveView)]
        public void RemoveView(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            if (modelNode == null)
            {
                //It will only work for views
                throw new NotSupportedException();
            }

            ViewNode viewNode = modelNode as ViewNode;
            if (viewNode == null)
            {
                //It will only work for views
                throw new NotSupportedException();
            }

            //Add the item to the view.
            _onlineStorage.RemoveView(viewNode.ModelInstance);
        }

        [CommandHandler(GuidanceExplorer.Commands.PublishView)]
        public void PublishView(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            if (modelNode == null)
            {
                //It will only work for views
                throw new NotSupportedException();
            }

            ViewNode viewNode = modelNode as ViewNode;
            if (viewNode == null)
            {
                //It will only work for views
                throw new NotSupportedException();
            }

            GuidanceItemFileReferenceCollection gifrs = GuidanceItemFileReferenceCollection.New();
            foreach (GuidanceItem guidanceItem in viewNode.ModelInstance.Items)
            {
                GuidanceItemFileReference gifr = guidanceItem as GuidanceItemFileReference;
                if (gifr != null)
                {
                    gifrs.Add(gifr.Id, gifr);
                }
                else if (guidanceItem.ReferencedItem != null)
                {
                    gifrs.Add(guidanceItem.ReferencedItem.Id, guidanceItem.ReferencedItem);
                }
                else
                {
                    //There must be a gifr for each item
                    throw new NotSupportedException();
                }
            }

            //Get the available libraries in the online storage.
            LibraryCollection onlineLibraries = _onlineStorage.GetAvailableLibraries(AvailableLibrariesMode.AllowCreation);

            //Display the list of libraries to select the library to subscribe.
            LibrariesBrowser browser = new LibrariesBrowser();
            browser.NodeTypes = GuidanceNodeTypes.All;
            browser.Libraries = onlineLibraries;
            if (browser.ShowDialog() == DialogResult.OK)
            {
                //Get the selected library.
                Library suscribedLibrary = browser.SelectedLibrary;

                //If the user will create a new library.
                if (suscribedLibrary.Name == GuidanceExplorer.Commands.NewLibrary)
                {
                    InputBox inputBox = new InputBox("New library", "Write the new library name", string.Empty);
                    if (inputBox.ShowDialog() == DialogResult.OK)
                    {
                        //Set the library properties
                        suscribedLibrary.Name = inputBox.UserText;
                        suscribedLibrary.Caption = inputBox.UserText;
                        suscribedLibrary.SourceFileInfo = new FileInfo(Path.GetFullPath(Path.Combine(ModelStorageService.LibraryFolder, suscribedLibrary.Name + ".xml")));

                        //Create the library in the online storage.
                        _onlineStorage.CreateLibrary(suscribedLibrary);
                    }
                }

                //Create the view.
                string viewId = _onlineStorage.CreateOnlineView(suscribedLibrary, viewNode);

                //Publish the selected guidance items
                foreach (GuidanceItemFileReference gifr in gifrs)
                {
                    //Add the item to the library.
                    using (StreamReader reader = new StreamReader(_htmlService.GetRawDocumentStream(gifr)))
                    {
                        _onlineStorage.PublishGuidanceItem(suscribedLibrary, gifr, reader.ReadToEnd(), GuidanceItemOperation.Create, true);
                    }
                }

                //Add the item to the view.
                _onlineStorage.AddGuidanceItemToView(new Guid(viewId), gifrs);
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.PublishGuidanceItemInView)]
        public void PublishGuidanceItemInView(object sender, EventArgs e)
        {
            IModelItem modelNode = _treeView.GetCurrentNode() as IModelItem;
            ViewNode viewNode = modelNode as ViewNode;
            if (viewNode == null)
            {
                MessageBox.Show("Current view can't be detected.");
                return;
            }

            GuidanceItemCollection guidanceItems = GuidanceItemCollection.New();
            foreach (GuidanceItem guidanceItem in _listView.GetSelectedGuidanceItems())
            {
                guidanceItems.Add(guidanceItem.Id, guidanceItem);
            }

            //If there is no selected items
            if (guidanceItems.Count == 0)
            {
                GuidanceItem guidanceItem = _listView.GetCurrentGuidanceItem();
                if (guidanceItem != null)
                {
                    guidanceItems.Add(guidanceItem.Id, guidanceItem);
                }
            }

            //Obtain the GuidanceItemFileReference for each guidance item.
            GuidanceItemFileReferenceCollection gifrs = GuidanceItemFileReferenceCollection.New();
            foreach (GuidanceItem guidanceItem in guidanceItems)
            {
                GuidanceItemFileReference gifr = guidanceItem as GuidanceItemFileReference;
                if (gifr == null)
                {
                    gifr = guidanceItem.ReferencedItem;
                    if (gifr == null)
                    {
                        //This should not happen.
                        throw new NotImplementedException();
                    }
                }
                gifrs.Add(gifr.Id, gifr);
            }

            //Get the id for the view.
            string viewId = viewNode.ModelInstance.Id;

            //Add the guidance items to the view.
            _onlineStorage.AddGuidanceItemToView(new Guid(viewId), gifrs);
        }

        [CommandHandler(GuidanceExplorer.Commands.PublishGuidanceItem)]
        public void PublishGuidanceItem(object sender, EventArgs e)
        {
            GuidanceItemCollection guidanceItems = GuidanceItemCollection.New();
            foreach (GuidanceItem guidanceItem in _listView.GetSelectedGuidanceItems())
            {
                guidanceItems.Add(guidanceItem.Id, guidanceItem);
            }

            //If there is no selected items
            if (guidanceItems.Count == 0)
            {
                GuidanceItem guidanceItem = _listView.GetCurrentGuidanceItem();
                if (guidanceItem != null)
                {
                    guidanceItems.Add(guidanceItem.Id, guidanceItem);
                }
            }

            //If there is no current item.
            if (guidanceItems.Count == 0)
            {
                MessageBox.Show(Messages.NO_SELECTED_ITEM, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            //Obtain the GuidanceItemFileReference for each guidance item.
            GuidanceItemFileReferenceCollection gifrs = GuidanceItemFileReferenceCollection.New();
            foreach (GuidanceItem guidanceItem in guidanceItems)
            {
                GuidanceItemFileReference gifr = guidanceItem as GuidanceItemFileReference;
                if (gifr == null)
                {
                    gifr = guidanceItem.ReferencedItem;
                    if (gifr == null)
                    {
                        //This should not happen.
                        throw new NotImplementedException();
                    }
                }
                gifrs.Add(gifr.Id, gifr);
            }

            //Suscribe to the online storage.
            LibraryCollection onlineLibraries = _onlineStorage.GetAvailableLibraries(AvailableLibrariesMode.AllowCreation);

            //Display the list of libraries to select the library to subscribe.
            LibrariesBrowser browser = new LibrariesBrowser();
            browser.NodeTypes = GuidanceNodeTypes.ReadOnlyLibraries;
            browser.Libraries = onlineLibraries;
            if (browser.ShowDialog() == DialogResult.OK)
            {
                //Get the selected library.
                Library suscribedLibrary = browser.SelectedLibrary;

                //If the user will create a new library.
                if (suscribedLibrary.Name == GuidanceExplorer.Commands.NewLibrary)
                {
                    InputBox inputBox = new InputBox("New library", "Write the new library name", string.Empty);
                    if (inputBox.ShowDialog() == DialogResult.OK)
                    {
                        //Set the library properties
                        suscribedLibrary.Name = inputBox.UserText;
                        suscribedLibrary.Caption = inputBox.UserText;
                        suscribedLibrary.SourceFileInfo = new FileInfo(Path.GetFullPath(Path.Combine(ModelStorageService.LibraryFolder, suscribedLibrary.Name + ".xml")));

                        //Create the library in the online storage.
                        _onlineStorage.CreateLibrary(suscribedLibrary);
                    }
                }

                //Publish the selected guidance items
                foreach (GuidanceItemFileReference gifr in gifrs)
                {
                    using (StreamReader reader = new StreamReader(_htmlService.GetRawDocumentStream(gifr)))
                    {
                        _onlineStorage.PublishGuidanceItem(suscribedLibrary, gifr, reader.ReadToEnd(), GuidanceItemOperation.Create, false);
                    }
                }

                //If a view was selected, then add the item to the view
                if (browser.SelectedView != null)
                {
                    //Get the id for the view.
                    string viewId = _onlineStorage.GetViewIdByCaption(suscribedLibrary, browser.SelectedView.Caption);

                    //Add the guidance items to the view.
                    _onlineStorage.AddGuidanceItemToView(new Guid(viewId), gifrs);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.RemoveGuidanceItem)]
        public void RemoveGuidanceItem(object sender, EventArgs e)
        {
            GuidanceItemCollection guidanceItems = GuidanceItemCollection.New();
            foreach (GuidanceItem guidanceItem in _listView.GetSelectedGuidanceItems())
            {
                guidanceItems.Add(guidanceItem.Id, guidanceItem);
            }

            //If there is no selected items
            if (guidanceItems.Count == 0)
            {
                GuidanceItem guidanceItem = _listView.GetCurrentGuidanceItem();
                if (guidanceItem != null)
                {
                    guidanceItems.Add(guidanceItem.Id, guidanceItem);
                }
            }

            //If there is no current item.
            if (guidanceItems.Count == 0)
            {
                MessageBox.Show(Messages.NO_SELECTED_ITEM, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            //Obtain the GuidanceItemFileReference for each guidance item.
            GuidanceItemFileReferenceCollection gifrs = GuidanceItemFileReferenceCollection.New();
            foreach (GuidanceItem guidanceItem in guidanceItems)
            {
                GuidanceItemFileReference gifr = guidanceItem as GuidanceItemFileReference;
                if (gifr == null)
                {
                    gifr = guidanceItem.ReferencedItem;
                    if (gifr == null)
                    {
                        //This should not happen.
                        throw new NotImplementedException();
                    }
                }
                gifrs.Add(gifr.Id, gifr);
            }

            //Publish the selected guidance items
            foreach (GuidanceItemFileReference gifr in gifrs)
            {
                _onlineStorage.RemoveGuidanceItem(gifr);
            }

        }

        [CommandHandler(GuidanceExplorer.Commands.SuscribeLibraryFromFile)]
        public void SuscribeLibraryFromFile(object sender, EventArgs e)
        {
            LibrariesNode librariesNode = _winformRender.GetLibrariesNode(_treeView);
            if (librariesNode == null)
            {
                //Executed from an unexpected context.
                throw new NotSupportedException();
            }

            //Prepare the dialog.
            OpenFileDialog openFile = new OpenFileDialog();
            openFile.CheckFileExists = true;
            openFile.CheckPathExists = true;
            openFile.DefaultExt = "*.xml";
            openFile.Filter = Messages.XML_FILES_FILTER;
            openFile.Multiselect = true;
            openFile.RestoreDirectory = true;
            openFile.Title = "Subscribe Library From File";

            //Ask the user to select the file.
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    foreach (string file in openFile.FileNames)
                    {
                        SuscribeLibraryFromFile(librariesNode, file);
                    }

                    LibrariesNodeSelected(null, null);
                }
                catch
                {
                    MessageBox.Show(Messages.LIBRARY_FILE_INVALID, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void SuscribeLibraryFromFile(LibrariesNode librariesNode, string file)
        {
            //Read the library to a file.
            Library newLibrary = _modelService.ConvertFileInfoToLibrary(new FileInfo(file));
            //Check if there's already a suscription to the library
            if (_modelService.Guidance.Libraries.ContainsKey(newLibrary.Name))
            {
                MessageBox.Show(Messages.SUSCRIPTION_EXISTS, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                newLibrary.FileSuscribed = true;
                //newLibrary.ReadProtection = ReadProtection.ReadOnly;
                newLibrary.OriginalSourceFileInfo = new FileInfo(file);

                //Copy the library file to the local library folder.
                DirectoryInfo newLibraryPath = new DirectoryInfo(Path.Combine(Environment.CurrentDirectory, ModelStorageService.LibraryFolder));
                DirectoryInfo itemsBaseFolder = new DirectoryInfo(Path.Combine(newLibraryPath.FullName, newLibrary.Name));

                //Check if there is already a file with such name in the current library folder.
                if (File.Exists(newLibraryPath.FullName))
                {
                    MessageBox.Show("a file with such name already exists in the current library folder.");
                }
                else
                {
                    //Process all the guidance items.
                    _asyncLoad.ProcessLibrary(newLibrary, null, null);

                    //Add the library to the guidance in the model.
                    _modelService.Guidance.Libraries.Add(newLibrary.Name, newLibrary);

                    //If the library is in the DeletedLibraries, delete it
                    if (_modelService.Guidance.DeletedLibraries.ContainsKey(newLibrary.Name))
                    {
                        _modelService.Guidance.DeletedLibraries.Remove(newLibrary.Name);
                    }

                    //Update the UI
                    _winformRender.RenderLibraryToLibrariesNode(_treeView.TreeView, _modelService.Guidance, newLibrary, librariesNode);

                    FileInfo newLibraryFileName = new FileInfo(Path.Combine(newLibraryPath.FullName, newLibrary.Caption + ".xml"));
                    //Set the library file to the new location.
                    newLibrary.SourceFileInfo = newLibraryFileName;
                    //Save the file to the new location.
                    _modelService.RenderLibraryToFileInfo(newLibrary, newLibraryFileName);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.DeleteSearchView)]
        public void DeleteSearchView(object sender, EventArgs e)
        {
            DeleteCurrentNode();
        }

        [CommandHandler(GuidanceExplorer.Commands.ViewGuidanceItem)]
        public void ViewItem(object sender, EventArgs e)
        {
            GuidanceItem[] guidanceItem = _listView.GetSelectedGuidanceItems();
            if (guidanceItem == null)
            {
                MessageBox.Show(Messages.SELECTED_ITEM_NOT_FOUND, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            GuidanceItemFileReference[] gifr = _modelService.Guidance.GetGuidanceItemFileReference(guidanceItem);
            GuidanceItemOpened(this, new GuidanceItemFileReferenceEventArgs(gifr, ReadProtection.ReadOnly));
        }

        [CommandHandler(GuidanceExplorer.Commands.EditGuidanceItem)]
        public void EditItem(object sender, EventArgs e)
        {
            GuidanceItem[] guidanceItem = _listView.GetSelectedGuidanceItems();
            if (guidanceItem == null)
            {
                MessageBox.Show(Messages.SELECTED_ITEM_NOT_FOUND, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            GuidanceItemFileReference[] gifr = _modelService.Guidance.GetGuidanceItemFileReference(guidanceItem);
            GuidanceItemOpened(this, new GuidanceItemFileReferenceEventArgs(gifr, ReadProtection.ReadWrite));
        }
    }
}
