//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI;
using Microsoft.Practices.CompositeUI.SmartParts;

using GuidanceExplorer.Model;
using GuidanceExplorer.Services;
using GuidanceExplorer.Controls;
using GuidanceExplorer.Config;

namespace GuidanceExplorer.Controls
{

    public partial class ServerBrowser : Form
    {

        /// <summary>
        /// The collection of libraries to display
        /// </summary>
        private ServerCollection _servers;

        /// <summary>
        /// A Form that shows the current view tree structure
        /// </summary>
        /// <param name="nodeType">Nodes to be shown</param>
        public ServerBrowser()
        {
            InitializeComponent();
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);


            foreach (ServerElement server in _servers)
            {
                treeView.Nodes.Add(new TreeNode(server.Url));
            }

            treeView.ExpandAll();
            treeView.SelectedNode = treeView.Nodes[0];
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
            this.Close();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            this.Close();
        }

        /// <summary>
        /// The collection of libraries to display
        /// </summary>
        public ServerCollection Servers
        {
            set
            {
                _servers = value;
            }
        }

        public string SelectedServer
        {
            get 
            { 
                return treeView.SelectedNode.Text; 
            }
        }
    }
}
