//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================



using System;
using System.Collections.Generic;
using System.Text;

using NUnit.Framework;

using GuidanceExplorer.Services;
using GuidanceExplorer.Model;
using System.IO;
using System.Xml.Schema;
using Microsoft.Practices.CompositeUI;
using GuidanceExplorer.WorkItems;

namespace GuidanceExplorer.Tests
{
    /*
*SimpleLibrary.xml
*EmptyLibraryNoCaption.xml
*EmptyLibraryNoItems.xml
*EmptyLibraryNoItems1.xml
*EmptyLibraryNoName.xml
*EmptyLibraryWithItems.xml
*DupColumnsInFolderLibrary.xml
*DupColumnsInViewLibrary.xml
*DupFolderNameSimpleLibrary.xml
*DupFolderNameSimpleLibrary1.xml
*DupItemViewLibrary.xml
*DupViewNameSimpleLibrary.xml
*DupViewNameSimpleLibrary1.xml
SchemaErrorLibrary.xml
*/
    [TestFixture]
    public class LibraryStorageTests
    {
        [Test]
        public void SimpleLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "SimpleLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));

            Assert.IsNotNull(library, "library");
            Assert.AreEqual("pandp", library.Name);
            Assert.AreEqual("pattern & practices", library.Caption);
            Assert.AreEqual(ReadProtection.ReadOnly, library.ReadProtection);
            Assert.AreEqual(3, library.Items.Count);

            Assert.AreEqual(1, library.Views.Count);
            Assert.AreEqual("rootView", library.Views["rootView"].Caption);

            Assert.AreEqual(1, library.Folders.Count);
            Assert.AreEqual("root", library.Folders["root"].Caption);
            Assert.AreEqual(1, library.Folders["root"].Columns.Count);

            Assert.AreEqual(2, library.Folders["root"].Folders.Count);
            Assert.AreEqual("subRootFolderA", library.Folders["root"].Folders["subRootFolderA"].Caption);
            Assert.AreEqual(0, library.Folders["root"].Folders["subRootFolderA"].Folders.Count);
            Assert.AreEqual(0, library.Folders["root"].Folders["subRootFolderA"].Views.Count);
            Assert.AreEqual("subRootFolderB", library.Folders["root"].Folders["subRootFolderB"].Caption);
            Assert.AreEqual(0, library.Folders["root"].Folders["subRootFolderB"].Folders.Count);
            Assert.AreEqual(1, library.Folders["root"].Folders["subRootFolderB"].Views.Count);
            Assert.AreEqual("subRootsubView", library.Folders["root"].Folders["subRootFolderB"].Views["subRootsubView"].Caption);

            Assert.AreEqual(1, library.Folders["root"].Views.Count);
            Assert.AreEqual("subRootView", library.Folders["root"].Views["subRootView"].Caption);
            Assert.AreEqual(2, library.Views["rootView"].Items.Count);
            Assert.AreEqual(1, library.Folders["root"].Views["subRootView"].Items.Count);
            Assert.AreEqual(1, library.Folders["root"].Folders["subRootFolderB"].Views["subRootsubView"].Items.Count);
        }

        [Test]
        public void EmptyLibraryNoCaption()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "EmptyLibraryNoCaption.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));

            Assert.IsNotNull(library, "library");
            Assert.AreEqual(0, library.Items.Count);

            Assert.AreEqual(0, library.Views.Count);
            Assert.AreEqual(0, library.Folders.Count);

            Assert.AreEqual(String.Empty, library.Caption);
            Assert.AreEqual("pandp", library.Name);
        }

        [Test]
        public void EmptyLibraryNoItems()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "EmptyLibraryNoItems.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));

            Assert.IsNotNull(library);
            Assert.AreEqual(0, library.Items.Count);

            Assert.AreEqual(0, library.Views.Count);
            Assert.AreEqual(0, library.Folders.Count);

            Assert.AreEqual("pandp", library.Name);
            Assert.AreEqual("pattern & practices", library.Caption);
            Assert.AreEqual(ReadProtection.ReadOnly, library.ReadProtection);
        }

        [Test]
        public void EmptyLibraryNoItems1()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "EmptyLibraryNoItems1.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));

            Assert.IsNotNull(library);
            Assert.AreEqual(0, library.Items.Count);

            Assert.AreEqual(0, library.Views.Count);
            Assert.AreEqual(0, library.Folders.Count);

            Assert.AreEqual("pandp", library.Name);
            Assert.AreEqual("pattern & practices", library.Caption);
            Assert.AreEqual(ReadProtection.ReadOnly, library.ReadProtection);
        }

        [Test]
        public void EmptyLibraryNoName()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "EmptyLibraryNoName.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));

            Assert.IsNotNull(library);
            Assert.AreEqual(0, library.Items.Count);

            Assert.AreEqual(0, library.Views.Count);
            Assert.AreEqual(0, library.Folders.Count);

            Assert.AreEqual("", library.Name);
            Assert.AreEqual("pattern & practices", library.Caption);
            Assert.AreEqual(ReadProtection.ReadOnly, library.ReadProtection);
        }

        [Test]
        public void EmptyLibraryWithItems()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "EmptyLibraryWithItems.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));

            Assert.IsNotNull(library);
            Assert.AreEqual(3, library.Items.Count);

            Assert.AreEqual(0, library.Views.Count);
            Assert.AreEqual(0, library.Folders.Count);

            Assert.AreEqual("pandp", library.Name);
            Assert.AreEqual("pattern & practices", library.Caption);
            Assert.AreEqual(ReadProtection.ReadOnly, library.ReadProtection);
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupColumnsInFolderLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "DupColumnsInFolderLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupColumnsInViewLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "DupColumnsInViewLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupFolderNameSimpleLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "DupFolderNameSimpleLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupFolderNameSimpleLibrary1()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "DupFolderNameSimpleLibrary1.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupItemViewLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "DupItemViewLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupViewNameSimpleLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "DupViewNameSimpleLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));
        }

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void DupViewNameSimpleLibrary1()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "DupViewNameSimpleLibrary1.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));
        }

        [Test]
        [ExpectedException(typeof(XmlSchemaValidationException))]
        public void SchemaErrorLibrary()
        {
            string fileName = Path.Combine(Path.GetFullPath(@"..\..\Tests\LibraryStorageTests\"), "SchemaErrorLibrary.xml");

            ModelStorageService svc = new ModelStorageService();

            Library library = svc.ConvertFileInfoToLibrary(new FileInfo(fileName));
        }

        [Test]
        public void RenderSingleViewLibrary()
        {
            FileInfo fileName = new FileInfo(Path.GetTempFileName());
            File.Delete(fileName.FullName); //Just in case :)

            Library library = Library.New();
            library.Name = "testLib";
            library.Caption = "Test Lib";
            library.ReadProtection = ReadProtection.ReadWrite;
            library.SourceFileInfo = fileName;
            View view = View.New();
            view.Caption = "TestView";
            view.Columns = ColumnCollection.DefaultColumns(GuidanceType.New(GuidanceType.DefaultTypeName));
            GuidanceItem guidanceItem = GuidanceItem.New();
            guidanceItem.Id = "11";
            view.Items.Add(guidanceItem.Id, guidanceItem);
            guidanceItem = GuidanceItem.New();
            guidanceItem.Id = "22";
            view.Items.Add(guidanceItem.Id, guidanceItem);
            library.Views.Add(view.Caption, view);

            GuidanceItemFileReference refe = GuidanceItemFileReference.New();
            refe.FileName = "SimpleLibrary\\itemA.xml";
            refe.Id = "11";
            library.Items.Add(refe.Id, refe);
            refe = GuidanceItemFileReference.New();
            refe.FileName = "SimpleLibrary\\itemB.xml";
            refe.Id = "22";
            library.Items.Add(refe.Id, refe);

            ModelStorageService svc = new ModelStorageService();
            svc.RenderLibraryToFileInfo(library, fileName);

            Library newLibrary = svc.ConvertFileInfoToLibrary(fileName);
            IAsyncLoadLibrariesService svc2 = new AsyncLoadLibrariesService();
            Guidance guidance = Guidance.New();
            guidance.Libraries.Add(newLibrary.Name, newLibrary);
            svc2.ProcessGuidance(guidance, null);

            Assert.AreEqual(library.Items, newLibrary.Items);
        }
    }
}

