//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI;
using Microsoft.Practices.CompositeUI.SmartParts;

using GuidanceExplorer.Model;
using GuidanceExplorer.Services;
using GuidanceExplorer.Controls;
using GuidanceExplorer.WorkItems;

namespace GuidanceExplorer.Views
{
    [SmartPart]
    public partial class GuidanceItemView : UserControl
    {
        #region Private members

        private GuidanceItemFileReference _item;

        /// <summary>
        /// The main model service instance.
        /// </summary>
        private IModelStorageService _modelService;

        /// <summary>
        /// The HTML rendering service for the guidance items.
        /// </summary>
        private IHtmlRenderService _htmlRenderService;

        /// <summary>
        /// The main work item instance.
        /// </summary>
        private GuidanceExplorerWorkItem _workItem;

        #endregion

        #region Services properties

        /// <summary>
        /// The HTML rendering service for the guidance items.
        /// </summary>
        [ServiceDependency]
        public IHtmlRenderService HtmlRenderService
        {
            set { _htmlRenderService = value; }
        }

        /// <summary>
        /// The main model service instance.
        /// </summary>
        [ServiceDependency]
        public IModelStorageService ModelService
        {
            set { _modelService = value; }
        }

        /// <summary>
        /// The main work item instance.
        /// </summary>
        [ServiceDependency]
        public GuidanceExplorerWorkItem WorkItem
        {
            set { _workItem = value; }
        }

        #endregion

        public GuidanceItemView(GuidanceItemFileReference item)
        {
            InitializeComponent();
            _item = item;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            detailView.HtmlRenderService = _htmlRenderService;
            detailView.ModelService = _modelService;
            detailView.WorkItem = _workItem;
            detailView.ShowDetails(_item);
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            if (ParentForm != null)
            {
                ParentForm.Close();
            }
        }

        private void btnEdit_Click(object sender, EventArgs e)
        {
            _workItem.GuidanceItemOpened(this, new GuidanceItemFileReferenceEventArgs(new GuidanceItemFileReference[] { _item }, ReadProtection.ReadWrite));
            if (ParentForm != null)
            {
                ParentForm.Close();
            }
        }

        private void GuidanceItemView_ParentChanged(object sender, EventArgs e)
        {
            if (ParentForm != null)
            {
                ParentForm.CancelButton = btnClose;
            }
        }
    }
}