//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.Configuration;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI.Commands;
using Microsoft.Practices.CompositeUI.EventBroker;
using Microsoft.Practices.CompositeUI.WinForms;
using Microsoft.Practices.CompositeUI.SmartParts;

using GuidanceExplorer.Config;
using GuidanceExplorer.Model;
using GuidanceExplorer.Controls;
using GuidanceExplorer.Views;
using GuidanceExplorer.Services;
using System.ComponentModel;

namespace GuidanceExplorer.WorkItems
{
    public partial class GuidanceExplorerWorkItem
    {
        private string SearchViewEditor = "SearchViewEditor";
        private string SearchViewControl = "SearchViewControl";

        [CommandHandler(GuidanceExplorer.Commands.FileExit)]
        public void FileExit(object sender, EventArgs e)
        {
            _modelService.SaveGuidance(_modelService.Guidance, Environment.CurrentDirectory);
            Application.Exit();
        }

        public void AddSearchView(SearchViewNode searchViewNode, SearchView searchView, IModelItem containerNode)
        {
            LibraryNode libraryNode = containerNode as LibraryNode;
            FolderNode folderNode = containerNode as FolderNode;
            MyViewsNode myViewsNode = containerNode as MyViewsNode;
            GlobalViewsNode globalViewsNode = containerNode as GlobalViewsNode;

            if (libraryNode != null)
            {
                int i = 0;
                while (libraryNode.ModelInstance.Views.ContainsKey(searchView.Caption))
                {
                    searchView.Caption = string.Format(Messages.NEW_SEARCH_VIEW + " {0}", i++);
                }
                searchViewNode.Text = searchView.Caption;
                searchView.ContainerScope = _modelService.Guidance;
                searchView.Container = libraryNode.ModelInstance;
                searchView.LastUpdate = DateTime.UtcNow;

                //Add as a child to the library.
                libraryNode.ModelInstance.Views.Add(searchView.Caption, searchView);
                libraryNode.ModelInstance.HasChanges = true;

                //Add the search view to the tree node.
                libraryNode.Nodes.Add(searchViewNode);

                _treeView.TreeView.SelectedNode = searchViewNode;

                searchViewNode.BeginEdit();
            }
            else if (folderNode != null)
            {
                int i = 0;
                while (folderNode.ModelInstance.Views.ContainsKey(searchView.Caption))
                {
                    searchView.Caption = string.Format(Messages.NEW_SEARCH_VIEW + " {0}", i++);
                }
                searchViewNode.Text = searchView.Caption;
                searchView.ContainerScope = _modelService.Guidance;
                searchView.ParentFolder = folderNode.ModelInstance;
                folderNode.ModelInstance.Views.Add(searchView.Caption, searchView);
                searchView.LastUpdate = DateTime.UtcNow;

                //It's added as a sibling to the folder.
                folderNode.Nodes.Add(searchViewNode);
                if (folderNode.ModelInstance.GetContainerLibrary() != null)
                {
                    folderNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                }

                _treeView.TreeView.SelectedNode = searchViewNode;

                searchViewNode.BeginEdit();
            }
            else if (myViewsNode != null)
            {
                int i = 0;
                while (myViewsNode.ModelInstance.Views.ContainsKey(searchView.Caption))
                {
                    searchView.Caption = string.Format(Messages.NEW_SEARCH_VIEW + " {0}", i++);
                }
                searchViewNode.Text = searchView.Caption;
                searchView.ContainerScope = _modelService.Guidance;
                searchView.Container = myViewsNode.ModelInstance;
                searchView.LastUpdate = DateTime.UtcNow;

                //Add as a child to the library.
                myViewsNode.ModelInstance.Views.Add(searchView.Caption, searchView);

                //Add the search view to the tree node.
                myViewsNode.Nodes.Add(searchViewNode);

                _treeView.TreeView.SelectedNode = searchViewNode;

                searchViewNode.BeginEdit();
            }
            else if (globalViewsNode != null)
            {
                int i = 0;
                while (globalViewsNode.ModelInstance.Views.ContainsKey(searchView.Caption))
                {
                    searchView.Caption = string.Format(Messages.NEW_SEARCH_VIEW + " {0}", i++);
                }
                searchViewNode.Text = searchView.Caption;
                searchView.ContainerScope = _modelService.Guidance;
                searchView.Container = globalViewsNode.ModelInstance;
                searchView.LastUpdate = DateTime.UtcNow;

                //Add as a child to the library.
                globalViewsNode.ModelInstance.Views.Add(searchView.Caption, searchView);

                //Add the search view to the tree node.
                globalViewsNode.Nodes.Add(searchViewNode);

                _treeView.TreeView.SelectedNode = searchViewNode;

                searchViewNode.BeginEdit();
            }
            else
            {
                SearchViewNodeSelected(this, new ModelNodeEventArgs<SearchView>(searchViewNode));
            }
        }


        [CommandHandler(GuidanceExplorer.Commands.NewSearchView)]
        public void NewSearchView(object sender, EventArgs e)
        {
            IWorkspace workspace = RootWorkItem.Workspaces.Get<WindowWorkspace>(SearchViewEditor);
            if (workspace != null)
            {
                RootWorkItem.Workspaces.Remove(workspace);
            }

            SearchCriteriaControl searchControl = RootWorkItem.SmartParts.Get<SearchCriteriaControl>(SearchViewControl);
            if (searchControl != null)
            {
                RootWorkItem.SmartParts.Remove(searchControl);
            }

            workspace = new WindowWorkspace(new Form());
            RootWorkItem.Workspaces.Add(workspace, SearchViewEditor);
            searchControl = new SearchCriteriaControl(_treeView.GetCurrentNode() as IModelItem);
            RootWorkItem.SmartParts.Add(searchControl, SearchViewControl);
            WindowSmartPartInfo info = new WindowSmartPartInfo();
            info.MaximizeBox = false;
            if (_treeView.GetCurrentNode() is SearchViewNode)
            {
                info.Title = Messages.EDIT_SEARCH_VIEW;
            }
            else
            {
                info.Title = Messages.NEW_SEARCH_VIEW;
            }
            workspace.Show(searchControl, info);
        }

        [CommandHandler(GuidanceExplorer.Commands.RefreshFileSubscribedLibrary)]
        public void RefreshFileSubscribedLibrary(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;

            if (modelNode != null)
            {
                LibraryNode libraryNode = modelNode as LibraryNode;
                if (libraryNode != null)
                {
                    LibrariesNode librariesNode = libraryNode.Parent as LibrariesNode;

                    if (librariesNode != null)
                    {
                        _modelService.SaveGuidance(_modelService.Guidance, Environment.CurrentDirectory);
                        _modelService.Guidance.Libraries.Remove(libraryNode.ModelInstance.Name);
                        libraryNode.Remove();

                        SubscribeLibraryFromFile(librariesNode, libraryNode.ModelInstance.OriginalSourceFileInfo.FullName);
                    }
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ColumnChooser)]
        public void ColumnChooser(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            LibrariesNode librariesNode = treeNode as LibrariesNode;
            if (modelNode != null)
            {
                LibraryNode libraryNode = modelNode as LibraryNode;
                FolderNode folderNode = modelNode as FolderNode;
                MyViewsNode myViewsNode = treeNode as MyViewsNode;
                GlobalViewsNode globalViewsNode = treeNode as GlobalViewsNode;
                ViewNode viewNode = treeNode as ViewNode;
                SearchViewNode searchViewNode = treeNode as SearchViewNode;
                GuidanceTypeNode typeNode = treeNode as GuidanceTypeNode;

                if (libraryNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(libraryNode.ModelInstance.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        _listView.ShowContents(libraryNode.ModelInstance, libraryNode.ModelInstance.Columns);
                    }
                }
                else if (folderNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(folderNode.ModelInstance.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        _listView.ShowContents(folderNode.ModelInstance, folderNode.ModelInstance.Columns);
                    }
                }
                else if (myViewsNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(myViewsNode.ModelInstance.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        _listView.ShowContents(myViewsNode.ModelInstance, myViewsNode.ModelInstance.Columns);
                    }
                }
                else if (globalViewsNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(globalViewsNode.ModelInstance.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        _listView.ShowContents(globalViewsNode.ModelInstance, globalViewsNode.ModelInstance.Columns);
                    }
                }
                else if (viewNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(viewNode.ModelInstance.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        _listView.ShowContents(viewNode.ModelInstance, viewNode.ModelInstance.Columns);
                    }
                }
                else if (searchViewNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(searchViewNode.ModelInstance.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        _listView.ShowContents(searchViewNode.ModelInstance, searchViewNode.ModelInstance.Columns);
                    }
                }
                else if (typeNode != null)
                {
                    ColumnChooserForm columnChooser = new ColumnChooserForm(typeNode.Columns);
                    if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                    {
                        GuidanceTypeNodeSelected(this, new ModelNodeEventArgs<GuidanceType>(typeNode));
                    }
                }
                else
                {
                    //do nothing
                }
            }
            else if (librariesNode != null)
            {
                ColumnChooserForm columnChooser = new ColumnChooserForm(_modelService.Guidance.Columns);
                if (columnChooser.ShowDialog() == DialogResult.OK && columnChooser.HasChanges)
                {
                    LibrariesNodeSelected(this, null);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.NewGuidanceItem)]
        public void NewGuidanceItem(object sender, EventArgs e)
        {
            //Get the new guidance item type.
            GuidanceTypeEventArgs gtea = e as GuidanceTypeEventArgs;
            GuidanceType newGuidanceItemType = null;

            if (gtea != null)
            {
                newGuidanceItemType = gtea.GuidanceType;
            }
            else
            {
                foreach (GuidanceType guidanceType in GuidanceType.AllGuidanceTypes)
                {
                    if (string.Compare(guidanceType.DisplayName, "Item") == 0)
                    {
                        newGuidanceItemType = guidanceType;
                    }
                }
            }

            if (newGuidanceItemType == null)
            {
                return;
            }

            //Find the context libray.
            Library library = GetCurrentLibrary();

            //Create the guidance item.
            GuidanceItemFileReference gifr = GuidanceItemFileReference.New();
            gifr.GuidanceType = newGuidanceItemType;

            if (library != null)
            {
                string baseLibraryPath = null;
                if (library.FileSubscribed)
                {
                    baseLibraryPath = library.OriginalSourceFileInfo.DirectoryName + "\\" + library.Caption;
                }
                else
                {
                    baseLibraryPath = Path.Combine(ModelStorageService.LibraryFolder, library.Caption);
                }
                gifr.FileName = baseLibraryPath + "\\";
                gifr.Library = library;
            }
            else
            {
                gifr.FileName = ModelStorageService.LibraryFolder;
            }

            gifr.Id = Guid.NewGuid().ToString();
            gifr.CSSFile = ModelStorageService.DefaultCSSFileName;
            gifr.Title = string.Empty;

            //Edit the guidance item.
            GuidanceItemOpened(this, new GuidanceItemFileReferenceEventArgs(new GuidanceItemFileReference[] { gifr }, ReadProtection.ReadWrite, _treeView.GetCurrentNode() as ViewNode));
        }

        [CommandHandler(GuidanceExplorer.Commands.ExportLibrary)]
        public void ExportLibrary(object sender, EventArgs e)
        {
            if (LibrariesSynchronizing())
            {
                return;
            }

            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            if (modelNode == null)
            {
                return;
            }

            LibraryNode libraryNode = modelNode as LibraryNode;
            if (libraryNode != null)
            {
                SaveFileDialog saveFile = new SaveFileDialog();
                saveFile.CheckPathExists = true;
                saveFile.DefaultExt = "*.xml";
                saveFile.Filter = Messages.XML_FILES_FILTER;
                saveFile.OverwritePrompt = true;
                saveFile.Title = "Save Library as";
                saveFile.ValidateNames = true;
                saveFile.RestoreDirectory = true;
                saveFile.FileName = libraryNode.Text + ".xml";

                //Ask the user for the file name.
                if (saveFile.ShowDialog() == DialogResult.OK)
                {
                    //If the target file exists, then delete it.
                    if (File.Exists(saveFile.FileName))
                    {
                        File.Delete(saveFile.FileName);
                    }

                    //Render the library to the file.
                    _modelService.ExportAsLibrary(libraryNode.ModelInstance, libraryNode.Text, new FileInfo(saveFile.FileName), _htmlService);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.NewFolder)]
        public void NewFolder(object sender, EventArgs e)
        {
            Folder folder = Folder.New();

            FolderNode newFolderNode = FolderNode.New();
            newFolderNode.ModelInstance = folder;

            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            
            if (modelNode == null)
            {
                //a folder has been created in an unknown node
                throw new NotImplementedException();
            }

            AddFolder(folder, newFolderNode, modelNode);
            newFolderNode.BeginEdit();
        }

        public void AddFolder(Folder folder, FolderNode newFolderNode, IModelItem modelNode)
        {
            LibraryNode libraryNode = modelNode as LibraryNode;
            FolderNode folderNode = modelNode as FolderNode;
            MyViewsNode myViewsNode = modelNode as MyViewsNode;
            GlobalViewsNode globalViewsNode = modelNode as GlobalViewsNode;
            if (libraryNode != null)
            {
                //Update the folder name
                folder.Caption = _treeView.GetNewNodeName(libraryNode, Messages.NEW_FOLDER);
                newFolderNode.Text = folder.Caption;

                //Update the model
                folder.Container = libraryNode.ModelInstance;
                folder.ParentFolder = null;
                libraryNode.ModelInstance.Folders.Add(folder.Caption, folder);
                libraryNode.ModelInstance.HasChanges = true;

                //Update the treeview
                libraryNode.Nodes.Add(newFolderNode);
                newFolderNode.TreeView.SelectedNode = newFolderNode;
            }
            else if (folderNode != null)
            {
                //Update the folder name
                folder.Caption = _treeView.GetNewNodeName(folderNode, Messages.NEW_FOLDER);
                newFolderNode.Text = folder.Caption;

                //Update the model
                folder.ParentFolder = folderNode.ModelInstance;
                folder.Container = null;
                folderNode.ModelInstance.Folders.Add(folder.Caption, folder);
                if (folderNode.ModelInstance.GetContainerLibrary() != null)
                {
                    folderNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                }

                //Update the treeview
                folderNode.Nodes.Add(newFolderNode);
                newFolderNode.TreeView.SelectedNode = newFolderNode;
            }
            else if (myViewsNode != null)
            {
                //Update the folder name
                folder.Caption = _treeView.GetNewNodeName(myViewsNode, Messages.NEW_FOLDER);
                newFolderNode.Text = folder.Caption;

                //Update the model
                _modelService.Guidance.UserViews.Folders.Add(folder.Caption, folder);
                folder.Container = _modelService.Guidance.UserViews;
                folder.ParentFolder = null;

                //Update the treeview
                myViewsNode.Nodes.Add(newFolderNode);
                newFolderNode.TreeView.SelectedNode = newFolderNode;
            }
            else if (globalViewsNode != null)
            {
                //Update the folder name
                folder.Caption = _treeView.GetNewNodeName(globalViewsNode, Messages.NEW_FOLDER);
                newFolderNode.Text = folder.Caption;

                //Update the model
                _modelService.Guidance.GlobalViews.Folders.Add(folder.Caption, folder);
                folder.Container = _modelService.Guidance.GlobalViews;
                folder.ParentFolder = null;

                //Update the treeview
                globalViewsNode.Nodes.Add(newFolderNode);
                newFolderNode.TreeView.SelectedNode = newFolderNode;
            }
            else
            {
                throw new NotImplementedException();
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.SaveViewAs)]
        public void SaveViewAs(object sender, EventArgs e)
        {
            ViewNode viewNode = _treeView.GetCurrentNode() as ViewNode;

            if (viewNode != null)
            {
                GuidanceItem[] items = new GuidanceItem[viewNode.ModelInstance.Items.Values.Count];
                viewNode.ModelInstance.Items.Values.CopyTo(items, 0);
                SaveItems(items);
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.SaveLibraryAs)]
        public void SaveLibraryAs(object sender, EventArgs e)
        {
            List<GuidanceItem> items = new List<GuidanceItem>();

            foreach (GuidanceItem item in _listView.Items)
            {
                items.Add(item);
            }

            SaveItems(items.ToArray());
        }

        [CommandHandler(GuidanceExplorer.Commands.SaveFolderAs)]
        public void SaveFolderAs(object sender, EventArgs e)
        {
            SaveLibraryAs(this, e);
        }

        [CommandHandler(GuidanceExplorer.Commands.SaveMyViewsAs)]
        public void SaveMyViewsAs(object sender, EventArgs e)
        {
            SaveLibraryAs(this, e);
        }

        [CommandHandler(GuidanceExplorer.Commands.SaveGlobalViewsAs)]
        public void SaveGlobalViewsAs(object sender, EventArgs e)
        {
            SaveLibraryAs(this, e);
        }

        [CommandHandler(GuidanceExplorer.Commands.SaveItemAs)]
        public void SaveItemAs(object sender, EventArgs e)
        {
            GuidanceItem[] items = _listView.GetSelectedGuidanceItems();

            if (items == null || items.Length == 0)
            {
                MessageBox.Show("Please select an item to save", Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                SaveItems(items);
            }
        }

        private void SaveItems(GuidanceItem[] items)
        {
            if (LibrariesSynchronizing())
            {
                return;
            }

            SaveFileDialog saveFileDlg = new SaveFileDialog();
            saveFileDlg.DefaultExt = ".doc";
            saveFileDlg.AddExtension = true;
            saveFileDlg.Filter = Messages.WORD_FILES_FILTER + "|" + Messages.HTML_FILES_FILTER;
            saveFileDlg.RestoreDirectory = true;
            saveFileDlg.ValidateNames = true;

            if (saveFileDlg.ShowDialog() == DialogResult.OK)
            {
                if (items.Length == 1)
                {
                    GuidanceItemFileReference item = items[0] as GuidanceItemFileReference;

                    if (item == null)
                    {
                        item = _modelService.Guidance.GetGuidanceItemFileReference(items[0]);
                    }

                    FileInfo file = new FileInfo(saveFileDlg.FileName);
                    if (Path.GetExtension(saveFileDlg.FileName) == ".doc")
                    {
                        _htmlService.RenderGuidanceItemToHtml(item, file, true);
                    }
                    else //Path extensionis htm
                    {
                        _htmlService.RenderGuidanceItemToHtml(item, file, false);
                    }
                }
                else
                {
                    List<GuidanceItemFileReference> guidanceItems = new List<GuidanceItemFileReference>(items.Length);

                    foreach (GuidanceItem guidanceItem in items)
                    {
                        GuidanceItemFileReference item = guidanceItem as GuidanceItemFileReference;

                        if (item == null)
                        {
                            item = _modelService.Guidance.GetGuidanceItemFileReference(guidanceItem);
                        }

                        guidanceItems.Add(item);
                    }

                    if (Path.GetExtension(saveFileDlg.FileName) == ".doc")
                    {
                        _htmlService.RenderGuidanceItemsToWordFileInfoWithIndex(guidanceItems, new FileInfo(saveFileDlg.FileName));
                    }
                    else //Path extensionis htm
                    {
                        _htmlService.RenderGuidanceItemsToHtmlFileInfoWithIndex(guidanceItems, new FileInfo(saveFileDlg.FileName));
                    }
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.SelectAll)]
        public void SelectAll(object sender, EventArgs e)
        {
            _listView.SelectAllItems();
            _listView.Focus();
        }

        [CommandHandler(GuidanceExplorer.Commands.NewView)]
        public void NewView(object sender, EventArgs e)
        {
            GuidanceExplorer.Model.View view = GuidanceExplorer.Model.View.New();

            ViewNode newViewNode = ViewNode.New();
            newViewNode.ModelInstance = view;

            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;

            if (modelNode == null)
            {
                //a view has been created in an unknown node
                throw new NotImplementedException();
            }

            AddView(view, newViewNode, modelNode);

            newViewNode.BeginEdit();
        }

        public void AddView(GuidanceExplorer.Model.View view, ViewNode newViewNode, IModelItem modelNode)
        {
            LibraryNode libraryNode = modelNode as LibraryNode;
            FolderNode folderNode = modelNode as FolderNode;
            MyViewsNode myViewsNode = modelNode as MyViewsNode;
            GlobalViewsNode globalViewsNode = modelNode as GlobalViewsNode;

            view.LastUpdate = DateTime.UtcNow;

            if (libraryNode != null)
            {
                //Update the view name
                view.Caption = _treeView.GetNewNodeName(libraryNode, Messages.NEW_VIEW);
                newViewNode.Text = view.Caption;

                //Update the model
                view.Container = libraryNode.ModelInstance;
                view.ParentFolder = null;
                libraryNode.ModelInstance.Views.Add(view.Caption, view);
                libraryNode.ModelInstance.HasChanges = true;

                //Update the treeview
                libraryNode.Nodes.Add(newViewNode);
                newViewNode.TreeView.SelectedNode = newViewNode;
            }
            else if (folderNode != null)
            {
                //Update the view name
                view.Caption = _treeView.GetNewNodeName(folderNode, Messages.NEW_VIEW);
                newViewNode.Text = view.Caption;

                //Update the model
                view.ParentFolder = folderNode.ModelInstance;
                view.Container = null;
                folderNode.ModelInstance.Views.Add(view.Caption, view);
                if (folderNode.ModelInstance.GetContainerLibrary() != null)
                {
                    folderNode.ModelInstance.GetContainerLibrary().HasChanges = true;
                }

                //Update the treeview
                folderNode.Nodes.Add(newViewNode);
                newViewNode.TreeView.SelectedNode = newViewNode;
            }
            else if (myViewsNode != null)
            {
                //Update the view name
                view.Caption = _treeView.GetNewNodeName(myViewsNode, Messages.NEW_VIEW);
                newViewNode.Text = view.Caption;

                //Update the model
                _modelService.Guidance.UserViews.Views.Add(view.Caption, view);
                view.Container = _modelService.Guidance.UserViews;
                view.ParentFolder = null;

                //Update the treeview
                myViewsNode.Nodes.Add(newViewNode);
                newViewNode.TreeView.SelectedNode = newViewNode;
            }
            else if (globalViewsNode != null)
            {
                //Update the view name
                view.Caption = _treeView.GetNewNodeName(globalViewsNode, Messages.NEW_VIEW);
                newViewNode.Text = view.Caption;

                //Update the model
                _modelService.Guidance.GlobalViews.Views.Add(view.Caption, view);
                view.Container = _modelService.Guidance.GlobalViews;
                view.ParentFolder = null;

                //Update the treeview
                globalViewsNode.Nodes.Add(newViewNode);
                newViewNode.TreeView.SelectedNode = newViewNode;
            }
            else
            {
                throw new NotImplementedException();
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.NewLibrary)]
        public void NewLibrary(object sender, EventArgs e)
        {
            //Update the TreeView
            LibrariesNode librariesNode = _treeView.GetCurrentNode() as LibrariesNode;
            if (librariesNode == null)
            {
                //The current node is not libraries
                throw new NotImplementedException();
            }

            //Update the model.
            Library library = Library.New();
            library.ReadProtection = ReadProtection.ReadWrite;
            library.Caption = _treeView.GetNewLibraryName(_modelService.Guidance.Libraries, librariesNode, Messages.NEW_LIBRARY);
            library.Name = Guid.NewGuid().ToString();
            library.SourceFileInfo = new FileInfo(Path.GetFullPath(Path.Combine(ModelStorageService.LibraryFolder, library.Name + ".xml")));

            //Add the library to the guidance.
            _modelService.Guidance.Libraries.Add(library.Name, library);

            LibraryNode libraryNode = LibraryNode.New();
            libraryNode.ModelInstance = library;
            libraryNode.Text = library.Caption;

            librariesNode.Nodes.Add(libraryNode);

            if (!librariesNode.IsExpanded)
            {
                librariesNode.Expand();
            }

            libraryNode.BeginEdit();
        }

        [CommandHandler(GuidanceExplorer.Commands.SubmitFeedback)]
        public void SubmitFeedback(object sender, EventArgs e)
        {
            WebBrowser browser = new WebBrowser();
            GuidanceExplorerSection section = ConfigurationManager.GetSection("GuidanceExplorer") as GuidanceExplorerSection;
            browser.Navigate(section.FeedbackLink);
        }

        [CommandHandler(GuidanceExplorer.Commands.NewGuidanceType)]
        public void NewGuidanceType(object sender, EventArgs e)
        {
            InputBox input = new InputBox("New Guidance Type", "Please write the name of the new guidance type:", "guidanceType");
            if (input.ShowDialog() == DialogResult.OK)
            {
                //TODO: Check that previous guidance type does not exist with the same name.

                GuidanceType guidanceType = GuidanceType.New(input.UserText.Replace(" ", string.Empty));
                guidanceType.DisplayName = input.UserText;
                guidanceType.ListDisplayName = input.UserText;
                guidanceType.NewGuidanceTypeDisplayName = input.UserText;
                guidanceType.TemplateFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, guidanceType.Name + "_template.xml"));

                //Create the base template file
                if (!File.Exists(guidanceType.TemplateFileName))
                {
                    string templateFileName = Path.GetFullPath(Path.Combine(ModelStorageService.TemplateFolder, GuidanceType.New("template").TemplateFileName));
                    File.Copy(templateFileName, guidanceType.TemplateFileName, true);
                }

                //Refresh the list view.
                AuthoringToolkitNodeSelected(this, e);
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.RenameLibrary)]
        public void RenameLibrary(object sender, EventArgs e)
        {
            //Get the selected node.
            TreeNode treeNode = _treeView.GetCurrentNode();
            treeNode.BeginEdit();
        }

        [CommandHandler(GuidanceExplorer.Commands.RenameView)]
        public void RenameView(object sender, EventArgs e)
        {
            RenameLibrary(null, null);
        }

        [CommandHandler(GuidanceExplorer.Commands.RenameFolder)]
        public void RenameFolder(object sender, EventArgs e)
        {
            RenameLibrary(null, null);
        }

        [CommandHandler(GuidanceExplorer.Commands.DeleteFolder)]
        public void DeleteFolder(object sender, EventArgs e)
        {
            DeleteCurrentNode();
        }

        [CommandHandler(GuidanceExplorer.Commands.DeleteView)]
        public void DeleteView(object sender, EventArgs e)
        {
            DeleteCurrentNode();
        }

        [CommandHandler(GuidanceExplorer.Commands.DeleteLibrary)]
        public void DeleteLibrary(object sender, EventArgs e)
        {
            DeleteCurrentNode();
        }

        [CommandHandler(GuidanceExplorer.Commands.ExportViewAsLibrary)]
        public void ExportViewAsLibrary(object sender, EventArgs e)
        {
            if (LibrariesSynchronizing())
            {
                return;
            }

            //Get the selected node.
            TreeNode treeNode = _treeView.GetCurrentNode();
            ViewNode viewNode = treeNode as ViewNode;
            SearchViewNode searchViewNode = treeNode as SearchViewNode;
            GuidanceTypeNode guidanceTypeNode = treeNode as GuidanceTypeNode;
            string fileName = "Library";

            if (viewNode == null && guidanceTypeNode == null && searchViewNode == null)
            {
                //This is only for saving views.
                throw new NotImplementedException();
            }

            if (viewNode != null)
            {
                fileName = viewNode.Text;
            }
            else if (searchViewNode != null)
            {
                fileName = searchViewNode.Text;
            }
            else if (guidanceTypeNode != null)
            {
                fileName = guidanceTypeNode.Text;
            }

            SaveFileDialog saveFile = new SaveFileDialog();
            saveFile.CheckPathExists = true;
            saveFile.DefaultExt = "*.xml";
            saveFile.Filter = Messages.XML_FILES_FILTER;
            saveFile.OverwritePrompt = true;
            saveFile.Title = "Save view as";
            saveFile.ValidateNames = true;
            saveFile.RestoreDirectory = true;
            saveFile.FileName = fileName;

            //Ask the user for the file name.
            if (saveFile.ShowDialog() == DialogResult.OK)
            {
                //If the target file exists, then delete it.
                if (File.Exists(saveFile.FileName))
                {
                    File.Delete(saveFile.FileName);
                }

                if (viewNode != null)
                {
                    //Render the view to the file.
                    _modelService.ExportAsLibrary(viewNode.ModelInstance, viewNode.Text, new FileInfo(saveFile.FileName), _htmlService);
                }
                if (searchViewNode != null)
                {
                    _modelService.ExportAsLibrary(searchViewNode.ModelInstance, searchViewNode.Text, new FileInfo(saveFile.FileName), _htmlService);
                }
                else if (guidanceTypeNode != null)
                {
                    //Get the library.
                    LibraryNode libraryNode = (LibraryNode)guidanceTypeNode.Parent;

                    //Create the list to display
                    GuidanceItemFileReferenceCollection guidanceItems = libraryNode.ModelInstance.GetGuidanceItemFileReferenceForGuidanceType(guidanceTypeNode.ModelInstance);
                    guidanceItems.Columns = guidanceTypeNode.Columns;

                    //Render the guidancetype list to the file.
                    _modelService.ExportAsLibrary(guidanceItems, guidanceTypeNode.Text, new FileInfo(saveFile.FileName), _htmlService);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.MoveGuidanceItemToLibrary)]
        public void MoveItemToLibrary(object sender, EventArgs e)
        {
            GuidanceItem[] items = _listView.GetSelectedGuidanceItems();
            bool warningShown = false;
            GuidanceExplorer.Model.Library selectedLibrary;
            GuidanceExplorer.Model.Library originalLibrary;

            LibraryNode library = _treeView.GetCurrentNode() as LibraryNode;
            if (library == null || library.ModelInstance.ReadProtection == ReadProtection.ReadOnly)
            {
                MessageBox.Show(Messages.CAN_ONLY_MOVE_ITEMS_FROM_WRITABLE_LIBRARIES, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (items != null && items.Length > 0)
            {
                LibrariesBrowser browser = new LibrariesBrowser();
                browser.NodeTypes = GuidanceNodeTypes.ReadWriteLibraries;
                browser.Guidance = _modelService.Guidance;

                GuidanceItemFileReference gifr = items[0] as GuidanceItemFileReference;
                originalLibrary = gifr.Library;

                if (browser.ShowDialog() == DialogResult.OK)
                {
                    //get the library
                    GuidanceItemCollection itemsAddCollection = GuidanceItemCollection.New();
                    GuidanceItemCollection itemsDeleteCollection = GuidanceItemCollection.New();
                    List<GuidanceType> typeCollection = new List<GuidanceType>();

                    selectedLibrary = browser.SelectedLibrary;
                    selectedLibrary.HasChanges = true;
               
                    //check if source and destination are the same
                    if (originalLibrary == selectedLibrary)
                    {
                        return;
                    }

                    if (!File.Exists(selectedLibrary.IndexFileName))
                    {
                        _asyncLoad.CreateLibraryIndex(selectedLibrary, null, selectedLibrary.IndexFileName);
                    }

                    if (!File.Exists(originalLibrary.IndexFileName))
                    {
                        _asyncLoad.CreateLibraryIndex(originalLibrary, null, originalLibrary.IndexFileName);
                    }

                    //for each guidance item, add to the new library. Make sure to retain the original GUID
                    foreach (GuidanceItemFileReference item in items)
                    {
                        //Create the file reference.
                        string file = item.FileName;
                        GuidanceItemFileReference fileRef = GuidanceItemFileReference.New();
                        fileRef.FileName = file;
                        fileRef.Library = selectedLibrary;
                        fileRef.Id = item.Id;

                        _asyncLoad.ProcessGuidanceItemFile(fileRef);

                        if (ValidateImportedItem(fileRef, selectedLibrary, !warningShown, false, file))
                        {
                            itemsAddCollection.Add(fileRef.Id, fileRef);
                            itemsDeleteCollection.Add(item.Id, item);
                            typeCollection.Add(fileRef.GuidanceType);
                        }
                    }
                    //Add the item to the library.
                    _modelService.AddGuidanceItemsToLibrary(selectedLibrary, itemsAddCollection);
 
                    //Remove from old library
                    GuidanceItemsDeleted(null, new GuidanceItemCollectionEventArgs(itemsDeleteCollection));

                    //Refresh type nodes
                    foreach (GuidanceType gType in typeCollection)
                    {
                        _winformRender.RefreshGuidanceTypeNodes(_winformRender.GetLibraryNodeByLibrary(_treeView, selectedLibrary), gType);
                        _winformRender.RefreshGuidanceTypeNodes(_winformRender.GetLibraryNodeByLibrary(_treeView, originalLibrary), gType);
                    }

                    //Refresh the listview
                    _listView.ShowContents(originalLibrary, originalLibrary.Columns);

                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.MoveGuidanceItemToView)]
        public void MoveItemToView(object sender, EventArgs e)
        {
            GuidanceItem[] items = _listView.GetSelectedGuidanceItems();

            if (items != null && items.Length > 0)
            {
                LibrariesBrowser browser = new LibrariesBrowser();
                browser.Guidance = _modelService.Guidance;
                browser.ParameterLibrary = items[0].Library;
                browser.NodeTypes = GuidanceNodeTypes.ReadWriteViews;

                if (browser.ShowDialog() == DialogResult.OK)
                {
                    GuidanceExplorer.Model.View selectedView = browser.SelectedView;
                    if (selectedView != null)
                    {
                        foreach (GuidanceItem item in items)
                        {
                            if (!selectedView.Items.ContainsKey(item.Id))
                            {
                                _modelService.AddGuidanceItemToView(selectedView, item);
                            }
                        }

                        _treeView.TreeView.SelectedNode = GetNodeByModelInstance<GuidanceExplorer.Model.View>(selectedView);

                        if (selectedView.GetContainerLibrary() != null)
                        {
                            selectedView.GetContainerLibrary().HasChanges = true;
                        }
                    }
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ExportMyViewsAsLibrary)]
        public void ExportMyViewsAsLibrary(object sender, EventArgs e)
        {
            if (LibrariesSynchronizing())
            {
                return;
            }

            //Get the selected node.
            TreeNode treeNode = _treeView.GetCurrentNode();
            MyViewsNode myViewsNode = treeNode as MyViewsNode;
            if (myViewsNode == null)
            {
                //This is only for saving views.
                throw new NotImplementedException();
            }

            SaveFileDialog saveFile = new SaveFileDialog();
            saveFile.CheckPathExists = true;
            saveFile.DefaultExt = "*.xml";
            saveFile.Filter = Messages.XML_FILES_FILTER;
            saveFile.OverwritePrompt = true;
            saveFile.Title = "Save My Views as";
            saveFile.ValidateNames = true;
            saveFile.RestoreDirectory = true;
            saveFile.FileName = myViewsNode.Text;

            //Ask the user for the file name.
            if (saveFile.ShowDialog() == DialogResult.OK)
            {
                //If the target file exists, then delete it.
                if (File.Exists(saveFile.FileName))
                {
                    File.Delete(saveFile.FileName);
                }

                //Render the view to the file.
                _modelService.ExportAsLibrary(myViewsNode.ModelInstance, myViewsNode.Text, new FileInfo(saveFile.FileName), _htmlService);
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ExportGlobalViewsAsLibrary)]
        public void ExportGlobalViewsAsLibrary(object sender, EventArgs e)
        {
            if (LibrariesSynchronizing())
            {
                return;
            }

            //Get the selected node.
            TreeNode treeNode = _treeView.GetCurrentNode();
            GlobalViewsNode globalViewsNode = treeNode as GlobalViewsNode;
            if (globalViewsNode == null)
            {
                //This is only for saving views.
                throw new NotImplementedException();
            }

            SaveFileDialog saveFile = new SaveFileDialog();
            saveFile.CheckPathExists = true;
            saveFile.DefaultExt = "*.xml";
            saveFile.Filter = Messages.XML_FILES_FILTER;
            saveFile.OverwritePrompt = true;
            saveFile.Title = "Save My Views as";
            saveFile.ValidateNames = true;
            saveFile.RestoreDirectory = true;
            saveFile.FileName = globalViewsNode.Text;

            //Ask the user for the file name.
            if (saveFile.ShowDialog() == DialogResult.OK)
            {
                //If the target file exists, then delete it.
                if (File.Exists(saveFile.FileName))
                {
                    File.Delete(saveFile.FileName);
                }

                //Render the view to the file.
                _modelService.ExportAsLibrary(globalViewsNode.ModelInstance, globalViewsNode.Text, new FileInfo(saveFile.FileName), _htmlService);
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ImportLibrary)]
        public void ImportLibrary(object sender, EventArgs e)
        {
            if (LibrariesSynchronizing())
            {
                return;
            }

            //Prepare the dialog.
            OpenFileDialog openFile = new OpenFileDialog();
            openFile.CheckFileExists = true;
            openFile.CheckPathExists = true;
            openFile.DefaultExt = "*.xml";
            openFile.Filter = Messages.XML_FILES_FILTER;
            openFile.Multiselect = true;
            openFile.RestoreDirectory = true;
            openFile.Title = "Import Library Xml";

            //Ask the user to select the file.
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    foreach (string file in openFile.FileNames)
                    {
                        //Read the library to a file.
                        Library newLibrary = _modelService.ConvertFileInfoToLibrary(new FileInfo(file));

                        //Check if the library exists
                        if (_modelService.Guidance.Libraries.ContainsKey(newLibrary.Name))
                        {
                            MessageBox.Show(Messages.CANNOT_IMPORT_TWICE, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        else
                        {
                            //Copy the library file to the local library folder.
                            DirectoryInfo newLibraryPath = new DirectoryInfo(Path.Combine(Environment.CurrentDirectory, ModelStorageService.LibraryFolder));
                            DirectoryInfo itemsBaseFolder = new DirectoryInfo(Path.Combine(newLibraryPath.FullName, newLibrary.Name));

                            //Check if there is already a file with such name in the current library folder.
                            if (File.Exists(newLibraryPath.FullName))
                            {
                                MessageBox.Show(Messages.FILE_NAME_ALREADY_EXISTS);
                                return;
                            }

                            //Copy the items to the local library folder.
                            foreach (GuidanceItem guidanceItem in newLibrary.Items.Values)
                            {
                                GuidanceItemFileReference gifr = guidanceItem as GuidanceItemFileReference;
                                if (gifr != null)
                                {
                                    gifr.FileName = Path.Combine(newLibrary.SourceFileInfo.DirectoryName, gifr.FileName);
                                    _asyncLoad.ProcessGuidanceItemFile(gifr);

                                    //Calculate the old and new file names for the item.
                                    string oldItemFileName = Path.Combine(newLibrary.SourceFileInfo.DirectoryName, gifr.FileName);
                                    string newItemFileName = Path.Combine(itemsBaseFolder.FullName, gifr.GuidanceType.Name +"\\" + Path.GetFileName(gifr.FileName));

                                    //Validate file names.
                                    if (!File.Exists(oldItemFileName))
                                    {
                                        MessageBox.Show(string.Format("The item '{0}' in the library can't be found.", oldItemFileName));
                                        return;
                                    }
                                    //if (File.Exists(newItemFileName))
                                    //{
                                    //    MessageBox.Show(string.Format("The item '{0}' in the source library already exist in the target library.", newItemFileName));
                                    //    return;
                                    //}

                                    //Create the required directories.
                                    if (!Directory.Exists(Path.GetDirectoryName(newItemFileName)))
                                    {
                                        Directory.CreateDirectory(Path.GetDirectoryName(newItemFileName));
                                    }

                                    //Copy the file.
                                    File.Copy(oldItemFileName, newItemFileName, true);

                                    //Adjust the references of the files in the library.
                                    gifr.Library = newLibrary;

                                    newItemFileName = newItemFileName.Replace(Path.GetFullPath(ModelStorageService.LibraryFolder) + Path.DirectorySeparatorChar, String.Empty);
                                    gifr.FileName = Path.Combine(ModelStorageService.LibraryFolder, newItemFileName);
                                }
                                else
                                {
                                    //Strange situation it should not happens.
                                    throw new NotSupportedException();
                                }
                            }

                            FileInfo newLibraryFileName = new FileInfo(Path.Combine(newLibraryPath.FullName, newLibrary.Caption + ".xml"));
                            //Set the library file to the new location.
                            newLibrary.SourceFileInfo = newLibraryFileName;

                            //Add the library to the guidance in the model.
                            _modelService.Guidance.Libraries.Add(newLibrary.Name, newLibrary);

                            if (File.Exists(newLibrary.IndexFileName))
                            {
                                File.Delete(newLibrary.IndexFileName);
                            }

                            //Process the library to adjust names and create the index.
                            _asyncLoad.ProcessLibrary(newLibrary, null, null);

                            //Update the UI
                            LibrariesNode librariesNode = _treeView.GetCurrentNode() as LibrariesNode;
                            if (librariesNode != null)
                            {
                                _winformRender.RenderLibraryToLibrariesNode(_treeView.TreeView, _modelService.Guidance, newLibrary, librariesNode);
                            }
                            else
                            {
                                //The current node is not libraries
                                throw new NotImplementedException();
                            }

                            //Save the file to the new location.
                            _modelService.RenderLibraryToFileInfo(newLibrary, newLibraryFileName);

                        }
                    }
                }
                catch
                {
                    MessageBox.Show(Messages.LIBRARY_FILE_INVALID, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.AddRemoveGuidanceType)]
        public void AddRemoveGuidanceType(object sender, EventArgs e)
        {
            AuthoringToolkitControl authoringTkit = new AuthoringToolkitControl(_modelService);
            authoringTkit.Text = Messages.ADD_REMOVE_GUIDANCE_TYPE;

            authoringTkit.ShowDialog();
        }

        [CommandHandler(GuidanceExplorer.Commands.ImportGuidanceItemToView)]
        public void ImportItemToView(object sender, EventArgs e)
        {
            if (LibrariesSynchronizing())
            {
                return;
            }

            //Get the view selected.
            ViewNode viewNode = _treeView.GetCurrentNode() as ViewNode;
            if (viewNode == null)
            {
                //This is only for importing into views.
                throw new NotImplementedException();
            }

            Library library = viewNode.ModelInstance.GetContainerLibrary();

            if (library == null)
            {
                LibrariesBrowser browser = new LibrariesBrowser();
                browser.Guidance = _modelService.Guidance;
                browser.NodeTypes = GuidanceNodeTypes.ReadWriteLibraries;
                browser.Text = Messages.SELECT_LIBRARY;

                if (browser.ShowDialog() == DialogResult.OK)
                {
                    library = browser.SelectedLibrary;
                }
                else
                {
                    return;
                }
            }

            //Prepare the dialog.
            OpenFileDialog openFile = new OpenFileDialog();
            openFile.CheckFileExists = true;
            openFile.CheckPathExists = true;
            openFile.DefaultExt = "*.xml";
            openFile.Filter = Messages.XML_FILES_FILTER;
            openFile.Multiselect = true;
            openFile.RestoreDirectory = true;
            openFile.Title = Messages.IMPORT_ITEM_XML;

            //Ask the user to select the file.
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    //Iterate through the selected files.
                    foreach (string file in openFile.FileNames)
                    {
                        //Create the file reference.
                        GuidanceItemFileReference fileRef = GuidanceItemFileReference.New();
                        fileRef.FileName = file;
                        fileRef.Library = library;

                        _asyncLoad.ProcessGuidanceItemFile(fileRef);

                        if (ValidateImportedItem(fileRef, library, true, true, file))
                        {
                            //Add the item to the library.
                            _modelService.AddGuidanceItemToLibrary(library, fileRef);

                            //Add the item to the view.
                            _modelService.AddGuidanceItemToView(viewNode.ModelInstance, fileRef);

                            //Refresh guidance type nodes
                            _winformRender.RefreshGuidanceTypeNodes(_winformRender.GetLibraryNodeByLibrary(_treeView, library), fileRef.GuidanceType);

                            //Refresh the list.
                            _listView.ShowContents(viewNode.ModelInstance, viewNode.ModelInstance.Columns);
                        }
                    }

                    ViewNodeSelected(this, new ModelNodeEventArgs<GuidanceExplorer.Model.View>(viewNode));
                }
                catch
                {
                    MessageBox.Show(Messages.ITEM_FILE_INVALID, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ImportItemToFolder)]
        public void ImportItemToFolder(object sender, EventArgs e)
        {
            if (LibrariesSynchronizing())
            {
                return;
            }

            //Get the view selected.
            FolderNode folderNode = _treeView.GetCurrentNode() as FolderNode;
            if (folderNode == null)
            {
                return;
            }

            Library library = folderNode.ModelInstance.GetContainerLibrary();
            if (library == null)
            {
                //TODO: ask for a library to import, since this view is defined in the MyViews node.
                throw new NotSupportedException();
            }

            //Prepare the dialog.
            OpenFileDialog openFile = new OpenFileDialog();
            openFile.CheckFileExists = true;
            openFile.CheckPathExists = true;
            openFile.DefaultExt = "*.xml";
            openFile.Filter = Messages.XML_FILES_FILTER;
            openFile.Multiselect = true;
            openFile.RestoreDirectory = true;
            openFile.Title = Messages.IMPORT_ITEM_XML;

            //Ask the user to select the file.
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    //Iterate through the selected files.
                    foreach (string file in openFile.FileNames)
                    {
                        //Create the file reference.
                        GuidanceItemFileReference fileRef = GuidanceItemFileReference.New();
                        fileRef.FileName = file;
                        fileRef.Library = library;

                        _asyncLoad.ProcessGuidanceItemFile(fileRef);

                        //Check if the item already exists in any other library.
                        if (_modelService.Guidance.GetGuidanceItemFileReference(fileRef) != null)
                        {
                            //The item already exists.
                            MessageBox.Show(Messages.ITEM_EXISTS_IN_ANOTHER_LIBRARY, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }

                        //Add the item to the library.
                        _modelService.AddGuidanceItemToLibrary(library, fileRef);

                        //Check if the item already exists in any other library.
                        if (_modelService.Guidance.GetGuidanceItemFileReference(fileRef) != null)
                        {
                            //The item already exists.
                            MessageBox.Show(Messages.ITEM_EXISTS_IN_ANOTHER_LIBRARY, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }

                        //Add the item to the library.
                        _modelService.AddGuidanceItemToLibrary(library, fileRef);

                        //Refresh the list.
                        _listView.ShowContents(folderNode.ModelInstance, folderNode.ModelInstance.Columns);
                    }
                }
                catch
                {
                    MessageBox.Show(Messages.ITEM_FILE_INVALID, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ImportGuidanceItemToLibrary)]
        public void ImportItemToLibrary(object sender, EventArgs e)
        {
            if (LibrariesSynchronizing())
            {
                return;
            }

            //Get the view selected.
            LibraryNode libraryNode = _treeView.GetCurrentNode() as LibraryNode;
            GuidanceTypeNode typeNode = _treeView.GetCurrentNode() as GuidanceTypeNode;
            if (libraryNode == null)
            {
                //Check if the selected node is a type node
                if (typeNode != null)
                {
                    libraryNode = typeNode.Parent as LibraryNode;
                }
                else
                {
                    //current node is invalid
                    throw new NotSupportedException();
                }
            }

            Library library = libraryNode.ModelInstance;
            if (library == null)
            {
                //TODO: ask for a library to import, since this folder is defined in the MyViews node.
                throw new NotSupportedException();
            }

            //Prepare the dialog.
            OpenFileDialog openFile = new OpenFileDialog();
            openFile.CheckFileExists = true;
            openFile.CheckPathExists = true;
            openFile.DefaultExt = "*.xml";
            openFile.Filter = Messages.XML_FILES_FILTER;
            openFile.Multiselect = true;
            openFile.RestoreDirectory = true;
            openFile.Title = Messages.IMPORT_ITEM_XML;

            //Ask the user to select the file.
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    bool warningShown = false;
                    //Iterate through the selected files.
                    foreach (string file in openFile.FileNames)
                    {
                        //Create the file reference.
                        GuidanceItemFileReference fileRef = GuidanceItemFileReference.New();
                        fileRef.FileName = file;
                        fileRef.Library = library;

                        _asyncLoad.ProcessGuidanceItemFile(fileRef);

                        if (ValidateImportedItem(fileRef, library, !warningShown, true, file))
                        {
                            library.HasChanges = true;

                            //Add the item to the library.
                            _modelService.AddGuidanceItemToLibrary(library, fileRef);

                            //Refresh type nodes
                            _winformRender.RefreshGuidanceTypeNodes(libraryNode, fileRef.GuidanceType);

                            //Refresh the list.
                            if (typeNode != null)
                            {
                                GuidanceTypeNodeSelected(this, new ModelNodeEventArgs<GuidanceType>(typeNode));
                            }
                            else
                            {
                                LibraryNodeSelected(this, new ModelNodeEventArgs<Library>(libraryNode));
                            }
                        }
                        else
                        {
                            warningShown = true;
                        }
                    }
                }
                catch 
                {
                    MessageBox.Show(Messages.ITEM_FILE_INVALID, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private bool ValidateImportedItem(GuidanceItemFileReference guidanceItem, Library selectedLibrary, bool showMessage, bool updateGUID, string sourceFile)
        {

            if (updateGUID)
            {
                guidanceItem.Id = Guid.NewGuid().ToString();
            }

            //Copy the file to the libraries folder path
            string newFilePath = string.Empty;

            if (selectedLibrary.FileSubscribed)
            {
                newFilePath = Path.Combine(
                        Path.Combine(
                            Path.Combine(selectedLibrary.OriginalSourceFileInfo.DirectoryName, selectedLibrary.Caption),
                            guidanceItem.GuidanceType.DisplayName),
                        guidanceItem.Id + ".xml");
            }
            else
            {
                newFilePath = Path.Combine(
                        Path.Combine(
                            Path.Combine(ModelStorageService.LibraryFolder, selectedLibrary.Caption),
                            guidanceItem.GuidanceType.DisplayName),
                        guidanceItem.Id + ".xml");
            }

            if (!Directory.Exists(Path.GetDirectoryName(newFilePath)))
            {
                Directory.CreateDirectory(Path.GetDirectoryName(newFilePath));
            }

            File.Copy(sourceFile, newFilePath, true);

            guidanceItem.FileName = newFilePath;

            //Save changes
            using (StreamReader reader = new StreamReader(_htmlService.GetReadOnlyDocumentStream(guidanceItem)))
            {
                _modelService.RenderGuidanceItemToFileInfo(guidanceItem, new FileInfo(guidanceItem.FileName), reader.ReadToEnd(), false);
            }
            return true;

        }

        [CommandHandler(GuidanceExplorer.Commands.SaveItemAsXml)]
        public void SaveItemAsXml(object sender, EventArgs e)
        {
            if (LibrariesSynchronizing())
            {
                return;
            }

            GuidanceItem[] guidanceItems = _listView.GetSelectedGuidanceItems();
            if (guidanceItems == null || guidanceItems.Length == 0)
            {
                MessageBox.Show("Please select an item to export", Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (guidanceItems.Length == 1)
            {
                GuidanceItemFileReference fileRef = _modelService.Guidance.GetGuidanceItemFileReference(guidanceItems[0]);
                if (fileRef != null)
                {
                    //If the source file does not exists, show an error.
                    if (!File.Exists(fileRef.FileName))
                    {
                        throw new NotSupportedException();
                    }

                    //Request user filename
                    SaveFileDialog saveFile = new SaveFileDialog();
                    saveFile.CheckPathExists = true;
                    saveFile.DefaultExt = "*.xml";
                    saveFile.Filter = Messages.XML_FILES_FILTER;
                    saveFile.OverwritePrompt = true;
                    saveFile.Title = "Save item as";
                    saveFile.ValidateNames = true;
                    saveFile.RestoreDirectory = true;
                    saveFile.FileName = Path.GetFileName(fileRef.FileName);

                    //Ask the user for the file name.
                    if (saveFile.ShowDialog() == DialogResult.OK)
                    {
                        //If the target file exists, then delete it.
                        if (File.Exists(saveFile.FileName))
                        {
                            File.Delete(saveFile.FileName);
                        }

                        //Copy the original file to the new location.
                        File.Copy(fileRef.FileName, saveFile.FileName);
                        //using (StreamReader reader = new StreamReader(_htmlService.GetReadOnlyDocumentStream(fileRef)))
                        //{
                        //    _modelService.RenderGuidanceItemToFileInfo(fileRef, new FileInfo(saveFile.FileName), reader.ReadToEnd());
                        //}
                    }
                }
            }
            else
            {
                FolderBrowserDialog folderDialog = new FolderBrowserDialog();
                folderDialog.ShowNewFolderButton = true;
                folderDialog.Description = Messages.SELECT_FOLDER_TO_EXPORT_ITEMS;

                if (folderDialog.ShowDialog() == DialogResult.OK)
                {
                    foreach (GuidanceItem item in guidanceItems)
                    {
                        GuidanceItemFileReference fileRef = _modelService.Guidance.GetGuidanceItemFileReference(item);
                        if (fileRef != null && File.Exists(fileRef.FileName))
                        {
                            string newFileName = Path.Combine(folderDialog.SelectedPath,
                                Path.GetFileName(fileRef.FileName));

                            //Copy the original file to the new location.
                            //File.Copy(fileRef.FileName, newFileName, true);
                            using (StreamReader reader = new StreamReader(_htmlService.GetReadOnlyDocumentStream(fileRef)))
                            {
                                _modelService.RenderGuidanceItemToFileInfo(fileRef, new FileInfo(newFileName), reader.ReadToEnd(), true);
                            }
                        }
                    }
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ExportFolderAsLibrary)]
        public void ExportFolderAsLibrary(object sender, EventArgs e)
        {
            if (LibrariesSynchronizing())
            {
                return;
            }

            //Get the selected node.
            TreeNode treeNode = _treeView.GetCurrentNode();
            FolderNode folderNode = treeNode as FolderNode;
            if (folderNode == null)
            {
                //This is only for saving folders.
                throw new NotImplementedException();
            }

            //Request user save file.
            SaveFileDialog saveFile = new SaveFileDialog();
            saveFile.CheckPathExists = true;
            saveFile.DefaultExt = "*.xml";
            saveFile.Filter = Messages.XML_FILES_FILTER;
            saveFile.OverwritePrompt = true;
            saveFile.Title = "Save folder as";
            saveFile.ValidateNames = true;
            saveFile.RestoreDirectory = true;
            saveFile.FileName = folderNode.Text;

            //Ask the user for the file name.
            if (saveFile.ShowDialog() == DialogResult.OK)
            {
                //If the target file exists, then delete it.
                if (File.Exists(saveFile.FileName))
                {
                    File.Delete(saveFile.FileName);
                }

                //Render the folder to the file.
                _modelService.ExportAsLibrary(folderNode.ModelInstance, folderNode.Text, new FileInfo(saveFile.FileName), _htmlService);
            }
        }


        [CommandHandler(GuidanceExplorer.Commands.UndoDragAndDrop)]
        public void UndoDragAndDrop(object sender, EventArgs e)
        {
            _treeView.UndoDragDrop();
        }

        [CommandHandler(GuidanceExplorer.Commands.ResetColumns)]
        public void ResetColumns(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            ViewNode viewNode = treeNode as ViewNode;
            SearchViewNode searchViewNode = treeNode as SearchViewNode;
            FolderNode folderNode = treeNode as FolderNode;
            LibraryNode libraryNode = treeNode as LibraryNode;
            GuidanceTypeNode typeNode = treeNode as GuidanceTypeNode;
            LibrariesNode librariesNode = treeNode as LibrariesNode;
            MyViewsNode myViewsNode = treeNode as MyViewsNode;
            GlobalViewsNode globalViewsNode = treeNode as GlobalViewsNode;
            if (viewNode != null)
            {
                ResetColumns(viewNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Position | ResetOptions.Visibility);
                _listView.ShowContents(viewNode.ModelInstance, viewNode.ModelInstance.Columns);
            }
            else if (searchViewNode != null)
            {
                ResetColumns(searchViewNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Position | ResetOptions.Visibility);

                //If this is the Whats new Node
                if (TreeView.TreeView.Nodes[0] == searchViewNode)
                {
                    foreach (Column col in searchViewNode.ModelInstance.Columns.Values)
                    {
                        if (string.Compare(col.Field.Name, Messages.DATE_FIELD) == 0)
                        {
                            col.Sort = SortOrder.Descending;
                        }
                        else
                        {
                            col.Sort = SortOrder.None;
                        }
                    }
                }

                _listView.ShowContents(searchViewNode.ModelInstance, searchViewNode.ModelInstance.Columns);
            }
            else if (folderNode != null)
            {
                ResetColumns(folderNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Position | ResetOptions.Visibility);
                _listView.ShowContents(folderNode.ModelInstance, folderNode.ModelInstance.Columns);
            }
            else if (libraryNode != null)
            {
                ResetColumns(libraryNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Position | ResetOptions.Visibility);
                _listView.ShowContents(libraryNode.ModelInstance, libraryNode.ModelInstance.Columns);
            }
            else if (typeNode != null)
            {
                ColumnCollection columns = ColumnCollection.DefaultColumns(typeNode.ModelInstance);
                foreach (Column col in columns.Values)
                {
                    col.Visible = col.Field.ShowInGuidanceTypeNode;
                }
                ResetColumns(typeNode.Columns, columns, ResetOptions.Position | ResetOptions.Visibility);
                GuidanceTypeNodeSelected(sender, new ModelNodeEventArgs<GuidanceType>(typeNode));
            }
            else if (librariesNode != null)
            {
                ResetColumns(_modelService.Guidance.Columns, _modelService.Guidance.DefaultColumns, ResetOptions.Position | ResetOptions.Visibility);
                LibrariesNodeSelected(this, null);
            }
            else if (myViewsNode != null)
            {
                ResetColumns(myViewsNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Position | ResetOptions.Visibility);
                MyViewsSelected(this, new ModelNodeEventArgs<UserViews>(myViewsNode));
            }
            else if (globalViewsNode != null)
            {
                ResetColumns(globalViewsNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Position | ResetOptions.Visibility);
                GlobalViewsSelected(this, new ModelNodeEventArgs<GlobalViews>(globalViewsNode));
            }
            else
            {
                // Any other tree node.
                throw new NotImplementedException();
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ResetFilters)]
        public void ResetFilters(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            ViewNode viewNode = treeNode as ViewNode;
            SearchViewNode searchViewNode = treeNode as SearchViewNode;
            FolderNode folderNode = treeNode as FolderNode;
            LibraryNode libraryNode = treeNode as LibraryNode;
            GuidanceTypeNode typeNode = treeNode as GuidanceTypeNode;
            LibrariesNode librariesNode = treeNode as LibrariesNode;
            MyViewsNode myViewsNode = treeNode as MyViewsNode;
            GlobalViewsNode globalViewsNode = treeNode as GlobalViewsNode;
            if (viewNode != null)
            {
                foreach (Column column in viewNode.ModelInstance.Columns.Values)
                {
                    column.AutoFilter = null;
                }
                _listView.ShowContents(viewNode.ModelInstance, viewNode.ModelInstance.Columns);
            }
            else if (searchViewNode != null)
            {
                foreach (Column column in searchViewNode.ModelInstance.Columns.Values)
                {
                    column.AutoFilter = null;
                }
                _listView.ShowContents(searchViewNode.ModelInstance, searchViewNode.ModelInstance.Columns);
            }
            else if (folderNode != null)
            {
                foreach (Column column in folderNode.ModelInstance.Columns.Values)
                {
                    column.AutoFilter = null;
                }
                _listView.ShowContents(folderNode.ModelInstance, folderNode.ModelInstance.Columns);
            }
            else if (libraryNode != null)
            {
                foreach (Column column in libraryNode.ModelInstance.Columns.Values)
                {
                    column.AutoFilter = null;
                }
                _listView.ShowContents(libraryNode.ModelInstance, libraryNode.ModelInstance.Columns);
            }
            else if (typeNode != null)
            {
                foreach (Column column in typeNode.Columns.Values)
                {
                    column.AutoFilter = null;
                }
                GuidanceTypeNodeSelected(sender, new ModelNodeEventArgs<GuidanceType>(typeNode));
            }
            else if (librariesNode != null)
            {
                foreach (Column column in _modelService.Guidance.Columns.Values)
                {
                    column.AutoFilter = null;
                }

                LibrariesNodeSelected(this, null);
            }
            else if (myViewsNode != null)
            {
                foreach (Column column in myViewsNode.ModelInstance.Columns.Values)
                {
                    column.AutoFilter = null;
                }

                MyViewsSelected(null, new ModelNodeEventArgs<UserViews>(myViewsNode));
            }
            else if (globalViewsNode != null)
            {
                foreach (Column column in globalViewsNode.ModelInstance.Columns.Values)
                {
                    column.AutoFilter = null;
                }

                GlobalViewsSelected(null, new ModelNodeEventArgs<GlobalViews>(globalViewsNode));
            }
            else
            {
                // Any other tree node.
                throw new NotImplementedException();
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.ResetSortOrder)]
        public void ResetSortOrder(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            ViewNode viewNode = treeNode as ViewNode;
            SearchViewNode searchViewNode = treeNode as SearchViewNode;
            FolderNode folderNode = treeNode as FolderNode;
            LibraryNode libraryNode = treeNode as LibraryNode;
            GuidanceTypeNode typeNode = treeNode as GuidanceTypeNode;
            LibrariesNode librariesNode = treeNode as LibrariesNode;
            MyViewsNode myViewsNode = treeNode as MyViewsNode;
            GlobalViewsNode globalViewsNode = treeNode as GlobalViewsNode;
            if (viewNode != null)
            {
                ResetColumns(viewNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);
                _listView.ShowContents(viewNode.ModelInstance, viewNode.ModelInstance.Columns);
            }
            else if (searchViewNode != null)
            {
                ResetColumns(searchViewNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);

                //If this is the Whats new Node
                if (TreeView.TreeView.Nodes[0] == searchViewNode)
                {
                    foreach (Column col in searchViewNode.ModelInstance.Columns.Values)
                    {
                        if (string.Compare(col.Field.Name, Messages.DATE_FIELD) == 0)
                        {
                            col.Sort = SortOrder.Descending;
                        }
                        else
                        {
                            col.Sort = SortOrder.None;
                        }
                    }
                }

                _listView.ShowContents(searchViewNode.ModelInstance, searchViewNode.ModelInstance.Columns);
            }
            else if (folderNode != null)
            {
                ResetColumns(folderNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);
                _listView.ShowContents(folderNode.ModelInstance, folderNode.ModelInstance.Columns);
            }
            else if (libraryNode != null)
            {
                ResetColumns(libraryNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);
                _listView.ShowContents(libraryNode.ModelInstance, libraryNode.ModelInstance.Columns);
            }
            else if (typeNode != null)
            {
                ResetColumns(typeNode.Columns, ColumnCollection.DefaultColumns(typeNode.ModelInstance), ResetOptions.Sort);
                GuidanceTypeNodeSelected(sender, new ModelNodeEventArgs<GuidanceType>(typeNode));
            }
            else if (librariesNode != null)
            {
                ResetColumns(_modelService.Guidance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);
                LibrariesNodeSelected(this, null);
            }
            else if (myViewsNode != null)
            {
                ResetColumns(myViewsNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);
                MyViewsSelected(null, new ModelNodeEventArgs<UserViews>(myViewsNode));
            }
            else if (globalViewsNode != null)
            {
                ResetColumns(globalViewsNode.ModelInstance.Columns, ColumnCollection.DefaultColumns(GuidanceType.New("guideline")), ResetOptions.Sort);
                GlobalViewsSelected(null, new ModelNodeEventArgs<GlobalViews>(globalViewsNode));
            }
            else
            {
                // Any other tree node.
                throw new NotImplementedException();
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.DeleteGuidanceItem)]
        public void DeleteGuidanceItem(object sender, EventArgs e)
        {
            GuidanceItem[] guidanceItems = _listView.GetSelectedGuidanceItems();
            if (guidanceItems == null)
            {
                MessageBox.Show(Messages.SELECTED_ITEM_NOT_FOUND, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            GuidanceItemCollection items = GuidanceItemCollection.New();

            foreach (GuidanceItem item in guidanceItems)
            {
                items.Add(item.Id, item);
            }

            GuidanceItemsDeleted(this, new GuidanceItemCollectionEventArgs(items));
        }

        [CommandHandler(GuidanceExplorer.Commands.HelpAbout)]
        public void HelpAbout(object sender, EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }

        [CommandHandler(GuidanceExplorer.Commands.RSSSubscribe)]
        public void RSSSubscribe(object sender, EventArgs e)
        {
            System.Diagnostics.Process rssProcess = new System.Diagnostics.Process();
            TreeNode treeNode = _treeView.GetCurrentNode();
            LibraryNode libraryNode = treeNode as LibraryNode;
            GuidanceTypeNode typeNode = treeNode as GuidanceTypeNode;
            String libraryId = null;
            String guidanceType = null;

            OnlineStorageSection section = ConfigurationManager.GetSection("OnlineStorage") as OnlineStorageSection;
            String webServiceUrl = section.Servers[0].Url;
            int index = webServiceUrl.LastIndexOf("/");

            if (index > 0)
            {
                webServiceUrl = webServiceUrl.Substring(0, index + 1);
            }

            if (libraryNode != null)
            {
                libraryId = libraryNode.ModelInstance.Name;
                webServiceUrl = webServiceUrl + "LibraryFeed.ashx?libraryId=" + libraryId;
            }
            else if (typeNode != null)
            {
                libraryNode = (LibraryNode)typeNode.Parent;
                libraryId = libraryNode.ModelInstance.Name;
                guidanceType = typeNode.ModelInstance.DisplayName;
                webServiceUrl = webServiceUrl + "LibraryFeed.ashx?libraryId=" + libraryId + "&guidanceType=" + guidanceType;
            }
            else
            {
                return;
            }

            rssProcess.StartInfo.FileName = webServiceUrl;
            rssProcess.Start();    
        }

        [CommandHandler(GuidanceExplorer.Commands.Synchronize)]
        public void Synchronize(object sender, EventArgs e)
        {
            BackgroundWorker bw = new BackgroundWorker();
            _listView.IsSynchronizing = true;
            _statusLabel.Text = "Synchronizing Libraries";
            bw.DoWork += delegate(object s, DoWorkEventArgs eventArgs)
            {
                //Synchronize the local libraries
                _onlineStorage.Syncronize(_modelService, _htmlService, true, false, eventArgs);
            };
            bw.RunWorkerCompleted += delegate(object s, RunWorkerCompletedEventArgs ev)
            {
                if (!ev.Cancelled)
                {
                    MessageBox.Show(Messages.SYNCHRONIZATION_COMPLETED, Messages.ONLINE_STORE);
                }

                string selectedNode = GetNodePath(_treeView.TreeView.SelectedNode);
                string selectedItem = null;
                GuidanceItem[] items = _listView.GetSelectedGuidanceItems();
                _listView.IsSynchronizing = false;

                if (items != null && items.Length > 0)
                {
                    selectedItem = items[0].Id;
                }


                //Repaint the guidance in the tree view control
                _winformRender.RenderGuidanceToTreeView(_modelService.Guidance, _treeView);

                SelectNode(selectedNode);

                if (selectedItem != null)
                {
                    _listView.SelectItem(selectedItem);
                }
            };
            bw.RunWorkerAsync();
        }

        private void SelectNode(string path)
        {
            string[] nodes = path.Split(new string[] { "/" }, StringSplitOptions.RemoveEmptyEntries);
            TreeNode currentNode = GetNode(_treeView.TreeView.Nodes, nodes[0]);

            if (currentNode != null)
            {
                for (int i = 1; i < nodes.Length && ContainsNode(currentNode.Nodes, nodes[i]); i++)
                {
                    currentNode = GetNode(currentNode.Nodes, nodes[i]);
                }

                SelectNode(currentNode);
            }
        }

        private static bool ContainsNode(TreeNodeCollection nodes, string nodeText)
        {
            foreach (TreeNode node in nodes)
            {
                if (string.Compare(node.Text, nodeText) == 0)
                {
                    return true;
                }
            }

            return false;
        }

        private static TreeNode GetNode(TreeNodeCollection nodes, string nodeText)
        {
            foreach (TreeNode node in nodes)
            {
                if (string.Compare(node.Text, nodeText) == 0)
                {
                    return node;
                }
            }

            return null;
        }

        private static string GetNodePath(TreeNode node)
        {
            string path = string.Empty;
            while (node != null)
            {
                path = node.Text + "/" + path;
                node = node.Parent;
            }

            return path;
        }

        [CommandHandler(GuidanceExplorer.Commands.SubscribeLibraryFromWeb)]
        public void SubscribeLibraryFromWeb(object sender, EventArgs e)
        {
            LibrariesNode librariesNode = _winformRender.GetLibrariesNode(_treeView);
            if (librariesNode == null)
            {
                //Executed from an unexpected context.
                throw new NotSupportedException();
            }

            //Subscribe to the online storage.
            LibraryCollection onlineLibraries = null;

            try
            {
                onlineLibraries = _onlineStorage.GetAvailableLibraries(AvailableLibrariesMode.DoNotAllowCreation);
            }
            catch { }

            if (onlineLibraries == null)
            {
                MessageBox.Show("No online store configured", Messages.ONLINE_STORE, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                //Display the list of libraries to select the library to subscribe.
                LibrariesBrowser browser = new LibrariesBrowser();
                browser.Text = "Select an online library";
                browser.NodeTypes = GuidanceNodeTypes.All;
                browser.Libraries = onlineLibraries;
                if (browser.ShowDialog() == DialogResult.OK)
                {
                    Library subscribedLibrary = browser.SelectedLibrary;

                    //Check if the library is already subscribed.
                    if (_modelService.Guidance.Libraries.ContainsKey(subscribedLibrary.Name))
                    {
                        MessageBox.Show("The library is already subscribed.", Messages.ONLINE_STORE);
                        return;
                    }

                    Synchronize(sender, e);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.PublishLibrary)]
        public void PublishLibrary(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            if (modelNode == null)
            {
                //It will only work for views
                throw new NotSupportedException();
            }

            LibraryNode libraryNode = modelNode as LibraryNode;
            if (libraryNode == null)
            {
                //It will only work for views
                throw new NotSupportedException();
            }

            if (libraryNode.ModelInstance.FileSubscribed)
            {
                MessageBox.Show("A file subscribed library can't be published.");
                return;
            }

            try
            {
                //Add the library to the online storage.
                string server = ChooseServer();
                if (string.IsNullOrEmpty(server))
                {
                    return;
                }
                else
                {
                    libraryNode.ModelInstance.URL = server;
                    if (_onlineStorage.CreateLibrary(libraryNode.ModelInstance, false))
                    {
                        Synchronize(sender, e);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        /// <summary>
        /// Allow user to choose the server they want to publish a library to
        /// </summary>
        /// <returns></returns>
        public string ChooseServer()
        {
            ServerBrowser browser = new ServerBrowser();
            OnlineStorageSection section = ConfigurationManager.GetSection("OnlineStorage") as OnlineStorageSection;
            browser.Text = "Select a server";
            browser.Servers = section.Servers;
            if (browser.ShowDialog() == DialogResult.OK)
            {
                return browser.SelectedServer;
            }
            else
            {
                return string.Empty;
            }
       }

        [CommandHandler(GuidanceExplorer.Commands.RemoveView)]
        public void RemoveView(object sender, EventArgs e)
        {
            TreeNode treeNode = _treeView.GetCurrentNode();
            IModelItem modelNode = treeNode as IModelItem;
            if (modelNode == null)
            {
                //It will only work for views
                throw new NotSupportedException();
            }

            ViewNode viewNode = modelNode as ViewNode;
            if (viewNode == null)
            {
                //It will only work for views
                throw new NotSupportedException();
            }

            //remove the view from online storage
            Library library = viewNode.ModelInstance.GetContainerLibrary();
            if ((library != null) && (library.WebSubscribed))
            {
                _onlineStorage.RemoveView(viewNode.ModelInstance);
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.PublishGuidanceItemInView)]
        public void PublishGuidanceItemInView(object sender, EventArgs e)
        {
            IModelItem modelNode = _treeView.GetCurrentNode() as IModelItem;
            ViewNode viewNode = modelNode as ViewNode;
            if (viewNode == null)
            {
                MessageBox.Show("Current view can't be detected.");
                return;
            }

            GuidanceItemCollection guidanceItems = GuidanceItemCollection.New();
            foreach (GuidanceItem guidanceItem in _listView.GetSelectedGuidanceItems())
            {
                guidanceItems.Add(guidanceItem.Id, guidanceItem);
            }

            //If there is no selected items
            if (guidanceItems.Count == 0)
            {
                GuidanceItem guidanceItem = _listView.GetCurrentGuidanceItem();
                if (guidanceItem != null)
                {
                    guidanceItems.Add(guidanceItem.Id, guidanceItem);
                }
            }

            //Obtain the GuidanceItemFileReference for each guidance item.
            GuidanceItemFileReferenceCollection gifrs = GuidanceItemFileReferenceCollection.New();
            foreach (GuidanceItem guidanceItem in guidanceItems)
            {
                GuidanceItemFileReference gifr = guidanceItem as GuidanceItemFileReference;
                if (gifr == null)
                {
                    gifr = guidanceItem.ReferencedItem;
                    if (gifr == null)
                    {
                        //This should not happen.
                        throw new NotImplementedException();
                    }
                }
                gifrs.Add(gifr.Id, gifr);
            }

            //Get the id for the view.
            string viewId = viewNode.ModelInstance.Id;
            Library library = viewNode.ModelInstance.GetContainerLibrary();
            if (library != null)
            {
                string url = library.URL;
                _onlineStorage.AddGuidanceItemToView(url, new Guid(viewId), gifrs);
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.SubscribeLibraryFromFile)]
        public void SubscribeLibraryFromFile(object sender, EventArgs e)
        {
            if (LibrariesSynchronizing())
            {
                return;
            }

            LibrariesNode librariesNode = _winformRender.GetLibrariesNode(_treeView);
            if (librariesNode == null)
            {
                //Executed from an unexpected context.
                throw new NotSupportedException();
            }

            //Prepare the dialog.
            OpenFileDialog openFile = new OpenFileDialog();
            openFile.CheckFileExists = true;
            openFile.CheckPathExists = true;
            openFile.DefaultExt = "*.xml";
            openFile.Filter = Messages.XML_FILES_FILTER;
            openFile.Multiselect = true;
            openFile.RestoreDirectory = true;
            openFile.Title = "Subscribe Library From File";

            //Ask the user to select the file.
            if (openFile.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    foreach (string file in openFile.FileNames)
                    {
                        SubscribeLibraryFromFile(librariesNode, file);
                    }
                }
                catch
                {
                    MessageBox.Show(Messages.LIBRARY_FILE_INVALID, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void SubscribeLibraryFromFile(LibrariesNode librariesNode, string file)
        {
            //Read the library to a file.
            Library newLibrary = _modelService.ConvertFileInfoToLibrary(new FileInfo(file));
            //Check if there's already a suscription to the library
            if (_modelService.Guidance.Libraries.ContainsKey(newLibrary.Name))
            {
                MessageBox.Show(Messages.SUSCRIPTION_EXISTS, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                newLibrary.FileSubscribed = true;
                //newLibrary.ReadProtection = ReadProtection.ReadOnly;
                newLibrary.OriginalSourceFileInfo = new FileInfo(file);

                //Copy the library file to the local library folder.
                DirectoryInfo newLibraryPath = new DirectoryInfo(Path.Combine(Environment.CurrentDirectory, ModelStorageService.LibraryFolder));
                DirectoryInfo itemsBaseFolder = new DirectoryInfo(Path.Combine(newLibraryPath.FullName, newLibrary.Name));

                //Check if there is already a file with such name in the current library folder.
                if (File.Exists(newLibraryPath.FullName))
                {
                    MessageBox.Show(Messages.FILE_NAME_ALREADY_EXISTS);
                }
                else
                {
                    //Process all the guidance items.
                    _asyncLoad.ProcessLibrary(newLibrary, null, null);

                    //Add the library to the guidance in the model.
                    _modelService.Guidance.Libraries.Add(newLibrary.Name, newLibrary);

                    //If the library is in the DeletedLibraries, delete it
                    if (_modelService.Guidance.DeletedLibraries.ContainsKey(newLibrary.Name))
                    {
                        _modelService.Guidance.DeletedLibraries.Remove(newLibrary.Name);
                    }

                    //Update the UI
                    _winformRender.RenderLibraryToLibrariesNode(_treeView.TreeView, _modelService.Guidance, newLibrary, librariesNode);

                    FileInfo newLibraryFileName = new FileInfo(Path.Combine(newLibraryPath.FullName, newLibrary.Caption + ".xml"));
                    //Set the library file to the new location.
                    newLibrary.SourceFileInfo = newLibraryFileName;
                    //Save the file to the new location.
                    _modelService.RenderLibraryToFileInfo(newLibrary, newLibraryFileName);
                }
            }
        }

        [CommandHandler(GuidanceExplorer.Commands.DeleteSearchView)]
        public void DeleteSearchView(object sender, EventArgs e)
        {
            DeleteCurrentNode();
        }

        [CommandHandler(GuidanceExplorer.Commands.ViewGuidanceItem)]
        public void ViewItem(object sender, EventArgs e)
        {
            GuidanceItem[] guidanceItem = _listView.GetSelectedGuidanceItems();
            if (guidanceItem == null)
            {
                MessageBox.Show(Messages.SELECTED_ITEM_NOT_FOUND, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            GuidanceItemFileReference[] gifr = _modelService.Guidance.GetGuidanceItemFileReference(guidanceItem);
            GuidanceItemOpened(this, new GuidanceItemFileReferenceEventArgs(gifr, ReadProtection.ReadOnly));
        }

        [CommandHandler(GuidanceExplorer.Commands.EditGuidanceItem)]
        public void EditItem(object sender, EventArgs e)
        {
            GuidanceItem[] guidanceItem = _listView.GetSelectedGuidanceItems();
            if (guidanceItem == null)
            {
                MessageBox.Show(Messages.SELECTED_ITEM_NOT_FOUND, Messages.ERROR, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            GuidanceItemFileReference[] gifr = _modelService.Guidance.GetGuidanceItemFileReference(guidanceItem);
            GuidanceItemOpened(this, new GuidanceItemFileReferenceEventArgs(gifr, ReadProtection.ReadWrite));
        }

        private bool LibrariesSynchronizing()
        {
            if (_onlineStorage.IsSynchronizing)
            {
                MessageBox.Show(Messages.OPERATION_FAILED_DUE_TO_SYNC, "Guidance Explorer");
                return true;
            }
            else
            {
                return false;
            }
        }
    }
}
