//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================



using System;
using System.Collections.Generic;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI.WinForms;
using Microsoft.Practices.CompositeUI;

using GuidanceExplorer.WorkItems;
using GuidanceExplorer.Views;
using GuidanceExplorer.Model;

namespace GuidanceExplorerNT
{
    class Program : FormShellApplication<GuidanceExplorerWorkItem, MainForm>
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            GuidanceExplorerWorkItem.ClientFlavour = ClientFlavour.Offline;
            Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(Application_ThreadException);
            new Program().Run();
        }

        protected override void AfterShellCreated()
        {
            base.AfterShellCreated();
            RootWorkItem.SmartParts.Add(Shell.progressBar, "MainProgressBar");
            RootWorkItem.SmartParts.Add(Shell.toolStripStatusLabel, "StatusLabel");
            RootWorkItem.UIExtensionSites.RegisterSite("MainMenu", Shell.MainMenuStrip);
        }

        public override void OnUnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            Exception ex = e.ExceptionObject as Exception;
            if (ex != null)
            {
                MessageBox.Show(BuildExceptionString(ex));
            }
            else
            {
                MessageBox.Show("An Exception has occured, unable to get details");
            }
            Environment.Exit(0);
        }

        private static string BuildExceptionString(Exception exception)
        {
            string errMessage = string.Empty;

            errMessage += exception.Message + Environment.NewLine + exception.StackTrace;

            while (exception.InnerException != null)
            {
                errMessage += BuildInnerExceptionString(exception.InnerException);
                exception = exception.InnerException;
            }

            return errMessage;
        }

        private static string BuildInnerExceptionString(Exception innerException)
        {
            string errMessage = string.Empty;

            errMessage += Environment.NewLine + " InnerException ";
            errMessage += Environment.NewLine + innerException.Message + Environment.NewLine + innerException.StackTrace;

            return errMessage;
        }

        public static void Application_ThreadException(object sender, System.Threading.ThreadExceptionEventArgs e)
        {
            MessageBox.Show("An application execution error has occured: \n\n" + BuildExceptionString(e.Exception), "Guidance Explorer", MessageBoxButtons.OK);
        }
    }
}
