//===============================================================================
// Microsoft patterns & practices
// Guidance Explorer Offline Client
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;

using GuidanceExplorer.Model;
using System.Windows.Forms;

namespace GuidanceExplorer.Controls
{
    [GuidanceTypeMenuItemAttribute(10, "&File\\&New Item", GuidanceExplorer.Commands.NewGuidanceItem, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [GuidanceTypeMenuItemAttribute(20, "&File\\New Item &Template", GuidanceExplorer.Commands.NewGuidanceItem, typeof(ModelItemMenuItemIsEditableCondition), typeof(NewGuidanceTypeItemMenuItemProvider), Keys.None)]
    [GuidanceTypeMenuItemAttribute(30, "&File\\&Import Item...", GuidanceExplorer.Commands.ImportGuidanceItemToLibrary, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(40, "&File\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(50, "&File\\Save &Item as Xml...", "Save Items as Xml...", GuidanceExplorer.Commands.SaveItemAsXml, Keys.None)]
    [MenuItemAttribute(60, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(70, "&File\\&Save View as...", GuidanceExplorer.Commands.SaveLibraryAs, Keys.None)]
    [MenuItemAttribute(80, "&File\\Save Item &as...", "Save Items as...", GuidanceExplorer.Commands.SaveItemAs, Keys.None)]
    [MenuItemAttribute(120, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(122, "&File\\Subscribe to &Library\\From &File...", GuidanceExplorer.Commands.SubscribeLibraryFromFile, Keys.None)]
    [MenuItemAttribute(124, "&File\\Subscribe to &Library\\From &Web...", GuidanceExplorer.Commands.SubscribeLibraryFromWeb, Keys.None)]
    [MenuItemAttribute(125, "&File\\S&ynchronize Libraries", Commands.Synchronize, Keys.None)]
    [MenuItemAttribute(126, "&File\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(127, "&File\\E&xport as Library...", GuidanceExplorer.Commands.ExportViewAsLibrary, Keys.None)]
    [MenuItemAttribute(128, "&File\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(129, "&File\\Add/Remove &Guidance Type...", GuidanceExplorer.Commands.AddRemoveGuidanceType, Keys.None)]
    [MenuItemAttribute(130, "&File\\-", null, Keys.None)]
    [MenuItemAttribute(131, "&File\\&Exit", GuidanceExplorer.Commands.FileExit, Keys.None)]

    [MenuItemAttribute(200, "&Edit\\&Undo drag/drop", GuidanceExplorer.Commands.UndoDragAndDrop, Keys.Control | Keys.Z)]
    [MenuItemAttribute(210, "&Edit\\-", null, Keys.None)]
    [MenuItemAttribute(230, "&Edit\\&Add To View...", GuidanceExplorer.Commands.MoveGuidanceItemToView, Keys.None)]
    [MenuItemAttribute(260, "&Edit\\-", null, Keys.None)]
    [MenuItemAttribute(270, "&Edit\\&Select All", GuidanceExplorer.Commands.SelectAll, Keys.Control | Keys.A)]

    [MenuItemAttribute(300, "&View\\Reset &Sort Order", GuidanceExplorer.Commands.ResetSortOrder, Keys.None)]
    [MenuItemAttribute(310, "&View\\&Reset Columns", GuidanceExplorer.Commands.ResetColumns, Keys.None)]
    [MenuItemAttribute(320, "&View\\Reset &Filters", GuidanceExplorer.Commands.ResetFilters, Keys.None)]
    [MenuItemAttribute(330, "&View\\&Column Chooser", GuidanceExplorer.Commands.ColumnChooser, Keys.None)]

    [MenuItemAttribute(400, "&Help\\&About", GuidanceExplorer.Commands.HelpAbout, Keys.None)]

    [GuidanceTypeMenuItemAttribute(505, "context\\New Item Template", GuidanceExplorer.Commands.NewGuidanceItem, typeof(ModelItemMenuItemIsEditableCondition), typeof(NewGuidanceTypeItemMenuItemProvider), Keys.None)]
    [GuidanceTypeMenuItemAttribute(510, "context\\Import Item...", GuidanceExplorer.Commands.ImportGuidanceItemToLibrary, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(515, "context\\-", null, typeof(ModelItemMenuItemIsEditableCondition), Keys.None)]
    [MenuItemAttribute(520, "context\\Save View As...", GuidanceExplorer.Commands.SaveLibraryAs, Keys.None)]
    [MenuItemAttribute(525, "context\\E&xport as Library...", GuidanceExplorer.Commands.ExportViewAsLibrary, Keys.None)]
    [MenuItemAttribute(530, "context\\-", null, typeof(WebSubscribedLibraryCondition), Keys.None)]
    [MenuItemAttribute(535, "context\\Subscribe to RSS Feed", GuidanceExplorer.Commands.RSSSubscribe, typeof(WebSubscribedLibraryCondition), Keys.None)]
    [MenuItemAttribute(540, "context\\Synchronize Library", Commands.Synchronize, typeof(WebSubscribedLibraryCondition), Keys.None)]

    public class GuidanceTypeNode : ModelNode<GuidanceType>
    {
        /// <summary>
        /// Factory.
        /// </summary>
        /// <returns></returns>
        public static GuidanceTypeNode New()
        {
            return new GuidanceTypeNode();
        }
        
        /// <summary>
        /// The columns for this guidance type.
        /// </summary>
        private ColumnCollection _columns;

        /// <summary>
        /// Default constructor.
        /// </summary>
        private GuidanceTypeNode() : base("<a guidancetype>", 0, 0) { }

        /// <summary>
        /// The GuidanceType instance.
        /// </summary>
        public override GuidanceType ModelInstance
        {
            set
            {
                base.ModelInstance = value;
            }
        }

        /// <summary>
        /// The columns for this guidance type.
        /// </summary>
        public ColumnCollection Columns
        {
            get { return _columns; }
            set { _columns = value; }
        }

        /// <summary>
        /// Whether the node is editable.
        /// </summary>
        public override bool IsEditable
        {
            get { return false; } //Type nodes cannot be modified by user
        }

    }
}

