using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;
using System.IO;
using GuidanceExplorer;
using GuidanceExplorer.Services;
using GuidanceExplorer.Model;
using System.Collections.Generic;

public class SourceManagment
{
    public static Source GetViewItems(ModelStorageService modelStorageService, string parent, string folder, string viewId, string searchCriteria)
    {
        Criteria criteria = null;
        if (!string.IsNullOrEmpty(searchCriteria))
            criteria = new FullTextCriteria(modelStorageService.Guidance, searchCriteria);

        GuidanceItemFileReferenceCollection guidanceItems = GuidanceItemFileReferenceCollection.New();
        if (viewId == string.Empty)
        {
            Library library = modelStorageService.Guidance.Libraries[parent];
            foreach (GuidanceExplorer.Model.View view in library.Folders[folder].Views.Values)
                SourceManagment.AddItems(modelStorageService, view, guidanceItems, string.Empty, criteria);
        }
        else
        {
            Library library = modelStorageService.Guidance.Libraries[parent];
            GuidanceExplorer.Model.View view = library.Folders[folder].Views[viewId];
            SourceManagment.AddItems(modelStorageService, view, guidanceItems, string.Empty, criteria);
        }
        return SourceManagment.BuildSource(guidanceItems);
    }
            
    public static Source GetLibraryItems(ModelStorageService modelStorageService, string guidanceType, string parent, string searchCriteria)
    {
        Criteria criteria = null;
        if (!string.IsNullOrEmpty(searchCriteria))
            criteria = new FullTextCriteria(modelStorageService.Guidance, searchCriteria);
        
        GuidanceItemFileReferenceCollection guidanceItems = GuidanceItemFileReferenceCollection.New();
        if (parent == string.Empty)
        {
            foreach (GuidanceExplorer.Model.Library library in modelStorageService.Guidance.Libraries.Values)
                SourceManagment.AddItems(modelStorageService, library, guidanceItems, guidanceType, criteria);
        }
        else
        {
            Library library = modelStorageService.Guidance.Libraries[parent];
            SourceManagment.AddItems(modelStorageService, library, guidanceItems, guidanceType, criteria);
        }
        return SourceManagment.BuildSource(guidanceItems);
    }

    private static void AddItems(ModelStorageService modelStorageService, IGuidanceItemContainer itemContainer, GuidanceItemFileReferenceCollection guidanceItems, string guidanceType, Criteria criteria)
    {
        foreach (GuidanceItem item in itemContainer.Items)
        {
            if (criteria != null && !criteria.Evaluate(item))
                continue;

            GuidanceItemFileReference fileRef = item as GuidanceItemFileReference;

            if (fileRef != null)
            {
                if (!guidanceItems.ContainsKey(fileRef.Id))
                    if (guidanceType != string.Empty && fileRef.Type == guidanceType)
                        guidanceItems.Add(fileRef.Id, fileRef);
                    else
                        if (guidanceType == string.Empty)
                            guidanceItems.Add(fileRef.Id, fileRef);
            }
            else
            {
                //If the guidance item is not a file reference, then it points to a file reference in a library.
                fileRef = modelStorageService.Guidance.GetGuidanceItemFileReference(item);
                if (fileRef != null && !guidanceItems.ContainsKey(fileRef.Id))
                    guidanceItems.Add(fileRef.Id, fileRef);
            }
        }
    }
        
    private static Source BuildSource(GuidanceItemFileReferenceCollection guidanceItems)
    {
        Source source = new Source();
        foreach (GuidanceItemFileReference gifr in guidanceItems.Values)
        {
            Source.DataRow row = source.Data.NewDataRow();
            row.Id = gifr.Id;
            row.Date = gifr.Date;
            row.Title = gifr.Title;
            row.Source = gifr.Source;
            row.Type = gifr.Type;
            

            //Set CustomData
            foreach (string name in gifr.CustomData.Keys)
            {
                if ((row.Table.Columns.Contains(name)) && (row[name] != null))
                    row[name] = gifr.CustomData[name];
            }

            source.Data.AddDataRow(row);
        }
        return source;
    }
}
