using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.IO;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using GuidanceExplorer;
using GuidanceExplorer.Model;
using GuidanceExplorer.Services;
using Resources;

public class WebFormRenderService
{
    private string _getItemsUrl;
    private string _panelItemsClientId;
    private string _siteDirectory;
    private string _startupScript;
    private const string TemplateFolder = @"Images\";
    private string _siteUrl;

    public WebFormRenderService(string siteUrl, string getItemsUrl, string panelItemsClientId, string siteDirectory)
    {
        _getItemsUrl = getItemsUrl;
        _panelItemsClientId = panelItemsClientId;
        _siteDirectory = siteDirectory;

        _siteUrl = siteUrl;
    }

    public void RenderGuidanceToTreeView(Guidance guidance, TreeView treeView)
    {
        TreeNode myViewsNode = new TreeNode();
        TreeNode librariesNode = new TreeNode();

        librariesNode.Text = "Libraries";
        librariesNode.ImageUrl = Resource.LibrariesIcon;
        //librariesNode.NavigateUrl = string.Format("javascript:GetItemsList('{0}', '{1}', '{2}', '{3}', '{4}', '{5}');", string.Empty, _getItemsUrl, _panelItemsClientId, string.Empty, Constants.Library, string.Empty);
        librariesNode.NavigateUrl = string.Format("{0}?{4}&{1}&{2}&{4}&{3}&{4}", _siteUrl, _getItemsUrl, _panelItemsClientId, Constants.Library, string.Empty);

        
        _startupScript = string.Format("javascript:GetItemsList('{0}', '{1}', '{2}', '{3}', '{4}', '{5}');", string.Empty, _getItemsUrl, _panelItemsClientId, string.Empty, Constants.Library, string.Empty);


        //treeView.Nodes.Add(myViewsNode);
        treeView.Nodes.Add(librariesNode);

        //this.RenderUserViewsToMyViews(guidance.UserViews, myViewsNode);
        this.RenderLibraryCollectionToLibrariesNode(guidance, guidance.Libraries, librariesNode);

        librariesNode.Expand();
        myViewsNode.Expand();
    }


    private void RenderLibraryCollectionToLibrariesNode(Guidance guidance, LibraryCollection libraries, TreeNode librariesNode)
    {
        foreach (KeyValuePair<string, Library> library in libraries)
        {
            this.RenderLibraryToLibrariesNode(guidance, library.Value, librariesNode);
        }
    }

    public void RenderLibraryToLibrariesNode(Guidance guidance, Library library, TreeNode librariesNode)
    {
        TreeNode libraryNode = new TreeNode();
        //libraryNode.NavigateUrl = string.Format("javascript:GetItemsList('{0}', '{1}', '{2}', '{3}', '{4}', '{5}');", string.Empty, _getItemsUrl, _panelItemsClientId, library.Name, Constants.Library, string.Empty);

        libraryNode.NavigateUrl = string.Format("{0}?{5}&{1}&{2}&{3}&{4}&{5}", _siteUrl, _getItemsUrl, _panelItemsClientId, library.Name, Constants.Library, string.Empty);
        libraryNode.ImageUrl = Resource.LibIcon;

        //if (library.Name == Constants.PandPLibrary)


        librariesNode.ChildNodes.Add(libraryNode);
        this.RenderLibraryToLibraryNode(guidance, library, libraryNode);
    }

    private void RenderLibraryToLibraryNode(Guidance guidance, Library library, TreeNode libraryNode)
    {
        libraryNode.Text = library.Caption;


        SortedList<string, GuidanceType> sortedTypes = new SortedList<string, GuidanceType>();

        foreach (GuidanceType guidanceType in GuidanceType.AllGuidanceTypes)
        {
            if (sortedTypes.ContainsKey(guidanceType.DisplayName))
                continue;
            sortedTypes.Add(guidanceType.DisplayName, guidanceType);
        }

        foreach (GuidanceType guidanceType in sortedTypes.Values)
        {
            //Guidance type with display name as empty, means not visible in the UI.
            if (!String.IsNullOrEmpty(guidanceType.DisplayName))
            {
                this.RefreshGuidanceTypeNodes(libraryNode, guidanceType, library);
            }
        }



        //foreach (GuidanceType guidanceType in GuidanceType.AllGuidanceTypes)
        //{
        //    //Guidance type with display name as empty, means not visible in the UI.
        //    if (!String.IsNullOrEmpty(guidanceType.DisplayName))
        //    {
        //        this.RefreshGuidanceTypeNodes(libraryNode, guidanceType, library);
        //    }
        //}

        foreach (Folder folder in library.Folders.Values)
        {
            TreeNode folderNode = new TreeNode();
            libraryNode.ChildNodes.AddAt(0, folderNode);
            folderNode.Collapse();
            folderNode.ImageUrl = Resource.Folder;
            this.RenderFolderToFolderNode(folder, folderNode, library);
        }
    }

    public void RefreshGuidanceTypeNodes(TreeNode libraryNode, GuidanceType guidanceType, Library library)
    {
        if (library.GuidanceItemCountInLibrary(guidanceType) > 0)
        {
            TreeNode guidanceTypeNode = new TreeNode();
            guidanceTypeNode.Text = guidanceType.ListDisplayName;
            //guidanceTypeNode.NavigateUrl = string.Format("javascript:GetItemsList('{0}', '{1}', '{2}', '{3}', '{4}');", guidanceType.DisplayName, _getItemsUrl, _panelItemsClientId, library.Name, Constants.Library);

            guidanceTypeNode.NavigateUrl = string.Format("{0}?{1}&{2}&{3}&{4}&{5}&{6}", _siteUrl, HttpUtility.UrlEncode(guidanceType.DisplayName), _getItemsUrl, _panelItemsClientId, library.Name, Constants.Library, string.Empty);

            libraryNode.ChildNodes.Add(guidanceTypeNode);

            string fullImageFileName = Path.Combine(Path.Combine(_siteDirectory, WebFormRenderService.TemplateFolder), guidanceType.ImageFileName);
            if (File.Exists(fullImageFileName))
            {
                guidanceTypeNode.ImageUrl = Path.Combine(WebFormRenderService.TemplateFolder, guidanceType.ImageFileName);
            }
            else
                guidanceTypeNode.ImageUrl = Resource.DefaultIcon;
        }
    }

    private void RenderFolderToFolderNode(Folder folder, TreeNode folderNode, Library library)
    {
        folderNode.Text = folder.Caption;
        //folderNode.NavigateUrl = string.Format("javascript:GetItemsList('{0}', '{1}', '{2}', '{3}', '{4}', '{5}');", folder.Caption, _getItemsUrl, _panelItemsClientId, library.Name, Constants.View, string.Empty);
        folderNode.NavigateUrl = string.Format("{0}?{1}&{2}&{3}&{4}&{5}&{6}", _siteUrl, folder.Caption, _getItemsUrl, _panelItemsClientId, library.Name, Constants.View, string.Empty);
        
        
        //this.RenderFolderCollectionToFolderNode(folder.Folders as FolderCollection, folderNode);



        SortedDictionary<string, GuidanceExplorer.Model.View> sortedViews = new SortedDictionary<string, GuidanceExplorer.Model.View>();
        foreach (GuidanceExplorer.Model.View view in folder.Views.Values)
        {
            sortedViews.Add(view.Caption, view);
        }

        foreach (GuidanceExplorer.Model.View view in sortedViews.Values)
        {
            TreeNode viewNode = new TreeNode(view.Caption, view.Id);
            //viewNode.NavigateUrl = string.Format("javascript:GetItemsList('{0}', '{1}', '{2}', '{3}', '{4}', '{5}');", folder.Caption, _getItemsUrl, _panelItemsClientId, library.Name, Constants.View, view.Caption);
            viewNode.NavigateUrl = string.Format("{0}?{1}&{2}&{3}&{4}&{5}&{6}", _siteUrl, folder.Caption, _getItemsUrl, _panelItemsClientId, library.Name, Constants.View, view.Caption);
            folderNode.ChildNodes.Add(viewNode);
            viewNode.ImageUrl = Resource.ViewIcon;
        }

        //foreach (GuidanceExplorer.Model.View view in folder.Views.Values)
        //{
        //    TreeNode viewNode = new TreeNode(view.Caption, view.Id);
        //    viewNode.NavigateUrl = string.Format("javascript:GetItemsList('{0}', '{1}', '{2}', '{3}', '{4}', '{5}');", folder.Caption, _getItemsUrl, _panelItemsClientId, library.Name, Constants.View, view.Caption);
        //    folderNode.ChildNodes.Add(viewNode);
        //    viewNode.ImageUrl = Resource.ViewIcon;
        //}
    }

    private void RenderFolderCollectionToFolderNode(FolderCollection folders, TreeNode parentFolderNode)
    {
        foreach (Folder folder in folders.Values)
        {
            //TreeNode folderNode = new TreeNode();
            //parentFolderNode.ChildNodes.Add(folderNode);
            //this.RenderFolderToFolderNode(folder, folderNode);
        }
    }

    public string StartupScript
    {
        get
        {
            return _startupScript;
        }
    }
}