using System;
using System.IO;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;
using Resources;
using GuidanceExplorer;
using GuidanceExplorer.Model;
using GuidanceExplorer.Services;

public partial class ItemsList : System.Web.UI.Page
{
    private int _itemsCount = 0;
    private string _pager = string.Empty;
    private string _filterQuery = null;
    private string _guidanceType;
    private string _parent;
    private string _type;
    private string _viewId;
    private string _sortExpression;
    private string _sortDirection;
    private string _textToSearch;
    private string _clientId;
    private string _url;
    private int _pageIndex;
    private ModelStorageService _modelStorageService;

    private void Initialize()
    {
        _guidanceType = this.Request["guidanceType"].Replace("'", "");
        _parent = this.Request["Parent"];
        _sortExpression = this.Request["SortExpression"];
        _sortDirection = this.Request["SortDirection"];
        _type = this.Request["Type"].Replace("'", "");
        _clientId = this.Request["ClientId"];
        _textToSearch = this.Request.QueryString["TextToSearch"] == null || this.Request["TextToSearch"] == "''" ? null : this.Request.QueryString["TextToSearch"].Replace("'", "");
        _filterQuery = this.Request["FilterQuery"] == null ? string.Empty : this.Request["FilterQuery"];
        _pageIndex = this.Request.QueryString["PageIndex"] == null || this.Request.QueryString["PageIndex"] == string.Empty ? 0 : Convert.ToInt32(this.Request.QueryString["PageIndex"]);
        _viewId = this.Request["ViewId"].Replace("'", "");
        _url = this.Request.Url.ToString().Split('?')[0];
        this.PnPGrid.PageSize = string.IsNullOrEmpty(ConfigurationManager.AppSettings["PageSize"]) ? 100 : Convert.ToInt32(ConfigurationManager.AppSettings["PageSize"]);

    }

    protected void Page_Load(object sender, EventArgs e)
    {
        if (this.Request["GuidanceType"] == null)
            return;

        this.Initialize();
        this.PnPGrid.PageIndex = _pageIndex;
        this.PopulatePnPGrid();

        StringWriter tw = new StringWriter();
        HtmlTextWriter hw = new HtmlTextWriter(tw);
        string pager = this.GetPager();
        tw.Write(pager);
        this.PnPGrid.RenderControl(hw);

        Response.Write(tw.ToString().Replace("<div>", "<div style='z-index:0'>"));
        Response.End();
    }

    private void PopulatePnPGrid()
    {
        Source source = _type == Constants.Library ? SourceManagment.GetLibraryItems(this.ModelStorageService, _guidanceType, _parent, _textToSearch) : SourceManagment.GetViewItems(this.ModelStorageService, _parent, _guidanceType, _viewId, _textToSearch);

        if (_sortExpression != null && _sortDirection != null)
        {
            string sort = _sortDirection == SortDirection.Ascending.ToString() ? sort = "ASC" : sort = "DESC";
            source.Data.DefaultView.Sort = _sortExpression + " " + sort;
        }

        if (_filterQuery != null && _filterQuery != string.Empty)
        {
            string rowFilter = string.Empty;
            string[] filters = _filterQuery.Split(Constants.FiltersSeparator);
            foreach (string filter in filters)
            {
                string[] values = filter.Split(Constants.FilterValueSeparator);
                string filterExpression = values[0];
                string filterValue = HttpUtility.UrlDecode(values[1]);
                if (rowFilter != string.Empty)
                    rowFilter += " AND " + string.Format("{0} = '{1}'", filterExpression, filterValue);
                else
                    rowFilter += string.Format("{0} = '{1}'", filterExpression, filterValue);
            }
            source.Data.DefaultView.RowFilter = rowFilter;
        }

        this.PnPGrid.DataSource = source.Data;
        _itemsCount = source.Data.DefaultView.Count;
        this.PnPGrid.DataBind();
    }

    protected void PnPGrid_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.Header)
        {
            this.SetSortControlHeaders(e);
            this.SetFilterControlHeaders(e);
        }

        if (e.Row.RowType != DataControlRowType.Header && e.Row.RowType != DataControlRowType.Footer && e.Row.RowType != DataControlRowType.Pager)
        {
            string url = this.Request.Url.ToString().Split('?')[0];
            DataRowView rv = ((DataRowView)e.Row.DataItem);
            string path = url.Replace(Path.GetFileName(url), "ItemData.aspx") + "?ItemId=" + rv["Id"];

            e.Row.Cells[0].Text = string.Format("<a class='ARef' href='{0}'>&nbsp;{1}&nbsp;<a>", path, this.FormatDate(Convert.ToDateTime(rv["Date"])));
            e.Row.Cells[1].Text = string.Format("<a class='ARef' href='{0}'>&nbsp;{1}&nbsp;<a>", path, Utilities.Capitalize((string)rv["Technology"]));
            e.Row.Cells[2].Text = string.Format("<a class='ARef' href='{0}'>&nbsp;{1}&nbsp;<a>", path, Utilities.Capitalize((string)rv["Topic"]));
            e.Row.Cells[3].Text = string.Format("<a class='ARef' href='{0}'>&nbsp;{1}&nbsp;<a>", path, Utilities.Capitalize((string)rv["Type"]));
            e.Row.Cells[4].Text = string.Format("<a class='ARef' href='{0}'>&nbsp;{1}&nbsp;<a>", path, Utilities.Capitalize((string)rv["Category"]));
            e.Row.Cells[5].Text = string.Format("<a class='ARef' href='{0}'>&nbsp;{1}&nbsp;<a>", path, (string)rv["Priority"]);
            e.Row.Cells[6].Text = string.Format("<a class='ARef' href='{0}'>&nbsp;{1}&nbsp;<a>", path, Utilities.Capitalize((string)rv["Title"]));
            e.Row.Cells[7].Text = string.Format("<a class='ARef' href='{0}'>&nbsp;{1}&nbsp;<a>", path, Utilities.Capitalize((string)rv["Rule_Type"]));
            e.Row.Cells[8].Text = string.Format("<a class='ARef' href='{0}'>&nbsp;{1}&nbsp;<a>", path, Utilities.Capitalize((string)rv["Source"]));
            e.Row.Cells[9].Text = string.Format("<a class='ARef' href='{0}'>&nbsp;{1}&nbsp;<a>", path, Utilities.Capitalize((string)rv["Status"]));
            e.Row.Cells[10].Text = string.Format("<a class='ARef' href='{0}'>&nbsp;{1}&nbsp;<a>", path, Utilities.Capitalize((string)rv["Author"]));
        }
    }

    private string GetPager()
    {
        string pager = "<table cellpadding='0' cellspacing='0' border='0' width='100%'><tr><td width='300px'><div  style='font-size=x-small; font-family:Microsoft Sans Serif' ALIGN=Left class='Pager'>";

        if (this.PnPGrid.PageCount > 1)
        {
            pager += string.Format("<b>&nbsp;{0} Items in {1} pages</b></div></td><td><div style='font-size=x-small; font-family:Microsoft Sans Serif' ALIGN='Left' width='100%'>", _itemsCount, this.PnPGrid.PageCount);

            for (int i = 0; i < this.PnPGrid.PageCount; i++)
            {
                if (this.PnPGrid.PageIndex == i)
                    pager += string.Format("&nbsp;<b>{0}</b>", i + 1);
                else
                    pager += string.Format("&nbsp;<a style='color: blue' href=javascript:PageView('{0}','{2}');>{1}</a>", i, i + 1, HttpUtility.UrlEncode(_filterQuery));
            }
        }
        else
        {
            pager += string.Format("<b>{0} Items</b></div></td><td></td>", _itemsCount);
        }
        pager += "</div></td>";
        ModelStorageService model = this.Application["ModelStorageService"] as ModelStorageService;
        if (model.Guidance.Libraries.ContainsKey(_parent))
        {
            string baseUrl = ConfigurationManager.AppSettings["OnlineStorageBaseUrl"];
            Library library = model.Guidance.Libraries[_parent];
            if (string.IsNullOrEmpty(this._guidanceType))
            {
                //The entire library contents are required.
                pager += "<td align='right'><a title='RSS feed' href='" + baseUrl + "LibraryFeed.ashx?libraryId=" + library.Name + "'> <img border='0' src='images/feed-icon16x16.png'/> </a> </td>";
            }
            else
            {
                if (this._guidanceType != "Views")
                {
                    //The guidance type contents are required.
                    pager += "<td><a title='RSS feed' href='" + baseUrl + "LibraryFeed.ashx?libraryId=" + library.Name + "&guidanceType=" + _guidanceType + "'> <img border='0' src='images/feed-icon16x16.png'/> </a> </td>";
                }
                else if(!string.IsNullOrEmpty(_viewId))
                {
                    //The view contents are required.
                    pager += "<td><a title='RSS feed' href='" + baseUrl + "ViewFeed.ashx?viewId=" + library.AllViews[_viewId].Id + "'> <img border='0' src='images/feed-icon16x16.png'/> </a> </td>";
                }
            }
        }
        pager += "</tr></table>";
        pager += "<PAGER/>";
        return pager;
    }

    private string FormatDate(DateTime date)
    {
        return date.ToString("MM/dd/yyyy");
    }

    private void SetSortControlHeaders(GridViewRowEventArgs e)
    {
        LinkButton view;
        string newSortDirection = SortDirection.Ascending.ToString();
        foreach (DataControlField field in this.PnPGrid.Columns)
        {
            view = (LinkButton)e.Row.FindControl(field.HeaderText + "HeaderLink");
            if (view == null)
                continue;
            if (_sortExpression == field.HeaderText)
                newSortDirection = _sortDirection == SortDirection.Ascending.ToString() ? SortDirection.Descending.ToString() : SortDirection.Ascending.ToString();
            view.Attributes.Add("onclick", this.ClientGetSortedItemsScript(newSortDirection, field.HeaderText));

            view.ForeColor = Utilities.IsFilterSelected(_filterQuery, field.HeaderText) ? System.Drawing.Color.Blue : System.Drawing.Color.Black;
        }
    }

    private string ClientGetSortedItemsScript(string newSortDirection, string header)
    {
        return string.Format("javascript:return GetSortedItemsList('{0}', '{1}', '{2}', '{3}', '{4}', '{5}', '{6}', '{7}', '{8}', '{9}');", _guidanceType, _url, _clientId.Replace("'", ""), header, newSortDirection, _textToSearch, _filterQuery, _parent, _type, _viewId);
    }

    private void SetFilterControlHeaders(GridViewRowEventArgs e)
    {
        HtmlInputImage filter;
        foreach (DataControlField field in this.PnPGrid.Columns)
        {
            filter = (HtmlInputImage)e.Row.FindControl("filter" + field.HeaderText);
            if (filter == null)
                continue;
            filter.Attributes.Add("onclick", this.ClientGetFiltersByScript(field.HeaderText));
            filter.Src = Utilities.IsFilterSelected(_filterQuery, field.HeaderText) ? Resource.FilterSelected : Resource.Filter;

        }
    }

    private string ClientGetFiltersByScript(string header)
    {
        return string.Format("javascript:return GetFiltersBy('{0}', '{1}' ,'{2}', '{3}', '{4}', '{5}', '{6}');", _guidanceType, _parent, _url.Replace("ItemsList.aspx", "Filters.aspx"), header, _type, _viewId, _textToSearch);
    }

    /// <summary>
    /// Render required
    /// </summary>
    /// <param name="control"></param>
    public override void VerifyRenderingInServerForm(Control control)
    {
    }

    private ModelStorageService ModelStorageService
    {
        get
        {
            if (_modelStorageService != null)
                return _modelStorageService;

            if (this.Application["ModelStorageService"] == null)
            {
                string currentDirectory = Server.MapPath(this.Request.Url.LocalPath.Replace(Path.GetFileName(this.Request.Url.ToString().Split('?')[0]), ""));
                _modelStorageService = new ModelStorageService();
                _modelStorageService.LoadGuidance(currentDirectory + ConfigurationManager.AppSettings["GuidanceTypesDirectory"], ClientFlavour.Online);
                GuidanceExplorer.Services.AsyncLoadLibrariesService asyncLoadSLibraryService = new GuidanceExplorer.Services.AsyncLoadLibrariesService();
                asyncLoadSLibraryService.ProcessGuidance(_modelStorageService.Guidance, null);

                this.Application["ModelStorageService"] = _modelStorageService;
            }
            else
            {
                _modelStorageService = (ModelStorageService)this.Application["ModelStorageService"];
            }
            return _modelStorageService;
        }
    }
}
