using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;
using System.Data.SqlClient;
using System.Data.Common;
using System.Xml.Serialization;
using System.Text;

/// <summary>
/// Summary description for GuidanceItemHandler
/// </summary>
public class GuidanceItemHandler : IHttpHandler
{
    public const string GEOSNAMESPACE = "http://schemas.microsoft.com/pag/guidanceonlinestorage";

    public void ProcessRequest(HttpContext context)
    {
        string guidanceItemId = context.Request.QueryString["guidanceItemId"];
        if (string.IsNullOrEmpty(guidanceItemId))
        {
            guidanceItemId = Guid.Empty.ToString();
        }

        context.Response.ContentType = "text/xml";

        XmlTextWriter writer = new XmlTextWriter(context.Response.OutputStream, Encoding.UTF8);
        writer.Formatting = Formatting.Indented;

        using (SqlConnection conn = GetConnection())
        {
            XmlSerializerNamespaces xsn = new XmlSerializerNamespaces();
            xsn.Add("gs", GEOSNAMESPACE);
            XmlSerializer xs = new XmlSerializer(typeof(GuidanceItem), GEOSNAMESPACE);

            SqlCommand cmd = new SqlCommand("dbo.GetGuidanceItemFromViewById", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@guidanceItemId", SqlDbType.UniqueIdentifier).Value = new Guid(guidanceItemId);
            cmd.CommandType = CommandType.Text;

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                if (reader.Read())
                {
                    //Get the columns
                    ColumnDefinition[] columns = OnlineStorage.GetGuidanceTypeColumns(reader.GetGuid(reader.GetOrdinal("gui_guidance_type")));

                    //Read the guidance item.
                    GuidanceItem guidanceItem = OnlineStorage.ReadGuidanceItem(columns, reader);

                    //Serialize the guidance item.
                    xs.Serialize(writer, guidanceItem, xsn);
                }
            }
        }
        writer.Flush();
    }

    public bool IsReusable
    {
        get
        {
            return false;
        }
    }

    private SqlConnection GetConnection()
    {
        ConnectionStringSettings s = ConfigurationManager.ConnectionStrings["storage"];
        DbProviderFactory f = DbProviderFactories.GetFactory(s.ProviderName);
        DbConnection conn = f.CreateConnection();
        conn.ConnectionString = s.ConnectionString;
        conn.Open();
        return conn as SqlConnection;
    }
}