using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;
using System.Text;
using System.Text.RegularExpressions;
using System.Data.SqlClient;
using System.Data.Common;

using SimpleSharingExtensions;
using System.Xml.Serialization;

public class LibraryFeed : IHttpHandler
{
    public const string GEOSNAMESPACE = "http://schemas.microsoft.com/pag/guidanceonlinestorage";

    public void ProcessRequest(HttpContext context)
    {
        string libraryId = context.Request.QueryString["libraryId"];
        if (string.IsNullOrEmpty(libraryId))
        {
            libraryId = Guid.Empty.ToString();
        }
        bool views = false;
        if (!string.IsNullOrEmpty(context.Request.QueryString["views"]))
        {
            views = true;
        }
        string guidanceTypeName = string.Empty;
        if (!views)
        {
            if (!string.IsNullOrEmpty(context.Request.QueryString["guidanceType"]))
            {
                guidanceTypeName = context.Request.QueryString["guidanceType"];
            }
        }

        context.Response.ContentType = "text/xml";

        XmlTextWriter writer = new XmlTextWriter(context.Response.OutputStream, Encoding.UTF8);
        writer.Formatting = Formatting.Indented;

        writer.WriteStartDocument();

        //<rss version="2.0">
        writer.WriteStartElement("rss");
        writer.WriteAttributeString("version", "2.0");
        #region SSE
        #if SSE
        writer.WriteAttributeString("xmlns", "sx", null, Constants.SXNAMESPACE);
        #endif
        #endregion
        writer.WriteAttributeString("xmlns", "gs", null, GEOSNAMESPACE);

        //<channel>
        writer.WriteStartElement("channel");

        //<title>....</title>
        StringBuilder title = new StringBuilder("Guidance Explorer Library");
        if (!String.IsNullOrEmpty(guidanceTypeName))
        {
            title.Append(" - " + guidanceTypeName);
        }

        writer.WriteElementString("title", title.ToString());

        //<link>....</link>
        writer.WriteElementString("link", "http://www.guidancelibrary.com/");

        //<description>....</description>
        writer.WriteElementString("description", "The contents of the library in the online storage");

        //<language>en-us</language>
        writer.WriteElementString("language", "en-us");

        //<pubDate>Tue, 10 Jun 2003 04:00:00 GMT</pubDate>
        writer.WriteElementString("pubDate", new W3CDateTime(DateTime.Now).ToString());

        //<lastBuildDate>Tue, 10 Jun 2003 09:41:01 GMT</lastBuildDate>
        writer.WriteElementString("lastBuildDate", new W3CDateTime(DateTime.Now).ToString());

        /*
<docs>http://blogs.law.harvard.edu/tech/rss</docs>
<generator>Weblog Editor 2.0</generator>
<managingEditor>editor@example.com</managingEditor>
<webMaster>webmaster@example.com</webMaster>
 */

        #region SSE
        #if SSE
        //<sx:sharing since="Thu, 24 Aug 2006 15:57:18 GMT" until="Fri, 25 Aug 2006 15:47:16 GMT" window="30" version="0.92">
        writer.WriteStartElement("sx", "sharing", Constants.SXNAMESPACE);
        writer.WriteAttributeString("until", new W3CDateTime(DateTime.Now).ToString());
        writer.WriteAttributeString("since", new W3CDateTime(DateTime.Now).ToString());
        writer.WriteAttributeString("window", XmlConvert.ToString(0));
        writer.WriteAttributeString("version", "0.92");

        //</sx:sharing>
        writer.WriteEndElement();
        #endif
        #endregion

        using (SqlConnection conn = GetConnection())
        using (SqlConnection sseConnection = GetConnection())
        {
            XmlSerializerNamespaces xsn = new XmlSerializerNamespaces();
            xsn.Add("gs", GEOSNAMESPACE);
            XmlSerializer xs = new XmlSerializer(typeof(GuidanceItem), GEOSNAMESPACE);

            SqlCommand cmd;

            if (views)
            {
                cmd = new SqlCommand("dbo.GetViewsForLibraryFeed", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@lib_id", SqlDbType.UniqueIdentifier).Value = new Guid(libraryId);
                using (SqlDataReader reader = cmd.ExecuteReader())
                {
                    while (reader.Read())
                    {
                        //<item>
                        writer.WriteStartElement("item");

                        #region SSE
#if SSE
                    string sxSync = string.Empty;
                    SqlCommand sseCommand = sseConnection.CreateCommand();
                    sseCommand.CommandText = "SELECT SyncElement FROM SseSyncData WHERE ( SseItemId = @SseItemId )";
                    sseCommand.CommandType = CommandType.Text;
                    sseCommand.Parameters.Add("@SseItemId", SqlDbType.VarChar).Value = libraryId;
                    object sxSyncObject = sseCommand.ExecuteScalar();
                    if (sxSyncObject != null)
                    {
                        sxSync = (string)sxSyncObject;
                    }
                    else
                    {
                        //<sx:sync>
                        writer.WriteStartElement("sx", "sync", Constants.SXNAMESPACE);
                        writer.WriteAttributeString("id", "view:" + reader.GetGuid(reader.GetOrdinal("viw_id")).ToString());
                        writer.WriteAttributeString("version", "1");
                        writer.WriteAttributeString("deleted", XmlConvert.ToString(reader.GetBoolean(reader.GetOrdinal("viw_deleted"))));
                        writer.WriteAttributeString("noconflicts", XmlConvert.ToString(true));

                        //<sx:history>
                        writer.WriteStartElement("sx", "history", Constants.SXNAMESPACE);
                        writer.WriteAttributeString("when", new W3CDateTime(DateTime.Now).ToString());
                        if (reader.IsDBNull(reader.GetOrdinal("viw_creator")))
                        {
                            writer.WriteAttributeString("by", "admin");
                        }
                        else
                        {
                            writer.WriteAttributeString("by", reader.GetGuid(reader.GetOrdinal("viw_creator")).ToString());
                        }

                        //</sx:history>
                        writer.WriteEndElement();

                        //</sx:sync>
                        writer.WriteEndElement();
                    }
#endif
                        #endregion

                        //<title>...</title>
                        writer.WriteElementString("title", reader.GetString(reader.GetOrdinal("viw_caption")));

                        //<guid>...</guid>
                        writer.WriteElementString("guid", "view:" + reader.GetGuid(reader.GetOrdinal("viw_id")).ToString());

                        //<category>...</category>
                        //writer.WriteElementString("category", reader.GetString(reader.GetOrdinal("viw_folder_name")));

                        //<link>...</link>
                        writer.WriteElementString("link", "ViewFeed.ashx?viewId=" + reader.GetGuid(reader.GetOrdinal("viw_id")).ToString());

                        //</item>
                        writer.WriteEndElement();
                    }
                }
            }
            else if (!string.IsNullOrEmpty(guidanceTypeName))
            {
                //Get the items for the library
                cmd = new SqlCommand("dbo.GetGuidanceItemsByTypeForLibraryFeed", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@libraryId", SqlDbType.UniqueIdentifier).Value = new Guid(libraryId);
                cmd.Parameters.Add("@guidanceTypeName", SqlDbType.VarChar).Value = guidanceTypeName;
                

                using (SqlDataReader reader = cmd.ExecuteReader())
                {
                    while (reader.Read())
                    {
                        //<item>
                        writer.WriteStartElement("item");

                        #region SSE
#if SSE
                    //<sx:sync>
                    writer.WriteStartElement("sx", "sync", Constants.SXNAMESPACE);
                    writer.WriteAttributeString("id", "guidanceItem:" + reader.GetGuid(reader.GetOrdinal("gui_id")).ToString());
                    writer.WriteAttributeString("version", "1");
                    writer.WriteAttributeString("deleted", XmlConvert.ToString(reader.GetBoolean(reader.GetOrdinal("gui_deleted"))));
                    writer.WriteAttributeString("noconflicts", XmlConvert.ToString(true));

                    //<sx:history>
                    writer.WriteStartElement("sx", "history", Constants.SXNAMESPACE);
                    writer.WriteAttributeString("when", new W3CDateTime(reader.GetDateTime(reader.GetOrdinal("gui_last_update"))).ToString());
                    if (reader.IsDBNull(reader.GetOrdinal("gui_creator")))
                    {
                        writer.WriteAttributeString("by", "admin");
                    }
                    else
                    {
                        writer.WriteAttributeString("by", reader.GetGuid(reader.GetOrdinal("gui_creator")).ToString());
                    }

                    //</sx:history>
                    writer.WriteEndElement();

                    //</sx:sync>
                    writer.WriteEndElement();
#endif
                        #endregion

                        //<title>...</title>
                        writer.WriteElementString("title", reader.GetString(reader.GetOrdinal("gui_title")));

                        //<guid>...</guid>
                        //writer.WriteElementString("guid", "guidanceItem:" + reader.GetGuid(reader.GetOrdinal("gui_id")).ToString());

                        //<link>...</link>
                        //writer.WriteElementString("link", "GuidanceItemHandler.ashx?guidanceItemId=" + reader.GetGuid(reader.GetOrdinal("gui_id")).ToString());

                        //Get the columns
                        ColumnDefinition[] columns = OnlineStorage.GetGuidanceTypeColumns(reader.GetGuid(reader.GetOrdinal("gui_guidance_type")));

                        //Read the guidance item.
                        GuidanceItem guidanceItem = OnlineStorage.ReadGuidanceItem(columns, reader);

                        //<description>...</description>
                        StringBuilder itemContent = new StringBuilder(reader.GetString(reader.GetOrdinal("gui_content")));
                        itemContent.Append("<h1>Attributes</h1>\n");
                        foreach (XmlAttribute att in guidanceItem.AnyAttr)
                        {
                            itemContent.Append("<li>" + att.Name + ": " + att.Value + "</li><br>");
                        }
                        itemContent.Append("<li>" + "Type: " + reader.GetString(reader.GetOrdinal("gty_caption")) + "</li><br>");
                        writer.WriteElementString("description", itemContent.ToString());

                        //Serialize the guidance item.
                        xs.Serialize(writer, guidanceItem, xsn);

                        //</item>
                        writer.WriteEndElement();
                    }
                }
            }
            else
            {
                //Get the items for the library
                cmd = new SqlCommand("dbo.GetGuidanceItemsForLibraryFeed", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@libraryId", SqlDbType.UniqueIdentifier).Value = new Guid(libraryId);

                using (SqlDataReader reader = cmd.ExecuteReader())
                {
                    while (reader.Read())
                    {
                        //<item>
                        writer.WriteStartElement("item");

                        #region SSE
#if SSE
                    //<sx:sync>
                    writer.WriteStartElement("sx", "sync", Constants.SXNAMESPACE);
                    writer.WriteAttributeString("id", "guidanceItem:" + reader.GetGuid(reader.GetOrdinal("gui_id")).ToString());
                    writer.WriteAttributeString("version", "1");
                    writer.WriteAttributeString("deleted", XmlConvert.ToString(reader.GetBoolean(reader.GetOrdinal("gui_deleted"))));
                    writer.WriteAttributeString("noconflicts", XmlConvert.ToString(true));

                    //<sx:history>
                    writer.WriteStartElement("sx", "history", Constants.SXNAMESPACE);
                    writer.WriteAttributeString("when", new W3CDateTime(reader.GetDateTime(reader.GetOrdinal("gui_last_update"))).ToString());
                    if (reader.IsDBNull(reader.GetOrdinal("gui_creator")))
                    {
                        writer.WriteAttributeString("by", "admin");
                    }
                    else
                    {
                        writer.WriteAttributeString("by", reader.GetGuid(reader.GetOrdinal("gui_creator")).ToString());
                    }

                    //</sx:history>
                    writer.WriteEndElement();

                    //</sx:sync>
                    writer.WriteEndElement();
#endif
                        #endregion

                        //<title>...</title>
                        writer.WriteElementString("title", reader.GetString(reader.GetOrdinal("gui_title")));

                        //<guid>...</guid>
                        //writer.WriteElementString("guid", "guidanceItem:" + reader.GetGuid(reader.GetOrdinal("gui_id")).ToString());

                        //<link>...</link>
                        //writer.WriteElementString("link", "GuidanceItemHandler.ashx?guidanceItemId=" + reader.GetGuid(reader.GetOrdinal("gui_id")).ToString());

                        //Get the columns
                        ColumnDefinition[] columns = OnlineStorage.GetGuidanceTypeColumns(reader.GetGuid(reader.GetOrdinal("gui_guidance_type")));

                        //Read the guidance item.
                        GuidanceItem guidanceItem = OnlineStorage.ReadGuidanceItem(columns, reader);

                        //<description>...</description>
                        StringBuilder itemContent = new StringBuilder(reader.GetString(reader.GetOrdinal("gui_content")));
                        itemContent.Append("<h1>Attributes</h1>\n");
                        foreach (XmlAttribute att in guidanceItem.AnyAttr)
                        {
                            itemContent.Append("<li>" + att.Name + ": " + att.Value + "</li><br>");
                        }
                        itemContent.Append("<li>" + "Type: " + reader.GetString(reader.GetOrdinal("gty_caption")) + "</li><br>");
                        writer.WriteElementString("description", itemContent.ToString());

                        //Serialize the guidance item.
                        xs.Serialize(writer, guidanceItem, xsn);

                        //</item>
                        writer.WriteEndElement();
                    }
                }
            }
        }

        //</channel>
        writer.WriteEndElement();

        //</rss>
        writer.WriteEndElement();

        writer.WriteEndDocument();

        writer.Flush();
    }

    public bool IsReusable
    {
        get
        {
            return false;
        }
    }

    private SqlConnection GetConnection()
    {
        ConnectionStringSettings s = ConfigurationManager.ConnectionStrings["storage"];
        DbProviderFactory f = DbProviderFactories.GetFactory(s.ProviderName);
        DbConnection conn = f.CreateConnection();
        conn.ConnectionString = s.ConnectionString;
        conn.Open();
        return conn as SqlConnection;
    }


}