using System;
using System.Transactions;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Configuration;
using System.Data.Common;
using System.Data;
using System.Diagnostics;
using System.Xml;
using System.Text;
using System.Web.Security;
using System.Net;

[WebService(Namespace = "http://schemas.microsoft.com/pag/guidanceonlinestorage")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
[SoapDocumentService(RoutingStyle = SoapServiceRoutingStyle.RequestElement)]
public class OnlineStorage : System.Web.Services.WebService, IOnlineStorage
{
    private const int FIELD_COUNT = 11;

    private const string Library = "library";
    private const string LibraryId = Library + ":{0}";

    private const string View = "view";
    private const string ViewId = View + ":{0}";

    private const string GuidanceType = "guidanceType";
    private const string GuidanceTypeId = GuidanceType + ":{0}:{1}";

    public Authentication authentication;

    public OnlineStorage()
    {
    }

    [WebMethod]
    public string GetAllUserLogs()
    {
        XmlReader reader = null;
        StringBuilder outputString = new StringBuilder();
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.GetAllUserLogs", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            try
            {
                reader = cmd.ExecuteXmlReader();
                reader.Read();
                reader.ReadOuterXml();
                outputString.Append("<UserLogsArray>");
                while (reader.ReadState != ReadState.EndOfFile)
                {
                    outputString.Append(reader.ReadOuterXml());
                }
                outputString.Append("</UserLogsArray>");
            }
            catch (Exception ex)
            {
                string test = ex.ToString();
            }
        }
        return outputString.ToString();
    }

    [WebMethod]
    public void LogUserGUID(string GUID)
    {
        using (SqlConnection conn = GetConnection())
        {
            //check if the user guid exists first
            SqlCommand cmd = new SqlCommand("dbo.GetUserLogByID", conn);
            Guid loggedGUID = Guid.Empty;
            Guid userGUID = new Guid(GUID);
            DateTime firstLog = DateTime.Now;
            DateTime lastLog = DateTime.Now;
            int numLogs = 0;
            
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Parameters.Add("@UserGUID", SqlDbType.UniqueIdentifier).Value = userGUID;

            try
            {
                using (SqlDataReader reader = cmd.ExecuteReader())
                {
                    if (reader.Read())
                    {
                        if (!reader.IsDBNull(reader.GetOrdinal("UserGUID")))
                        {
                            loggedGUID = reader.GetGuid(reader.GetOrdinal("UserGUID"));
                        }

                        if (!reader.IsDBNull(reader.GetOrdinal("FirstLog")))
                        {
                            firstLog = reader.GetDateTime(reader.GetOrdinal("FirstLog"));
                        }

                        if (!reader.IsDBNull(reader.GetOrdinal("LastLog")))
                        {
                            lastLog = reader.GetDateTime(reader.GetOrdinal("LastLog"));
                        }

                        if (!reader.IsDBNull(reader.GetOrdinal("NumLogs")))
                        {
                            numLogs = reader.GetInt32(reader.GetOrdinal("NumLogs"));
                        }
                    }
                }

                if (loggedGUID == userGUID)
                {
                    // update an existing log
                    cmd = new SqlCommand("dbo.UpdateUserLog", conn);
                    lastLog = DateTime.Now;
                }
                else
                {
                    // create a new log
                    cmd = new SqlCommand("dbo.CreateUserLog", conn);
                }

                numLogs++;
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("@UserGUID", SqlDbType.UniqueIdentifier).Value = userGUID;
                cmd.Parameters.Add("@FirstLog", SqlDbType.DateTime).Value = firstLog;
                cmd.Parameters.Add("@LastLog", SqlDbType.DateTime).Value = lastLog;
                cmd.Parameters.Add("@NumLogs", SqlDbType.Int).Value = numLogs;

                cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
            }
        }
    }

    [WebMethod]
    public NavigationTree GetNavigationTree(int maxNesting)
    {
        NavigationTree navTree = new NavigationTree();

        //Populate all the libraries.
        PopulateLibraries(navTree);

        //Add the views for each library.
        PopulateViewsToLibraries(navTree);

        //Add the guidance type views to each library.
        PopulateGuidanceTypeViews(navTree);

        //Create the folders based on the view attributes
        CreateFoldersInLibraries(navTree);

        return navTree;
    }

    [WebMethod]
    public NavigationTree GetNavigationTreeChilds(string nodeId, int maxNesting)
    {
        //TODO:
        return new NavigationTree();
    }

    [WebMethod]
    public GuidanceType GetGuidanceTypeById(string guidanceTypeId)
    {
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.GetGuidanceTypeById", conn); 
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@gty_id", SqlDbType.UniqueIdentifier).Value = new Guid(guidanceTypeId);
            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                if (reader.Read())
                {
                    GuidanceType guidanceType = ConvertSqlDataReaderToGuidanceType(reader);
                    return guidanceType;
                }
            }
        }
        return null;
    }

    [WebMethod]
    public GuidanceType[] GetGuidanceTypes()
    {
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.GetGuidanceTypes", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            List<GuidanceType> gtypes = new List<GuidanceType>();

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                while (reader.Read())
                {
                    GuidanceType guidanceType = ConvertSqlDataReaderToGuidanceType(reader);
                    gtypes.Add(guidanceType);
                }
            }

            return gtypes.ToArray();
        }
    }

    [WebMethod]
    public GuidanceType GetGuidanceTypeByCaption(string guidanceTypeCaption)
    {
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.GetGuidanceTypeByCaption", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@gty_caption", SqlDbType.VarChar).Value = guidanceTypeCaption;
            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                if (reader.Read())
                {
                    GuidanceType guidanceType = ConvertSqlDataReaderToGuidanceType(reader);
                    return guidanceType;
                }
            }
        }
        return null;
    }

    [WebMethod]
    public GuidanceType GetGuidanceTypeByName(string guidanceTypeName)
    {
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.GetGuidanceTypeByName", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@gty_name", SqlDbType.VarChar).Value = guidanceTypeName;
            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                if (reader.Read())
                {
                    GuidanceType guidanceType = ConvertSqlDataReaderToGuidanceType(reader);
                    return guidanceType;
                }
            }
        }
        return null;
    }

    [WebMethod]
    public Schema GetSchemaById(string schemaId)
    {
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.GetSchemaById", conn);
            cmd.CommandType = CommandType.StoredProcedure;
            
            cmd.Parameters.Add("@sch_id", SqlDbType.UniqueIdentifier).Value = new Guid(schemaId);

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                if (reader.Read())
                {
                    Schema schema = new Schema();
                    schema.id = reader.GetGuid(reader.GetOrdinal("sch_id")).ToString();

                    List<Field> fields = new List<Field>();
                    for (int fieldIdx = 1; fieldIdx <= FIELD_COUNT; fieldIdx++)
                    {
                        string fieldName = string.Format("sch_field_name_{0}", fieldIdx);
                        if (!reader.IsDBNull(reader.GetOrdinal(fieldName)))
                        {
                            Field field = new Field();
                            field.id = fieldIdx.ToString();
                            field.name = reader.GetString(reader.GetOrdinal(fieldName));
                            field.type = FieldType.Text;
                            field.distinct = false;

                            fields.Add(field);
                        }
                    }
                    schema.Fields = fields.ToArray();

                    return schema;
                }
            }
        }
        return null;
    }

    [WebMethod]
    public String[] GetAllLibraryIds()
    {
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.GetAllLibraryIds", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            List<String> libraryIds = new List<String>();

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                while (reader.Read())
                {
                    String libId = reader.GetGuid(reader.GetOrdinal("lib_id")).ToString();
                    libraryIds.Add(libId);
                }
            }

            return libraryIds.ToArray();
        }
    }

    [WebMethod]
    public Library GetLibraryById(string libraryId)
    {
        //TODO: ensure id, is not library:id
        using (SqlConnection conn = GetConnection())
        {
            Library library = null;

            SqlCommand cmd = new SqlCommand("dbo.GetLibraryById", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@lib_id", SqlDbType.UniqueIdentifier).Value = new Guid(libraryId);
            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                if (reader.Read())
                {
                    library = new Library();
                    library.id = string.Format(LibraryId, reader.GetGuid(reader.GetOrdinal("lib_id")).ToString());
                    library.caption = reader.GetString(reader.GetOrdinal("lib_caption"));
                    library.readProtection = reader.GetBoolean(reader.GetOrdinal("lib_read_protection"));
                    library.delete = reader.GetBoolean(reader.GetOrdinal("lib_deleted"));
                    //TODO: author.
                }
                else
                {
                    return null;
                }
            }

            //Get the views for the library
            cmd = new SqlCommand("dbo.GetViewsByLibrary", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@viw_library", SqlDbType.UniqueIdentifier).Value = new Guid(libraryId);
            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                List<View> viewList = new List<View>();
                while (reader.Read())
                {
                    //Create the view and add it to the library.
                    View view = new View();
                    view.id = string.Format(ViewId, reader.GetGuid(reader.GetOrdinal("viw_id")).ToString());
                    view.caption = reader.GetString(reader.GetOrdinal("viw_caption"));
                    view.parentFolder = reader.GetString(reader.GetOrdinal("viw_folder_name"));
                    if (!reader.IsDBNull(reader.GetOrdinal("viw_creator")))
                    {
                        view.creator = reader.GetGuid(reader.GetOrdinal("viw_creator")).ToString();
                    }
                    if (!reader.IsDBNull(reader.GetOrdinal("viw_criteria")))
                    {
                        view.criteria = reader.GetString(reader.GetOrdinal("viw_criteria")).ToString();
                    }
                    if (!reader.IsDBNull(reader.GetOrdinal("viw_last_update")))
                    {
                        view.lastUpdate = reader.GetDateTime(reader.GetOrdinal("viw_last_update"));
                        view.lastUpdateSpecified = true;
                    }
                    view.library = reader.GetGuid(reader.GetOrdinal("viw_library")).ToString();
                    view.delete = reader.GetBoolean(reader.GetOrdinal("viw_deleted"));

                    viewList.Add(view);
                } //while

                //Set the list of views to the library.
                library.Views = viewList.ToArray();
            }

            return library;
        }
    }

    [WebMethod]
    public string[] GetGuidanceItemKeywords(string itemId)
    {
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.GetGuidanceItemKeywords", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@gui_id", SqlDbType.UniqueIdentifier).Value = new Guid(itemId);

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                List<string> keywords = new List<string>();
                while (reader.Read())
                {
                    keywords.Add(reader.GetString(reader.GetOrdinal("kwd_value")));
                }
                return keywords.ToArray();
            }
        }
    }

    [WebMethod]
    public View GetViewById(string viewId)
    {
        //TODO: ensure id, is not view:id
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.GetViewById", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@viw_id", SqlDbType.UniqueIdentifier).Value = new Guid(viewId);
            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                if (reader.Read())
                {
                    View view = new View();
                    view.id = string.Format(ViewId, reader.GetGuid(reader.GetOrdinal("viw_id")).ToString());
                    view.caption = reader.GetString(reader.GetOrdinal("viw_caption"));
                    view.parentFolder = reader.GetString(reader.GetOrdinal("viw_folder_name"));
                    if (!reader.IsDBNull(reader.GetOrdinal("viw_creator")))
                    {
                        view.creator = reader.GetGuid(reader.GetOrdinal("viw_creator")).ToString();
                    }
                    if (!reader.IsDBNull(reader.GetOrdinal("viw_criteria")))
                    {
                        view.criteria = reader.GetString(reader.GetOrdinal("viw_criteria")).ToString();
                    }
                    if (!reader.IsDBNull(reader.GetOrdinal("viw_last_update")))
                    {
                        view.lastUpdate = reader.GetDateTime(reader.GetOrdinal("viw_last_update"));
                    }
                    view.library = reader.GetGuid(reader.GetOrdinal("viw_library")).ToString();

                    return view;
                }
            }
        }
        return null;
    }

    [WebMethod]
    public GuidanceItem GetGuidanceItemById(string guidanceItemid)
    {
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.GetGuidanceItemById", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@guidanceItemId", SqlDbType.UniqueIdentifier).Value = new Guid(guidanceItemid);

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                if (reader.Read())
                {
                    //Get the columns for the type of the guidance item
                    ColumnDefinition[] columns = GetGuidanceTypeColumns(reader.GetGuid(reader.GetOrdinal("gui_guidance_type")));

                    //Get the guidance item.
                    GuidanceItem guidanceItem = ReadGuidanceItem(columns, reader);

                    return guidanceItem;
                }
            }
        }
        return null;
    }

    [WebMethod]
    public LibraryIndex GetLibraryIndex(string libraryId)
    {
        //Detect the class of id provided. Id's have the form of class:guid
        string[] splitId = libraryId.Split(':');

        string className = string.Empty;
        string classId = string.Empty;
        string subClassId = string.Empty;

        if (splitId.Length < 2)
        {
            classId = libraryId;
        }
        else
        {
            className = splitId[0];
            classId = splitId[1];
            subClassId = ((splitId.Length == 3) ? splitId[2] : String.Empty);
        }

        LibraryIndex libraryIndex = new LibraryIndex();
        libraryIndex.id = libraryId;
        
        try
        {
            using (SqlConnection conn = GetConnection())
            {
                SqlCommand cmd = new SqlCommand("dbo.GetLibraryIndex", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@libraryId", SqlDbType.UniqueIdentifier).Value = new Guid(classId);

                using (SqlDataReader reader = cmd.ExecuteReader())
                {
                    List<GuidanceItemIndex> guidanceItems = new List<GuidanceItemIndex>();
                    while (reader.Read())
                    {
                        GuidanceItemIndex guidanceItem = new GuidanceItemIndex();
                        guidanceItem.id = reader.GetGuid(reader.GetOrdinal("gui_id")).ToString();
                        guidanceItem.lastUpdate = reader.GetDateTime(reader.GetOrdinal("gui_last_update"));
                        guidanceItem.deleted = reader.GetBoolean(reader.GetOrdinal("gui_deleted"));

                        guidanceItems.Add(guidanceItem);
                    }

                    libraryIndex.GuidanceItems = guidanceItems.ToArray();
                }
            }
        }
        catch (Exception ex)
        {
            string message = ex.ToString();
        }

        return libraryIndex;
    }

    [WebMethod]
    public ListView GetListViewForNodeId(string nodeId, string criteria, ColumnDefinition[] columns, int from, int count)
    {
        //Detect the class of id provided. Id's have the form of class:guid
        string[] splitId = nodeId.Split(':');

        if (splitId.Length < 2)
        {
            throw new Exception("invalid id.");
        }

        string className = splitId[0];
        string classId = splitId[1];
        string subClassId = ((splitId.Length == 3) ? splitId[2] : String.Empty);

        ListView listView = new ListView();
        listView.Content = new Content();
        listView.id = nodeId;
        listView.Content.from = from;
        listView.Content.count = count;

        switch (className)
        {
            case Library:
                {
                    Guid libraryId = new Guid(classId);

                    if (columns == null)
                    {
                        //Get the default type id for the library
                        listView.ColumnDefinitions = GetLibraryColumns(libraryId);
                    }

                    //Populate the library content.
                    PopulateLibraryContent(listView, libraryId, criteria);

                    break;
                }
            case View:
                {
                    Guid viewId = new Guid(classId);

                    if (columns == null)
                    {
                        //Get the default type id for the view
                        listView.ColumnDefinitions = GetViewColumns(viewId);
                    }

                    //Populate the view content.
                    PopulateViewContent(listView, viewId, criteria);

                    break;
                }
            case GuidanceType:
                {
                    Guid libraryId = new Guid(classId);
                    Guid guidanceTypeId = new Guid(subClassId);

                    if (columns == null)
                    {
                        //Get the default type id for the view
                        listView.ColumnDefinitions = GetGuidanceTypeColumns(guidanceTypeId);
                    }

                    //Populate the view content.
                    PopulateGuidanceTypeContent(listView, libraryId, guidanceTypeId, criteria);

                    break;
                }
        }

        return listView;
    }

    [WebMethod]
    public string GetGuidanceItemContent(string guidanceItemId)
    {
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.GetGuidanceItemContent", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@guidanceItemId", SqlDbType.UniqueIdentifier).Value = new Guid(guidanceItemId);

            return cmd.ExecuteScalar().ToString();
        }
    }

    [WebMethod]
    [SoapHeader("authentication")]
    public void CreateLibrary(Library library)
    {
        ValidateCredentials();

        //TODO: ensure id, is not library:id
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.CreateLibrary", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@lib_id", SqlDbType.UniqueIdentifier).Value = new Guid(library.id);
            cmd.Parameters.Add("@lib_caption", SqlDbType.VarChar).Value = library.caption;
            cmd.Parameters.Add("@lib_creator", SqlDbType.UniqueIdentifier).Value = DBNull.Value; //TODO:
            cmd.Parameters.Add("@lib_read_protection", SqlDbType.Bit).Value = library.readProtection;
            cmd.Parameters.Add("@lib_user_views", SqlDbType.Bit).Value = false;

            try
            {
                cmd.ExecuteNonQuery();
            }
            catch (SqlException ex)
            {
                if (ex.Message.Contains("PK_Library"))
                {
                    string message = "The library already exists in the storage. The synchronization will handle the update of the library.";
                    SoapException exception = new SoapException(message, new XmlQualifiedName("StorageError"), string.Empty, new XmlDocument().CreateNode(XmlNodeType.Element, "Detail", ""));
                    exception.Detail.InnerText = message;
                    throw exception;
                }
                else if (ex.Message.Contains("IX_LibraryCaption"))
                {
                    string message = "The library name already exists in the storage, please change the name of the library and try again.";
                    SoapException exception = new SoapException(message, new XmlQualifiedName("StorageError"), string.Empty, new XmlDocument().CreateNode(XmlNodeType.Element, "Detail", ""));
                    exception.Detail.InnerText = message;
                    throw exception;
                }
                else
                {
                    throw;
                }
            }
        }
    }

    [WebMethod]
    [SoapHeader("authentication")]
    public void UpdateLibrary(Library library)
    {
        ValidateCredentials();

        //TODO: ensure id, is not library:id
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd;
            if (library.delete)
            {
                cmd = new SqlCommand("dbo.DeleteLibrary", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@lib_id", SqlDbType.UniqueIdentifier).Value = new Guid(library.id);
            }
            else
            {
                cmd = new SqlCommand("dbo.UpdateLibrary", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@lib_id", SqlDbType.UniqueIdentifier).Value = new Guid(library.id);
                cmd.Parameters.Add("@lib_caption", SqlDbType.VarChar).Value = library.caption;
                cmd.Parameters.Add("@lib_creator", SqlDbType.UniqueIdentifier).Value = DBNull.Value; //TODO:
                cmd.Parameters.Add("@lib_read_protection", SqlDbType.Bit).Value = library.readProtection;
                cmd.Parameters.Add("@lib_user_views", SqlDbType.Bit).Value = false;
            }

            cmd.ExecuteNonQuery();
        }
    }

    [WebMethod]
    [SoapHeader("authentication")]
    public void CreateView(View view)
    {
        ValidateCredentials();

        //TODO: ensure id, is not view:id

        //Set the creator id if the caption has been provided.
        SetViewCreatorId(view);

        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.CreateView", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@viw_id", SqlDbType.UniqueIdentifier).Value = new Guid(view.id);
            cmd.Parameters.Add("@viw_caption", SqlDbType.VarChar).Value = view.caption;
            cmd.Parameters.Add("@viw_library", SqlDbType.UniqueIdentifier).Value = new Guid(view.library);
            cmd.Parameters.Add("@viw_creator", SqlDbType.UniqueIdentifier).Value = new Guid(view.creator);
            cmd.Parameters.Add("@viw_folder_name", SqlDbType.VarChar).Value = view.parentFolder;
            cmd.Parameters.Add("@viw_criteria", SqlDbType.Text).Value = view.criteria;
            cmd.Parameters.Add("@viw_default_type", SqlDbType.UniqueIdentifier).Value = new Guid("f531f722-47d1-43a3-b0cf-62cf5dcf8c35");
            if (DateTime.MinValue == view.lastUpdate)
            {
                view.lastUpdate = DateTime.UtcNow;
            }
            cmd.Parameters.Add("@viw_last_update", SqlDbType.DateTime).Value = view.lastUpdate;
            cmd.Parameters.Add("@viw_deleted", SqlDbType.Bit).Value = false;

            cmd.ExecuteNonQuery();
        }
    }

    [WebMethod]
    [SoapHeader("authentication")]
    public GuidanceType CreateGuidanceType(GuidanceType guidanceType, string[] columns)
    {
        ValidateCredentials();

        Guid schemaID;
        Guid Id;

        try
        {
            schemaID = new Guid(guidanceType.schemaId);
            RemoveGuidanceTypeColumns(guidanceType.schemaId);
            CreateGuidanceTypeColumns(schemaID, columns);
        }
        catch
        {
            schemaID = Guid.NewGuid();
            CreateGuidanceTypeColumns(schemaID, columns);
        }

        try
        {
            Id = new Guid(guidanceType.id);
        }
        catch
        {
            Id = Guid.NewGuid();
        }
        
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.CreateGuidanceType", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@gty_id", SqlDbType.UniqueIdentifier).Value = Id;
            cmd.Parameters.Add("@gty_schema", SqlDbType.UniqueIdentifier).Value = schemaID;
            cmd.Parameters.Add("@gty_name", SqlDbType.VarChar).Value = guidanceType.name;
            cmd.Parameters.Add("@gty_caption", SqlDbType.VarChar).Value = guidanceType.caption;
            cmd.Parameters.Add("@gty_list_caption", SqlDbType.VarChar).Value = guidanceType.listCaption;
            cmd.Parameters.Add("@gty_new_caption", SqlDbType.VarChar).Value = guidanceType.newCaption;
            cmd.Parameters.Add("@gty_example_content", SqlDbType.Text).Value = guidanceType.exampleContent;
            cmd.Parameters.Add("@gty_template_schema_content", SqlDbType.Text).Value = guidanceType.templateSchemaContent;
            cmd.Parameters.Add("@gty_template_content", SqlDbType.Text).Value = guidanceType.templateContent;
            cmd.Parameters.Add("@gty_last_update", SqlDbType.DateTime).Value = guidanceType.lastUpdate;
            cmd.Parameters.Add("@gty_deleted", SqlDbType.Bit).Value = false;

            cmd.ExecuteNonQuery();
        }

        guidanceType.id = Id.ToString();
        guidanceType.schemaId = schemaID.ToString();
        return guidanceType;
    }

    [WebMethod]
    [SoapHeader("authentication")]
    public void RemoveGuidanceTypeColumns(string schemaId)
    {
        ValidateCredentials();

        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.RemoveGuidanceTypeColumns", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@sch_id", SqlDbType.UniqueIdentifier).Value = new Guid(schemaId);

            cmd.ExecuteNonQuery();
        }
    }

    [WebMethod]
    [SoapHeader("authentication")]
    public void UpdateGuidanceType(GuidanceType guidanceType, string[] columns)
    {
        ValidateCredentials();

        RemoveGuidanceTypeColumns(guidanceType.schemaId);
        CreateGuidanceTypeColumns(new Guid(guidanceType.schemaId), columns);
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd;

            try
            {
                if (guidanceType.delete)
                {
                    cmd = new SqlCommand("dbo.DeleteGuidanceType", conn);
                    cmd.CommandType = CommandType.StoredProcedure;

                    cmd.Parameters.Add("@gty_id", SqlDbType.UniqueIdentifier).Value = new Guid(guidanceType.id);
                    cmd.Parameters.Add("@gty_last_update", SqlDbType.DateTime).Value = guidanceType.lastUpdate;
                }
                else
                {
                    cmd = new SqlCommand("dbo.UpdateGuidanceType", conn);
                    cmd.CommandType = CommandType.StoredProcedure;

                    cmd.Parameters.Add("@gty_schema", SqlDbType.UniqueIdentifier).Value = new Guid(guidanceType.schemaId);
                    cmd.Parameters.Add("@gty_name", SqlDbType.VarChar).Value = guidanceType.name;
                    cmd.Parameters.Add("@gty_caption", SqlDbType.VarChar).Value = guidanceType.caption;
                    cmd.Parameters.Add("@gty_list_caption", SqlDbType.VarChar).Value = guidanceType.listCaption;
                    cmd.Parameters.Add("@gty_new_caption", SqlDbType.VarChar).Value = guidanceType.newCaption;
                    cmd.Parameters.Add("@gty_example_content", SqlDbType.Text).Value = guidanceType.exampleContent;
                    cmd.Parameters.Add("@gty_template_schema_content", SqlDbType.Text).Value = guidanceType.templateSchemaContent;
                    cmd.Parameters.Add("@gty_template_content", SqlDbType.Text).Value = guidanceType.templateContent;
                    cmd.Parameters.Add("@gty_last_update", SqlDbType.DateTime).Value = guidanceType.lastUpdate;
                    cmd.Parameters.Add("@gty_id", SqlDbType.UniqueIdentifier).Value = new Guid(guidanceType.id);
                }

                cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                string text = ex.ToString();
            }
        }
    }

    private void CreateGuidanceTypeColumns(Guid id, string[] columns)
    {
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.CreateGuidanceTypeColumns", conn);
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Parameters.Add("@sch_id", SqlDbType.UniqueIdentifier).Value = id;

            if (columns.Length > 0)
            {
                
                int index = 1;
                foreach (string col in columns)
                {
                    cmd.Parameters.Add("@sch_field_name_" + index.ToString(), SqlDbType.VarChar).Value = col;
                    index++;
                }
            }
            cmd.ExecuteNonQuery();
        }
    }

    [WebMethod]
    [SoapHeader("authentication")]
    public void UpdateView(View view)
    {
        ValidateCredentials();

        //TODO: ensure id, is not view:id

        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd;
            if (view.delete)
            {
                cmd = new SqlCommand("dbo.DeleteView", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@viw_id", SqlDbType.UniqueIdentifier).Value = new Guid(view.id);
                if (DateTime.MinValue == view.lastUpdate)
                {
                    view.lastUpdate = DateTime.UtcNow;
                }
                cmd.Parameters.Add("@viw_last_update", SqlDbType.DateTime).Value = view.lastUpdate;
            }
            else
            {
                //Set the creator id if the caption has been provided.
                SetViewCreatorId(view);

                cmd = new SqlCommand("dbo.UpdateView", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@viw_id", SqlDbType.UniqueIdentifier).Value = new Guid(view.id);
                cmd.Parameters.Add("@viw_caption", SqlDbType.VarChar).Value = view.caption;
                cmd.Parameters.Add("@viw_library", SqlDbType.UniqueIdentifier).Value = new Guid(view.library);
                cmd.Parameters.Add("@viw_creator", SqlDbType.UniqueIdentifier).Value = new Guid(view.creator);
                cmd.Parameters.Add("@viw_folder_name", SqlDbType.VarChar).Value = view.parentFolder;
                cmd.Parameters.Add("@viw_criteria", SqlDbType.Text).Value = view.criteria;
                cmd.Parameters.Add("@viw_default_type", SqlDbType.UniqueIdentifier).Value = new Guid("f531f722-47d1-43a3-b0cf-62cf5dcf8c35");
                if (DateTime.MinValue == view.lastUpdate)
                {
                    view.lastUpdate = DateTime.UtcNow;
                }
                cmd.Parameters.Add("@viw_last_update", SqlDbType.DateTime).Value = view.lastUpdate;
                cmd.Parameters.Add("@viw_deleted", SqlDbType.Bit).Value = false;
            }

            cmd.ExecuteNonQuery();
        }
    }


    [WebMethod]
    [SoapHeader("authentication")]
    public void AddGuidanceItemsToView(string viewId, GuidanceItem[] items)
    {
        ValidateCredentials();

        //TODO: ensure id, is not view:id
        using (SqlConnection conn = GetConnection())
        {
            StringBuilder message = new StringBuilder();
            foreach (GuidanceItem item in items)
            {
                SqlCommand cmd = new SqlCommand("dbo.AddGuidanceItemsToView", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@viw_view", SqlDbType.UniqueIdentifier).Value = new Guid(viewId);
                cmd.Parameters.Add("@gui_guidance_item", SqlDbType.UniqueIdentifier).Value = new Guid(item.id);

                try
                {
                    cmd.ExecuteNonQuery();
                }
                catch (SqlException ex)
                {
                    if (ex.Message.Contains("PK_ViewContent"))
                    {
                        string text = "The item {0} already exists in the view {1}. The synchronization will handle the update of the item.";
                        message.AppendFormat(text, item.id, viewId);
                    }
                    else
                    {
                        throw;
                    }
                }
            }

            if (message.Length != 0)
            {
                SoapException exception = new SoapException(message.ToString(), new XmlQualifiedName("StorageError"), string.Empty, new XmlDocument().CreateNode(XmlNodeType.Element, "Detail", ""));
                exception.Detail.InnerText = message.ToString();
                throw exception;
            }
        }
    }

    [WebMethod]
    [SoapHeader("authentication")]
    public void RemoveGuidanceItemsFromView(string viewId)
    {
        ValidateCredentials();

        //TODO: ensure id, is not view:id
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.RemoveGuidanceItemsFromView", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@viw_view", SqlDbType.UniqueIdentifier).Value = new Guid(viewId);

            cmd.ExecuteNonQuery();
        }
    }


    [WebMethod]
    [SoapHeader("authentication", Required=true)]
    public void CreateGuidanceItem(GuidanceItem item, string content)
    {
        ValidateCredentials();

        using (SqlConnection conn = GetConnection())
        {
            //Get the creator Id if the name has been provided
            SetGuidanceItemCreatorId(item);

            //Get the guidance type Id if the caption has been provided
            SetGuidanceTypeId(item);

            SqlCommand cmd = new SqlCommand("dbo.CreateGuidanceItem", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@gui_id", SqlDbType.UniqueIdentifier).Value = new Guid(item.id);
            cmd.Parameters.Add("@gui_title", SqlDbType.VarChar).Value = item.title;
            cmd.Parameters.Add("@gui_guidance_type", SqlDbType.UniqueIdentifier).Value = new Guid(item.guidanceType);
            cmd.Parameters.Add("@gui_library", SqlDbType.UniqueIdentifier).Value = new Guid(item.library);
            cmd.Parameters.Add("@gui_creator", SqlDbType.UniqueIdentifier).Value = new Guid(item.creator);
            cmd.Parameters.Add("@gui_content", SqlDbType.Text).Value = content;
            cmd.Parameters.Add("@gui_images", SqlDbType.Text).Value = item.images;
            cmd.Parameters.Add("@gui_last_update", SqlDbType.DateTime).Value = item.lastUpdate;

            int fieldIdx = 1;
            if (item.AnyAttr != null)
            {
                foreach (XmlAttribute attrib in item.AnyAttr)
                {
                    cmd.Parameters.Add(string.Format("@gui_field_value_{0}", fieldIdx), SqlDbType.VarChar).Value = attrib.Value;
                    fieldIdx++;
                }
            }
            for (; fieldIdx <= FIELD_COUNT; fieldIdx++)
            {
                cmd.Parameters.Add(string.Format("@gui_field_value_{0}", fieldIdx), SqlDbType.VarChar).Value = DBNull.Value;
            }

            try
            {
                cmd.ExecuteNonQuery();
            }
            catch (SqlException ex)
            {
                if (ex.Message.Contains("PK_GuidanceItem"))
                {
                    string message = "The item already exists in the storage. The synchronization will handle the update of the item.";
                    SoapException exception = new SoapException(message, new XmlQualifiedName("StorageError"), string.Empty, new XmlDocument().CreateNode(XmlNodeType.Element, "Detail", ""));
                    exception.Detail.InnerText = message;
                    throw exception;
                }
                else
                {
                    throw;
                }
            }
        }
    }

    [WebMethod]
    [SoapHeader("authentication")]
    public void UpdateGuidanceItem(GuidanceItem item, string content)
    {
        ValidateCredentials();

        if (item == null)
        {
            return;
        }

        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = conn.CreateCommand();

            //Prepare the command
            if (item.delete)
            {
                cmd = new SqlCommand("dbo.DeleteGuidanceItem", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@gui_id", SqlDbType.UniqueIdentifier).Value = new Guid(item.id);
                cmd.Parameters.Add("@gui_last_update", SqlDbType.DateTime).Value = item.lastUpdate;
            }
            else
            {
                //Get the creator Id if the name has been provided
                SetGuidanceItemCreatorId(item);

                //Get the guidance type Id if the caption has been provided
                SetGuidanceTypeId(item);

                cmd = new SqlCommand("dbo.UpdateGuidanceItem", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@gui_id", SqlDbType.UniqueIdentifier).Value = new Guid(item.id);
                cmd.Parameters.Add("@gui_title", SqlDbType.VarChar).Value = item.title;
                cmd.Parameters.Add("@gui_guidance_type", SqlDbType.UniqueIdentifier).Value = new Guid(item.guidanceType);
                cmd.Parameters.Add("@gui_library", SqlDbType.UniqueIdentifier).Value = new Guid(item.library);
                cmd.Parameters.Add("@gui_creator", SqlDbType.UniqueIdentifier).Value = new Guid(item.creator);
                cmd.Parameters.Add("@gui_content", SqlDbType.Text).Value = content;
                cmd.Parameters.Add("@gui_images", SqlDbType.Text).Value = item.images;
                cmd.Parameters.Add("@gui_last_update", SqlDbType.DateTime).Value = item.lastUpdate;

                int fieldIdx = 1;
                if (item.AnyAttr != null)
                {
                    foreach (XmlAttribute attrib in item.AnyAttr)
                    {
                        cmd.Parameters.Add(string.Format("@gui_field_value_{0}", fieldIdx), SqlDbType.VarChar).Value = attrib.Value;
                        fieldIdx++;
                    }
                }
                for (; fieldIdx <= FIELD_COUNT; fieldIdx++)
                {
                    cmd.Parameters.Add(string.Format("@gui_field_value_{0}", fieldIdx), SqlDbType.VarChar).Value = DBNull.Value;
                }
            }

            cmd.ExecuteNonQuery();
        }
    }

    [WebMethod]
    [SoapHeader("authentication")]
    public void SetGuidanceItemKeywords(string itemId, string[] keywords)
    {
        ValidateCredentials();

        using (SqlConnection conn = GetConnection())
        {
            using (SqlTransaction transaction = conn.BeginTransaction())
            {
                SqlCommand cmd = new SqlCommand("dbo.DeleteGuidanceItemKeyword", conn);
                cmd.CommandType = CommandType.StoredProcedure;

                cmd.Parameters.Add("@gui_id", SqlDbType.UniqueIdentifier).Value = new Guid(itemId);

                cmd.ExecuteNonQuery();

                //Prepare the command
                cmd = conn.CreateCommand();
                foreach (string keyword in keywords)
                {
                    cmd = new SqlCommand("dbo.CreateGuidanceItemKeyword", conn);
                    cmd.CommandType = CommandType.StoredProcedure;

                    cmd.Parameters.Add("@kwd_id", SqlDbType.UniqueIdentifier).Value = Guid.NewGuid();
                    cmd.Parameters.Add("@kwd_guidance_item", SqlDbType.UniqueIdentifier).Value = new Guid(itemId);
                    cmd.Parameters.Add("@kwd_value", SqlDbType.UniqueIdentifier).Value = keyword;
                    cmd.ExecuteNonQuery();
                }
                transaction.Commit();
            }
        }
    }

    private void ValidateCredentials()
    {
        //if (authentication == null)
        //{
        //    throw new Exception("Authorization exception, credentials not found.");
        //}

        //if (authentication.User != ConfigurationManager.AppSettings["UserName"] || authentication.Password != ConfigurationManager.AppSettings["Password"])
        //{
        //    throw new Exception("Authorization exception, invalid credentials.");
        //}
    }

    /// <summary>
    /// Create the folder structure based on the view information.
    /// </summary>
    /// <param name="navTree"></param>
    private void CreateFoldersInLibraries(NavigationTree navTree)
    {
        foreach (Library library in navTree.Libraries.Library)
        {
            if (library.Views == null)
            {
                continue;
            }
            foreach (View view in library.Views)
            {
                if (string.IsNullOrEmpty(view.parentFolder))
                {
                    continue;
                }

                Folder libFolder = null;
                string[] folders = view.parentFolder.Split('/');
                int folderIdx = 0;
                foreach (string folder in folders)
                {
                    if (libFolder == null)
                    {
                        if (library.Folders != null)
                        {
                            foreach (Folder tempFolder in library.Folders)
                            {
                                if (tempFolder.caption == folder)
                                {
                                    libFolder = tempFolder;
                                    break;
                                }
                            }
                        }

                        if (libFolder == null)
                        {
                            libFolder = new Folder();
                            libFolder.caption = folder;

                            List<Folder> allFolders = new List<Folder>();
                            if (library.Folders != null)
                            {
                                allFolders.AddRange(library.Folders);
                            }
                            allFolders.Add(libFolder);
                            library.Folders = allFolders.ToArray();
                        }
                    }
                    else
                    {
                        Folder newFolder = null;
                        if (libFolder.Folders != null)
                        {
                            foreach (Folder tempFolder in libFolder.Folders)
                            {
                                if (tempFolder.caption == folder)
                                {
                                    newFolder = tempFolder;
                                    break;
                                }
                            }
                        }

                        if (newFolder == null)
                        {
                            newFolder = new Folder();
                            newFolder.caption = folder;

                            List<Folder> allFolders = new List<Folder>();
                            if (libFolder.Folders != null)
                            {
                                allFolders.AddRange(libFolder.Folders);
                            }
                            allFolders.Add(newFolder);
                            libFolder.Folders = allFolders.ToArray();
                        }

                        libFolder = newFolder;
                    }//if (libFolder == null)

                    //If this is the leaf folder
                    if (folderIdx == folders.Length - 1)
                    {
                        List<View> allView = new List<View>();
                        if (libFolder.Views != null)
                        {
                            allView.AddRange(libFolder.Views);
                        }
                        allView.Add(view);
                        libFolder.Views = allView.ToArray();
                    }

                    //Increase the folderIdx
                    folderIdx++;
                }
            }
        }
    }

    private GuidanceType ConvertSqlDataReaderToGuidanceType(SqlDataReader reader)
    {
        GuidanceType guidanceType = new GuidanceType();
        
        if (!reader.IsDBNull(reader.GetOrdinal("gty_id")))
        {
            guidanceType.id = reader.GetGuid(reader.GetOrdinal("gty_id")).ToString();
        }

        if (!reader.IsDBNull(reader.GetOrdinal("gty_name")))
        {
            guidanceType.name = reader.GetString(reader.GetOrdinal("gty_name"));
        }

        if (!reader.IsDBNull(reader.GetOrdinal("gty_caption")))
        {
            guidanceType.caption = reader.GetString(reader.GetOrdinal("gty_caption"));
        }

        if (!reader.IsDBNull(reader.GetOrdinal("gty_list_caption")))
        {
            guidanceType.listCaption = reader.GetString(reader.GetOrdinal("gty_list_caption"));
        }

        if (!reader.IsDBNull(reader.GetOrdinal("gty_new_caption")))
        {
            guidanceType.newCaption = reader.GetString(reader.GetOrdinal("gty_new_caption"));
        }

        if (!reader.IsDBNull(reader.GetOrdinal("gty_example_content")))
        {
            guidanceType.exampleContent = reader.GetString(reader.GetOrdinal("gty_example_content"));
        }

        if (!reader.IsDBNull(reader.GetOrdinal("gty_template_content")))
        {
            guidanceType.templateContent = reader.GetString(reader.GetOrdinal("gty_template_content"));
        }

        if (!reader.IsDBNull(reader.GetOrdinal("gty_last_update")))
        {
            guidanceType.lastUpdate = reader.GetDateTime(reader.GetOrdinal("gty_last_update"));
            guidanceType.lastUpdateSpecified = true;
        }

        if (!reader.IsDBNull(reader.GetOrdinal("gty_template_schema_content")))
        {
            guidanceType.templateSchemaContent = reader.GetString(reader.GetOrdinal("gty_template_schema_content"));
        }

        if (!reader.IsDBNull(reader.GetOrdinal("gty_schema")))
        {
            guidanceType.schemaId = reader.GetGuid(reader.GetOrdinal("gty_schema")).ToString();
        }

        if (!reader.IsDBNull(reader.GetOrdinal("gty_deleted")))
        {
            guidanceType.delete = reader.GetBoolean(reader.GetOrdinal("gty_deleted"));
            guidanceType.deleteSpecified = true;
        }

        return guidanceType;
    }

    private void SetGuidanceTypeId(GuidanceItem item)
    {
        using (SqlConnection conn = GetConnection())
        {
            if (String.IsNullOrEmpty(item.guidanceType))
            {
                //The guidanceType ID has NOT been set.
                if (String.IsNullOrEmpty(item.guidanceTypeCaption))
                {
                    throw new ArgumentException("The guidance type ID or Caption has not been set.", "item");
                }

                //Get the creator Id
                SqlCommand getGuidanceTypeId = new SqlCommand("dbo.GetGuidanceTypeId", conn);
                getGuidanceTypeId.CommandType = CommandType.StoredProcedure;
                getGuidanceTypeId.Parameters.Add("@guidanceTypeName", SqlDbType.VarChar).Value = item.guidanceTypeCaption.ToLower();
                object guidanceTypeId = getGuidanceTypeId.ExecuteScalar();
                if (guidanceTypeId == null)
                {
                    throw new ArgumentException("The guidance type caption has not been set up.");
                }
                item.guidanceType = ((Guid)guidanceTypeId).ToString();
            }
            else
            {
                //The guidanceType ID has been set.
            }
        }
    }

    private void SetGuidanceItemCreatorId(GuidanceItem item)
    {
        using (SqlConnection conn = GetConnection())
        {
            if (String.IsNullOrEmpty(item.creator))
            {
                //The creator ID has NOT been set.
                if (String.IsNullOrEmpty(item.creatorCaption))
                {
                    throw new ArgumentException("The creator ID or Caption has not been set.", "item");
                }

                //Get the creator Id
                SqlCommand getUserId = new SqlCommand("dbo.GetGuidanceItemCreatorId", conn);
                getUserId.CommandType = CommandType.StoredProcedure;
                getUserId.Parameters.Add("@userName", SqlDbType.VarChar).Value = item.creatorCaption.ToLower();
                object userId = getUserId.ExecuteScalar();
                if (userId == null)
                {
                    throw new ArgumentException("The user name has not been found.");
                }
                item.creator = ((Guid)userId).ToString();
            }
            else
            {
                //The creator ID has been set.
            }
        }
    }

    private void SetViewCreatorId(View view)
    {
        using (SqlConnection conn = GetConnection())
        {
            if (String.IsNullOrEmpty(view.creator))
            {
                //The creator ID has NOT been set.
                if (String.IsNullOrEmpty(view.creatorCaption))
                {
                    throw new ArgumentException("The creator ID or Caption has not been set.", "item");
                }

                //Get the creator Id
                SqlCommand getUserId = new SqlCommand("dbo.GetViewCreatorId", conn);
                getUserId.CommandType = CommandType.StoredProcedure;
                getUserId.Parameters.Add("@userName", SqlDbType.VarChar).Value = view.creatorCaption.ToLower();
                object userId = getUserId.ExecuteScalar();
                if (userId == null)
                {
                    throw new ArgumentException("The user name has not been found.");
                }
                view.creator = ((Guid)userId).ToString();
            }
            else
            {
                //The creator ID has been set.
            }
        }
    }

    private void PopulateViewContent(ListView listView, Guid viewId, string criteria)
    {
        using (SqlConnection conn = GetConnection())
        {
            //not being used
            //StringBuilder orderBy = CreateOrderBy(listView);

            SqlCommand cmd = new SqlCommand("dbo.PopulateViewContent", conn);

            cmd.Parameters.Add("@searchCriteria", SqlDbType.NVarChar).Value = criteria;
            cmd.Parameters.Add("@viewId", SqlDbType.UniqueIdentifier).Value = viewId;
            cmd.CommandType = CommandType.StoredProcedure;

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                List<GuidanceItem> guidanceItems = new List<GuidanceItem>();
                while (reader.Read())
                {
                    GuidanceItem guidanceItem = ReadGuidanceItem(listView.ColumnDefinitions, reader);

                    guidanceItems.Add(guidanceItem);
                }

                listView.Content.GuidanceItem = guidanceItems.ToArray();
            }
        }
    }

    private ColumnDefinition[] GetViewColumns(Guid viewId)
    {
        using (SqlConnection conn = GetConnection())
        {
            List<ColumnDefinition> columns = new List<ColumnDefinition>();

            SqlCommand cmd = new SqlCommand("dbo.GetViewColumns", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@viewId", SqlDbType.UniqueIdentifier).Value = viewId;

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                while (reader.Read())
                {
                    int fieldNumber = reader.GetInt32(reader.GetOrdinal("vcl_extended_field_number"));

                    ColumnDefinition column = new ColumnDefinition();
                    column.caption = reader.GetString(reader.GetOrdinal("vcl_caption"));
                    column.width = reader.GetInt32(reader.GetOrdinal("vcl_width"));
                    column.autoFilterEnabled = false;
                    column.schemaFieldId = fieldNumber;
                    column.sortEnabled = false;

                    columns.Add(column);
                }
            }

            return columns.ToArray();
        }
    }

    private void PopulateLibraryContent(ListView listView, Guid libraryId, string criteria)
    {
        using (SqlConnection conn = GetConnection())
        {
            //not being used
            //StringBuilder orderBy = CreateOrderBy(listView);

            //Prepare the AutoFilter
            //TODO:

            SqlCommand cmd = new SqlCommand("dbo.PopulateLibraryContent", conn);

            cmd.Parameters.Add("@searchCriteria", SqlDbType.NVarChar).Value = criteria;
            cmd.Parameters.Add("@libraryId", SqlDbType.UniqueIdentifier).Value = libraryId;
            cmd.CommandType = CommandType.StoredProcedure;

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                List<GuidanceItem> guidanceItems = new List<GuidanceItem>();
                while (reader.Read())
                {
                    GuidanceItem guidanceItem = ReadGuidanceItem(listView.ColumnDefinitions, reader);

                    guidanceItems.Add(guidanceItem);
                }

                listView.Content.GuidanceItem = guidanceItems.ToArray();
            }
        }
    }

    private ColumnDefinition[] GetLibraryColumns(Guid libraryId)
    {
        using (SqlConnection conn = GetConnection())
        {
            List<ColumnDefinition> columns = new List<ColumnDefinition>();

            StringBuilder sb = new StringBuilder();
            for (int fieldIdx = 1; fieldIdx <= FIELD_COUNT; fieldIdx++)
            {
                if (fieldIdx == 1)
                {
                    sb.AppendFormat("sch_field_name_{0}", fieldIdx);
                }
                else
                {
                    sb.AppendFormat(", sch_field_name_{0}", fieldIdx);
                }
            }

            SqlCommand cmd = new SqlCommand("dbo.GetLibraryColumns", conn);
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.Parameters.Add("@libraryId", SqlDbType.UniqueIdentifier).Value = libraryId;
            cmd.Parameters.Add("@colNames", SqlDbType.NVarChar).Value = sb.ToString();

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                if (reader.Read())
                {
                    for (int fieldIdx = 1; fieldIdx <= FIELD_COUNT; fieldIdx++)
                    {
                        string fieldName = string.Format("sch_field_name_{0}", fieldIdx);

                        if (!reader.IsDBNull(reader.GetOrdinal(fieldName)))
                        {
                            ColumnDefinition column = new ColumnDefinition();
                            column.caption = reader.GetString(reader.GetOrdinal(fieldName));
                            column.width = 150; //TODO: default size
                            column.autoFilterEnabled = false;
                            column.schemaFieldId = fieldIdx;
                            column.sortEnabled = false;

                            columns.Add(column);
                        }
                    }
                }
            }

            return columns.ToArray();
        }
    }

    private void PopulateGuidanceTypeContent(ListView listView, Guid libraryId, Guid guidanceTypeId, string criteria)
    {
        using (SqlConnection conn = GetConnection())
        {
            //not being used
            //StringBuilder orderBy = CreateOrderBy(listView);

            //Prepare the AutoFilter
            //TODO:

            SqlCommand cmd = new SqlCommand("dbo.GuidanceTypeContent", conn);

            cmd.Parameters.Add("@libraryId", SqlDbType.UniqueIdentifier).Value = libraryId;
            cmd.Parameters.Add("@guidanceTypeId", SqlDbType.UniqueIdentifier).Value = guidanceTypeId;
            cmd.Parameters.Add("@searchCriteria", SqlDbType.NVarChar).Value = criteria;
            
            cmd.CommandType = CommandType.StoredProcedure; 
            
            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                List<GuidanceItem> guidanceItems = new List<GuidanceItem>();
                while (reader.Read())
                {
                    GuidanceItem guidanceItem = ReadGuidanceItem(listView.ColumnDefinitions, reader);

                    guidanceItems.Add(guidanceItem);
                }

                listView.Content.GuidanceItem = guidanceItems.ToArray();
            }
        }
    }

    /// <summary>
    /// Read the values from the current data reader to instance a new guidance item.
    /// </summary>
    /// <param name="columns"></param>
    /// <param name="reader"></param>
    /// <returns></returns>
    internal static GuidanceItem ReadGuidanceItem(ColumnDefinition[] columns, SqlDataReader reader)
    {
        GuidanceItem guidanceItem = new GuidanceItem();
        guidanceItem.id = reader.GetGuid(reader.GetOrdinal("gui_id")).ToString();
        guidanceItem.title = reader.GetString(reader.GetOrdinal("gui_title"));
        guidanceItem.library = reader.GetGuid(reader.GetOrdinal("gui_library")).ToString();
        if (!reader.IsDBNull(reader.GetOrdinal("gui_creator")))
        {
            guidanceItem.creator = reader.GetGuid(reader.GetOrdinal("gui_creator")).ToString();
        }
        guidanceItem.creatorCaption = reader.GetString(reader.GetOrdinal("usr_name"));
        guidanceItem.guidanceType = reader.GetGuid(reader.GetOrdinal("gui_guidance_type")).ToString();
        guidanceItem.guidanceTypeCaption = reader.GetString(reader.GetOrdinal("gty_name"));
        guidanceItem.lastUpdate = reader.GetDateTime(reader.GetOrdinal("gui_last_update"));
        guidanceItem.delete = reader.GetBoolean(reader.GetOrdinal("gui_deleted"));

        try
        {
            if (!reader.IsDBNull(reader.GetOrdinal("gui_content")))
            {
                guidanceItem.content = reader.GetString(reader.GetOrdinal("gui_content"));
            }

            if (!reader.IsDBNull(reader.GetOrdinal("gui_images")))
            {
                guidanceItem.images = reader.GetString(reader.GetOrdinal("gui_images"));
            }
        }
        catch
        {
        }

        List<XmlAttribute> attributes = new List<XmlAttribute>();
        foreach (ColumnDefinition column in columns)
        {
            XmlAttribute attribute = new XmlDocument().CreateAttribute(column.caption);
            if (column.schemaFieldId == 0)
            {
                attribute.Value = guidanceItem.title;
            }
            else
            {
                string fieldName = string.Format("gui_field_value_{0}", column.schemaFieldId);
                if (!reader.IsDBNull(reader.GetOrdinal(fieldName)))
                {
                    attribute.Value = reader.GetString(reader.GetOrdinal(fieldName));
                }
            }
            attributes.Add(attribute);
        }
        guidanceItem.AnyAttr = attributes.ToArray();

        return guidanceItem;
    }

    private StringBuilder CreateOrderBy(ListView listView)
    {
        StringBuilder orderBy = new StringBuilder();
        foreach (ColumnDefinition column in listView.ColumnDefinitions)
        {
            if (column.sortEnabled && column.sortOrder != SortOrder.None)
            {
                if (orderBy.Length == 0)
                {
                    orderBy.Append(string.Format("ORDER BY "));
                }
                else
                {
                    orderBy.Append(string.Format(", "));
                }

                string fieldName = string.Empty;
                if (column.schemaFieldId == 0)
                {
                    fieldName = "gui_title";
                }
                else
                {
                    fieldName = string.Format("gui_field_value_{0}", column.schemaFieldId);
                }

                if (column.sortOrder != SortOrder.Ascending)
                {
                    orderBy.Append(string.Format("{0} ASC", column.schemaFieldId));
                }
                else if (column.sortOrder != SortOrder.Descending)
                {
                    orderBy.Append(string.Format("{0} DESC", column.schemaFieldId));
                }
            }
        }
        return orderBy;
    }

    internal static ColumnDefinition[] GetGuidanceTypeColumns(Guid guidanceTypeId)
    {
        using (SqlConnection conn = GetConnection())
        {
            List<ColumnDefinition> columns = new List<ColumnDefinition>();

            SqlCommand cmd = new SqlCommand("dbo.GetGuidanceTypeColumns", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@guidanceTypeId", SqlDbType.UniqueIdentifier).Value = guidanceTypeId;

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                if (reader.Read())
                {
                    for (int fieldIdx = 1; fieldIdx <= FIELD_COUNT; fieldIdx++)
                    {
                        string fieldName = string.Format("sch_field_name_{0}", fieldIdx);

                        if (!reader.IsDBNull(reader.GetOrdinal(fieldName)))
                        {
                            ColumnDefinition column = new ColumnDefinition();
                            column.caption = reader.GetString(reader.GetOrdinal(fieldName));
                            column.width = 150; //TODO: default size
                            column.autoFilterEnabled = false;
                            column.schemaFieldId = fieldIdx;
                            column.sortEnabled = false;

                            columns.Add(column);
                        }
                    }
                }
            }

            return columns.ToArray();
        }
    }

    /// <summary>
    /// Load the libraries and user views to the navigation tree.
    /// </summary>
    /// <param name="navTree"></param>
    private void PopulateLibraries(NavigationTree navTree)
    {
        using (SqlConnection conn = GetConnection())
        {
            //Get all the libraries
            Libraries libraries = new Libraries();

            SqlCommand cmd = new SqlCommand("dbo.PopulateLibraries", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                List<Library> libList = new List<Library>();
                while (reader.Read())
                {
                    Library library = new Library();
                    library.id = string.Format(LibraryId, reader.GetGuid(reader.GetOrdinal("lib_id")).ToString());
                    library.caption = reader.GetString(reader.GetOrdinal("lib_caption"));
                    library.readProtection = reader.GetBoolean(reader.GetOrdinal("lib_read_protection"));
                    //TODO: author.

                    libList.Add(library);
                }
                libraries.Library = libList.ToArray();
                navTree.Libraries = libraries;
            }

            UserViews userViews = new UserViews();

            cmd = new SqlCommand("dbo.GetUserViews", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            cmd.Parameters.Add("@userName", SqlDbType.VarChar).Value = "DiegoG";

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                List<View> libList = new List<View>();
                while (reader.Read())
                {
                    userViews.id = string.Format("userViews:{0}", reader.GetGuid(reader.GetOrdinal("lib_id")).ToString());
                    userViews.caption = reader.GetString(reader.GetOrdinal("lib_caption"));

                    View view = new View();
                    view.id = string.Format(ViewId, reader.GetGuid(reader.GetOrdinal("viw_id")).ToString());
                    view.caption = reader.GetString(reader.GetOrdinal("viw_caption"));
                    view.parentFolder = reader.GetString(reader.GetOrdinal("viw_folder_name"));
                    if (!reader.IsDBNull(reader.GetOrdinal("viw_creator")))
                    {
                        view.creator = reader.GetGuid(reader.GetOrdinal("viw_creator")).ToString();
                    }
                    view.library = userViews.id;

                    libList.Add(view);
                }

                userViews.Views = libList.ToArray();
                navTree.UserViews = userViews;
            }
        }
    }

    /// <summary>
    /// Load the guidance type special views.
    /// </summary>
    /// <param name="navTree"></param>
    private void PopulateGuidanceTypeViews(NavigationTree navTree)
    {
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.PopulateGuidanceTypeViews", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                Guid libraryId = Guid.Empty;
                Library library = null;
                List<View> viewList = null;

                while (reader.Read())
                {
                    if (reader.GetGuid(reader.GetOrdinal("lib_id")) != libraryId)
                    {
                        //Keep the list of views.
                        if (library != null)
                        {
                            if (library.Views != null)
                            {
                                viewList.AddRange(library.Views);
                            }
                            library.Views = viewList.ToArray();
                        }

                        //The library has changed.
                        libraryId = reader.GetGuid(reader.GetOrdinal("lib_id"));
                        library = null;
                        viewList = new List<View>();

                        //Find the library
                        foreach (Library lib in navTree.Libraries.Library)
                        {
                            if (lib.id == string.Format(LibraryId, libraryId.ToString()))
                            {
                                library = lib;
                                break;
                            }
                        }

                        //The library must exist.
                        Debug.Assert(library != null);
                    }

                    //Create the view and add it to the library.
                    View view = new View();

                    view.id = string.Format(GuidanceTypeId, libraryId.ToString(), reader.GetGuid(reader.GetOrdinal("gty_id")).ToString());
                    view.caption = reader.GetString(reader.GetOrdinal("gty_caption"));
                    view.creator = null;
                    view.library = library.id;

                    viewList.Add(view);
                } //while

                //Keep the list of views.
                if (library != null)
                {
                    if (library.Views != null)
                    {
                        viewList.AddRange(library.Views);
                    }
                    library.Views = viewList.ToArray();
                }
            }
        }
    }

    /// <summary>
    /// Load the views for each library.
    /// </summary>
    /// <param name="navTree"></param>
    private void PopulateViewsToLibraries(NavigationTree navTree)
    {
        using (SqlConnection conn = GetConnection())
        {
            SqlCommand cmd = new SqlCommand("dbo.PopulateViewsToLibraries", conn);
            cmd.CommandType = CommandType.StoredProcedure;

            using (SqlDataReader reader = cmd.ExecuteReader())
            {
                Guid libraryId = Guid.Empty;
                Library library = null;
                List<View> viewList = null;

                while (reader.Read())
                {
                    if (reader.GetGuid(reader.GetOrdinal("viw_library")) != libraryId)
                    {
                        //Keep the list of views.
                        if (library != null)
                        {
                            library.Views = viewList.ToArray();
                        }

                        //The library has changed.
                        libraryId = reader.GetGuid(reader.GetOrdinal("viw_library"));
                        library = null;
                        viewList = new List<View>();

                        //Find the library
                        foreach (Library lib in navTree.Libraries.Library)
                        {
                            if (lib.id == string.Format(LibraryId, libraryId.ToString()))
                            {
                                library = lib;
                                break;
                            }
                        }
                    }

                    //If the library was not found, then skip. TODO: This is due to personal views
                    //we will have to skip them in the select statement.
                    if (library == null)
                    {
                        continue;
                    }

                    //Create the view and add it to the library.
                    View view = new View();
                    view.id = string.Format(ViewId, reader.GetGuid(reader.GetOrdinal("viw_id")).ToString());
                    view.caption = reader.GetString(reader.GetOrdinal("viw_caption"));
                    view.parentFolder = reader.GetString(reader.GetOrdinal("viw_folder_name"));
                    if (!reader.IsDBNull(reader.GetOrdinal("viw_creator")))
                    {
                        view.creator = reader.GetGuid(reader.GetOrdinal("viw_creator")).ToString();
                    }
                    view.library = reader.GetGuid(reader.GetOrdinal("viw_library")).ToString();
                    view.delete = reader.GetBoolean(reader.GetOrdinal("viw_deleted"));

                    viewList.Add(view);
                } //while

                //Set the list of views.
                if (library != null)
                {
                    library.Views = viewList.ToArray();
                }
            }
        }
    }

    private static SqlConnection GetConnection()
    {
        ConnectionStringSettings s = ConfigurationManager.ConnectionStrings["storage"];
        DbProviderFactory f = DbProviderFactories.GetFactory(s.ProviderName);
        DbConnection conn = f.CreateConnection();
        conn.ConnectionString = s.ConnectionString;
        conn.Open();
        return conn as SqlConnection;
    }

    public class Authentication : SoapHeader
    {
        public string User;
        public string Password;
    }

}
