using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Data.Common;
using System.Data.SqlClient;
using System.Collections.Generic;

using SimpleSharingExtensions;
using SimpleSharingExtensions.Xml;
using SimpleSharingExtensions.Data;

/// <summary>
/// Summary description for SseHandler
/// </summary>
public class SseHandler : IHttpHandler
{
    public void ProcessRequest(HttpContext context)
    {
        context.Response.ContentType = "text/plain";

        if ("POST" == context.Request.HttpMethod)
        {
            AuditProvider audit = new AuditProvider(Environment.UserName);
            GuidanceItemSerializer entitySerializer = new GuidanceItemSerializer();
            GuidanceItemMapper mapper = new GuidanceItemMapper();
            StreamSseFeedLoader streamLoader = new StreamSseFeedLoader(context.Request.InputStream, entitySerializer);

            GuidanceExplorerRepositoryAdapter dbRepository = new GuidanceExplorerRepositoryAdapter();

            SseRepositoryAdapter<GuidanceItemSerializer> sseRepository = new SseRepositoryAdapter<GuidanceItemSerializer>("storage", "feedName???");

            SseAwareRepository<GuidanceItem, string> repository = new SseAwareRepository<GuidanceItem, string>(
                dbRepository, sseRepository, audit, mapper);

            SsePubSubAdapter<string, GuidanceItem> ssePubSub = new SsePubSubAdapter<string, GuidanceItem>(
                dbRepository, sseRepository, mapper, entitySerializer);

            IConflictResolver resolver = new DoNothingConflictResolver();

            SseSubscriber subscriber = new SseSubscriber(streamLoader, ssePubSub, resolver);
            subscriber.IncorporateFeed(string.Empty);

            context.Response.Write("Done!");
        }
        else
        {
            context.Response.Write("Only POST method supported!");
        }
    }

    public bool IsReusable
    {
        get
        {
            return false;
        }
    }
}

public class GuidanceExplorerRepositoryAdapter : IDbRepository<GuidanceItem, string>
{
    private static SqlConnection GetConnection()
    {
        ConnectionStringSettings s = ConfigurationManager.ConnectionStrings["storage"];
        DbProviderFactory f = DbProviderFactories.GetFactory(s.ProviderName);
        DbConnection conn = f.CreateConnection();
        conn.ConnectionString = s.ConnectionString;
        conn.Open();
        return conn as SqlConnection;
    }

    #region IDbRepository<GuidanceItem,string> Members

    public void Add(GuidanceItem entity)
    {
        new OnlineStorage().CreateGuidanceItem(entity, entity.content);
    }

    public List<GuidanceItem> GetAll()
    {
        throw new Exception("The method or operation is not implemented.");
    }

    public GuidanceItem GetByKey(string key)
    {
        string[] splitString = key.Split(':');
        string className = splitString[0];
        string instanceId = splitString[1];

        switch (className)
        {
            case "guidanceItem":
                GuidanceItem guidanceItem = new OnlineStorage().GetGuidanceItemById(instanceId);
                return guidanceItem;
        }

        return null;
    }

    public void Remove(string key)
    {
        string[] splitString = key.Split(':');
        string className = splitString[0];
        string instanceId = splitString[1];

        GuidanceItem guidanceItem = new OnlineStorage().GetGuidanceItemById(instanceId);
        guidanceItem.delete = true;
        new OnlineStorage().UpdateGuidanceItem(guidanceItem, guidanceItem.content);
    }

    public void Save(GuidanceItem entity)
    {
        new OnlineStorage().UpdateGuidanceItem(entity, entity.content);
    }

    #endregion
}

public class GuidanceItemMapper : ISseEntityIdMapper<string, GuidanceItem>
{
    #region ISseEntityIdMapper<string,GuidanceItem> Members

    public string GetEntityKeyFromEntity(GuidanceItem entity)
    {
        return "guidanceItem:" + entity.id;
    }

    public string GetEntityKeyFromSseId(string sseId)
    {
        return sseId;
    }

    public string GetSseIdFromEntity(GuidanceItem entity)
    {
        return "guidanceItem:" + entity.id;
    }

    public string GetSseIdFromEntityKey(string entityKey)
    {
        return entityKey;
    }

    public void SetSseIdOnEntity(string sseId, GuidanceItem entity)
    {
        entity.id = sseId.Split(':')[1];
    }

    #endregion
}

public class StreamSseFeedLoader : IRssFeedLoader
{
    private Stream _stream;
    private ISseEntitySerializer _entitySerializer;

    public StreamSseFeedLoader(Stream stream, ISseEntitySerializer entitySerializer)
    {
        _stream = stream;
        _entitySerializer = entitySerializer;
    }

    #region IRssFeedLoader Members

    public RssFeed LoadRssFeedFromLink(string link)
    {
        SseXmlSerializer serializer = new SseXmlSerializer(_entitySerializer);
        using (XmlTextReader reader = new XmlTextReader(_stream))
        {
            return serializer.ReadRssFeed(reader);
        }
    }

    #endregion
}

public class GuidanceItemSerializer : EntitySerializer<GuidanceItem>
{
    XmlSerializerNamespaces _xsn = new XmlSerializerNamespaces();
    XmlSerializer _xs = new XmlSerializer(typeof(GuidanceItem), GEOSNAMESPACE);

    public const string GEOSNAMESPACE = "http://schemas.microsoft.com/pag/guidanceonlinestorage";

    public GuidanceItemSerializer()
        : base( "gs", GEOSNAMESPACE, "GuidanceItem", "Description", "GuidanceItem")
    {
        _xsn.Add("gs", GEOSNAMESPACE);
    }

    public override object ReadEntityFromXml(XmlReader reader)
    {
        //WTF?
        if (reader.Read())
        {
            if (reader.Name == "PLACEHOLDER")
            {
                reader.Read();
            }
        }

        GuidanceItem guidanceItem = _xs.Deserialize(reader) as GuidanceItem;
        return guidanceItem;
    }

    public override void WriteEntity(object entity, XmlWriter writer)
    {
        GuidanceItem guidanceItem = entity as GuidanceItem;
        if (guidanceItem != null)
        {
            _xs.Serialize(writer, guidanceItem, _xsn);
            return;
        }
    }
}
