using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Xml;
using System.Text;
using System.Collections.Generic;
using Microsoft.Office.Interop.Owc11;

public partial class _Default : System.Web.UI.Page 
{
    protected void Page_Load(object sender, EventArgs e)
    {
        List<UserLog> userLogsList = LoadUserLogsFromStore();

        //CreateTotalUsersChart(userLogsList);

        CreateTotalUsersPerDayChart(userLogsList, 90);

        CreateUniqueUsersPerDayChart(userLogsList, 90);
        
        //count the total number of unique users we've had up to each day
        //CreateCumulativeUsersPerDayChart(dateList, 90);
        
        //remove old files
        RemoveFiles(Server.MapPath("."));
    }

    private void CreateTotalUsersPerDayChart(List<UserLog> userLogsList, int daysToReport)
    {
        StringBuilder UUPDCols = new StringBuilder();
        StringBuilder UUPDData = new StringBuilder();

        //count how many users we had each day
        SortedList<DateTime, DateRecord> dateList = new SortedList<DateTime, DateRecord>();
        foreach (UserLog userLog in userLogsList)
        {
            if (dateList.Keys.Contains(userLog.FirstLog))
            {
                dateList[userLog.FirstLog].NumUsers++;
            }
            else
            {
                dateList.Add(userLog.FirstLog, new DateRecord(userLog.FirstLog, 1));
            }
        }
        int countStart;
        if (dateList.Count < daysToReport)
        {
            countStart = 0;
        }
        else
        {
            countStart = dateList.Count - daysToReport;
        }

        for (int i = countStart; i < dateList.Count; i++)
        {
            DateRecord dateRecord = dateList.Values[i];
            UUPDCols.Append(dateRecord.LastLog.Month + "/" + dateRecord.LastLog.Day + "\t");
            UUPDData.Append(dateRecord.NumUsers + "\t");
        }

        UUPDReportImage.Src = CreateChart(UUPDCols.ToString(), UUPDData.ToString());
    }

    private void CreateUniqueUsersPerDayChart(List<UserLog> userLogsList, int daysToReport)
    {
        StringBuilder UUPDCols = new StringBuilder();
        StringBuilder UUPDData = new StringBuilder();

        //count how many unique new users we had each day
        SortedList<DateTime, DateRecord> dateList = new SortedList<DateTime, DateRecord>();
        foreach (UserLog userLog in userLogsList)
        {
            if (dateList.Keys.Contains(userLog.FirstLog))
            {
                dateList[userLog.FirstLog].NumUsers++;
            }
            else
            {
                dateList.Add(userLog.FirstLog, new DateRecord(userLog.FirstLog, 1));
            }
        }

        int countStart;
        if (dateList.Count < daysToReport)
        {
            countStart = 0;
        }
        else
        {
            countStart = dateList.Count - daysToReport;
        }

        for (int i = countStart; i < dateList.Count; i++)
        {
            DateRecord dateRecord = dateList.Values[i];
            UUPDCols.Append(dateRecord.LastLog.Month + "/" + dateRecord.LastLog.Day + "\t");
            UUPDData.Append(dateRecord.NumUsers + "\t");
        }

        UUPDReportImage.Src = CreateChart(UUPDCols.ToString(), UUPDData.ToString());
    }

    private void CreateCumulativeUsersPerDayChart(SortedList<DateTime, DateRecord> dateList, int daysToReport)
    {

        StringBuilder CUCols = new StringBuilder();
        StringBuilder CUData = new StringBuilder();
        int cumulativeUsers = 0;

        int countStart;
        if (dateList.Count < daysToReport)
        {
            countStart = 0;
        }
        else
        {
            countStart = dateList.Count - daysToReport;
        }

        for (int i = countStart; i < dateList.Count; i++)
        {
            DateRecord dateRecord = dateList.Values[i];
            CUCols.Append(dateRecord.LastLog.Month + "/" + dateRecord.LastLog.Day + "\t");
            cumulativeUsers += dateRecord.NumUsers;
            CUData.Append(cumulativeUsers + "\t");
        }

        CUReportImage.Src = CreateChart(CUCols.ToString(), CUData.ToString());
    }

    private List<UserLog> LoadUserLogsFromStore()
    {
        OnlineStore.OnlineStorage onlineStore = new OnlineStore.OnlineStorage();

        string userLogs = onlineStore.GetAllUserLogs();
        XmlDocument xmlUserLogs = new XmlDocument();
        xmlUserLogs.LoadXml(userLogs);
        XmlNodeList childNodes = xmlUserLogs.ChildNodes;
        childNodes = childNodes.Item(0).ChildNodes;

        List<UserLog> userLogsList = new List<UserLog>(childNodes.Count);
        foreach (XmlNode node in childNodes)
        {
            Guid userGUID = new Guid(node.Attributes.GetNamedItem("UserGUID").InnerText);
            DateTime firstLog = DateTime.Parse(node.Attributes.GetNamedItem("FirstLog").InnerText);
            firstLog = firstLog.Date;
            DateTime lastLog = DateTime.Parse(node.Attributes.GetNamedItem("LastLog").InnerText);
            lastLog = lastLog.Date;
            int numLogs = int.Parse(node.Attributes.GetNamedItem("NumLogs").InnerText);
            UserLog userLog = new UserLog(userGUID, firstLog, lastLog, numLogs);

            userLogsList.Add(userLog);
        }
        return userLogsList;
    }

    private string CreateChart(string cols, string data)
    {
        Microsoft.Office.Interop.Owc11.ChartSpaceClass chartSpace = new Microsoft.Office.Interop.Owc11.ChartSpaceClass();

        chartSpace.Charts.Add(0);
        chartSpace.Charts[0].Type = ChartChartTypeEnum.chChartTypeLineMarkers;
        chartSpace.Charts[0].SeriesCollection.Add(0);
        chartSpace.Charts[0].SeriesCollection[0].SetData(Microsoft.Office.Interop.Owc11.ChartDimensionsEnum.chDimCategories, Convert.ToInt32(Microsoft.Office.Interop.Owc11.ChartSpecialDataSourcesEnum.chDataLiteral), cols);
        chartSpace.Charts[0].SeriesCollection[0].SetData(Microsoft.Office.Interop.Owc11.ChartDimensionsEnum.chDimValues, Convert.ToInt32(Microsoft.Office.Interop.Owc11.ChartSpecialDataSourcesEnum.chDataLiteral), data);

        string strFileName = "UUPD" + System.DateTime.Now.Ticks.ToString() + ".gif";
        string strFullPathAndName = Server.MapPath(strFileName);
        chartSpace.ExportPicture(strFullPathAndName, "gif", 800, 480);

        return strFileName;
    }

    private void RemoveFiles(string strPath)
    {
        System.IO.DirectoryInfo di = new DirectoryInfo(strPath);
        FileInfo[] fiArr = di.GetFiles();
        foreach (FileInfo fi in fiArr)
        {
            if (fi.Extension.ToString() == ".gif")
            {
                // if file is older than 2 minutes, we'll clean it up
                TimeSpan min = new TimeSpan(0, 0, 2, 0, 0);
                if (fi.CreationTime < DateTime.Now.Subtract(min))
                {
                    fi.Delete();
                }
            }
        }
    } 
}
